package org.rcisoft.business.bbanner.service.impl;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bbanner.dto.AddOrUpdateDTO;
import org.rcisoft.business.bbanner.dto.BannerInfoRspDTO;
import org.rcisoft.business.bbanner.dto.FindBannerPaginDTO;
import org.rcisoft.business.bbanner.dto.TopDownDTO;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultExceptionEnum;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.business.bbanner.dao.BBannerRepository;
import org.rcisoft.business.bbanner.entity.BBanner;
import org.rcisoft.business.bbanner.service.BBannerService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.InputStream;
import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;


/**
 * @author
 * @date 2019/9/10
 */
@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BBannerServiceImpl implements BBannerService {

    @Autowired
    private BBannerRepository bBannerRepository;

    @Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
    @Override
    public PersistModel persist(AddOrUpdateDTO addOrUpdateDTO) {

        BBanner model = new BBanner();
        BeanUtils.copyProperties(addOrUpdateDTO, model);

        int line = 0;
        //外部链接但外部URL为空或长度为0 抛异常
        if ("1".equals(model.getIsExternalLink()) && StringUtils.isEmpty(model.getExternalUrl()))
            throw new ServiceException(ResultServiceEnums.JUMP_ADDRESS_NULL);
        else if ("1".equals(model.getIsExternalLink()) && StringUtils.isNotBlank(model.getExternalUrl())){
//            String regex = "^([hH][tT]{2}[pP]:/*|[hH][tT]{2}[pP][sS]:/*|[fF][tT][pP]:/*)(([A-Za-z0-9-~]+).)+([A-Za-z0-9-~\\/])+(\\?{0,1}(([A-Za-z0-9-~]+\\={0,1})([A-Za-z0-9-~]*)\\&{0,1})*)$"; // +(\?{0,1}(([A-Za-z0-9-~]+\={0,1})([A-Za-z0-9-~]*)\&{0,1})*)$
//            Pattern pattern = Pattern.compile(regex);
//
//            if (!pattern.matcher(model.getExternalUrl()).matches()) {
//                throw new ServiceException(ResultServiceEnums.ILLEGAL_URL);
//            }
            try {
                URL url = new URL(model.getExternalUrl());
                InputStream in = ((URL) url).openStream();
            } catch (Exception e1) {
                throw new ServiceException(ResultServiceEnums.ILLEGAL_URL);
            }
        }
        //内部链接但类型或课程ID为空或长度为0 抛异常
        if ("0".equals(model.getIsExternalLink()) && (StringUtils.isEmpty(model.getLineType()) || StringUtils.isEmpty(model.getCourseId())))
            throw new ServiceException(ResultServiceEnums.JUMP_ADDRESS_NULL);
        if (model.getBusinessId() != null) {
            //按ID查询Banner个数
            if (bBannerRepository.selectCountById(model.getBusinessId()) == 0){
                throw new ServiceException(ResultServiceEnums.NO_DATA);
            }
            UserUtil.setCurrentMergeOperation(model);
            line = bBannerRepository.updateById(model);
        } else {
            UserUtil.setCurrentPersistOperation(model);
            //查询Banner是否大于等于5个，大于等于五个不允许添加
            int countBanner =  bBannerRepository.selectCountBanner(addOrUpdateDTO);
            if(countBanner >= 5){
                throw new ServiceException(ResultServiceEnums.NOT_MORE_COUNT);
            }
            //查询最大排序值+1后放入Banner中
            int sort = bBannerRepository.selectMaxSort();
            model.setBannerSort(sort);
            line = bBannerRepository.insertSelective(model);
        }
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel removeBanner(String id) {
        if (bBannerRepository.selectCountById(id) == 0){
            throw new ServiceException(ResultServiceEnums.NO_DATA);
        }

        return new PersistModel(1, bBannerRepository.deleteBanner(id));
    }

    @Override
    public BBanner selectOne(String id) {
        BBanner banner = bBannerRepository.queryOne(id);
        return banner;
    }

    @Override
    public List<BBanner> queryBannerByPagination(PageUtil pageUtil, FindBannerPaginDTO model) {
        return bBannerRepository.queryBanners(model);
    }


    /**
     * @param bannerOrderMap
     * @return
     * @author: jiangpengpeng
     * @description: 更新banner顺序
     * @date: 2019/9/26 16:27
     */
    @Override
    public int updateBannerOrder(String bannerOrderMap) {
        //获取将字符串中的主键和位置对应关系转成Map
        Map parse = (Map) JSON.parse(bannerOrderMap);
        int i = bBannerRepository.updateBannerOrderByid(parse);
        return i;
    }

}
