package org.rcisoft.business.bchapter.controller;

import cn.hutool.core.bean.BeanUtil;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bchapter.dto.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.IsCompleteEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bfile.dto.uploadDTO;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.enums.RoleFlagEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//import org.rcisoft.business.bsl.service.BSlService;

/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@Api(tags = "02 章节")
@RestController
@RequestMapping("/BChapter")
@Slf4j
public class BChapterController extends PaginationController<BChapter> {
    @Autowired
    private BChapterService bChapterService;

    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Autowired
    private Global global;

    @ApiOperation(value = "201 添加/编辑章节", notes = "businessId为空时是添加方法，不为空时是修改方法")
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid ChapterDTO chapterDTO, BindingResult bindingResult, uploadDTO dto) {
        PersistModel data = bChapterService.persist(chapterDTO,curUser.getCorpId(),dto);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                chapterDTO);
    }


    @ApiOperation(value = "202 删除章节", notes = "根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "章节Id", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(CurUser curUser,String id) {
        PersistModel data = bChapterService.removeBChapter(id,curUser.getUserId());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value = "203 分页查询章节", notes = "根据条件分页查询章节", response = QueryChapterListResDTO.class)
    @PostMapping(value = "/queryBChaptersByPagination")
    public Result queryBChaptersByPagination(CurUser curUser,@Valid QueryChapterListDTO param,BindingResult br) {
        bChapterService.queryBChaptersByPagination(getPaginationUtility(), param,curUser.getCorpId());
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value = "204 查询章节", notes = "根据lessonId查询章节", response = QueryChapterListResDTO.class)
    @GetMapping(value = "/queryBChaptersWithoutAuth")
    public Result queryBChapters(CurUser curUser,@Valid QueryChapterListDTO param,BindingResult br) {
        Map<String, Object> map = new HashedMap();
        map.put("chapterList", bChapterService.queryBChapters(param,curUser.getCorpId()));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }
    @ApiOperation(value = "205 查询章节带进度", notes = "根据lessonId查询章节带进度", response = QueryChapterListResDTO.class)
    @GetMapping(value = "/queryBChaptersWithProgress")
    public Result queryBChaptersWithProgress(CurUser curUser, @RequestParam String lessonId, BindingResult br) {
        Map<String, Object> map = new HashedMap();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapterService.queryBChaptersWithProgress(lessonId,curUser));
    }


    @ApiOperation(value = "206 将ppt转为pdf", notes = "流 输出ppt转为pdf")
    @GetMapping("/getFile")
    public Result getFile(CurUser curUser, @Valid String chapterId, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.changeFileToPdf(chapterId);
        //读取路径下面的文件
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }


    @ApiOperation(value = "207 预览文件", notes = "(预览时传)")
    @GetMapping("/getFileAtView")
    public Result getFileAtView(CurUser curUser, @Valid BFile file, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.changeFileToPdfAtView(file);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }

    @ApiOperation(value = "208 下载文件 返回服务器地址", notes = "下载文件 返回服务器地址")
    @GetMapping("/getDownLoadUrl")
    public Result getDownLoadUrl(CurUser curUser, @Valid BFile file, BindingResult br) throws IOException {
        String newFileUrl = bChapterService.getDownLoadUrl(file);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                newFileUrl);
    }
    @RequestMapping(value = "/downloadFile", method = RequestMethod.GET)
    public ResponseEntity<InputStreamResource> downloadFile(BFile bfile)
            throws IOException {
        log.debug("----------df1----------");
        String filePath = bChapterService.getDownLoadUrl(bfile);
        log.debug("----------df2----------");
        FileSystemResource file = new FileSystemResource(filePath);
        log.debug("----------df3----------");
        HttpHeaders headers = new HttpHeaders();
        log.debug("----------df4----------");
        headers.add("Cache-Control", "no-cache, no-store, must-revalidate");
        headers.add("Content-Disposition", String.format("attachment; filename=\"%s\"", file.getFilename()));
        headers.add("Pragma", "no-cache");
        headers.add("Expires", "0");
        log.debug("----------df5----------");
        return ResponseEntity
                .ok()
                .headers(headers)
                .contentLength(file.contentLength())
                .contentType(MediaType.parseMediaType("application/octet-stream"))
                .body(new InputStreamResource(file.getInputStream()));
    }

    //文件下载相关代码
    @RequestMapping("/download")
    public void downloadFile(BFile bfile, HttpServletResponse response) {
        String filePath = bChapterService.getDownLoadUrl(bfile);
        File file = new File(filePath);
        if (StringUtils.isNotEmpty(file.getName())) {
            //设置文件路径
            if (file.exists()) {
                log.debug("------------------test1------------------");
  //              response.setContentType("application/force-download");
//                response.setContentType("multipart/form-data");

                response.setContentType("application/octet-stream;charset=utf-8");
//                response.setHeader("content-type", "application/octet-stream");
                String asd = file.getName();
                //response.setHeader("Content-Disposition", "attachment;fileName=" + file.getName());// 设置文件名
                response.setHeader("Content-Disposition", "attachment; filename=\"" + file.getName() + "\"; filename*=utf-8''" + file.getName());

                log.debug("------------------test2------------------");
                byte[] buffer = new byte[2048];
                FileInputStream fis = null;
                log.debug("------------------test3------------------");
                try {
                    fis = new FileInputStream(file);
                    OutputStream out = new BufferedOutputStream(response.getOutputStream());
                    int b = 0;
                    while ((b = fis.read(buffer)) != -1) {
                        out.write(buffer, 0, b); //4.写到输出流(out)中
                    }
                    out.flush();
                    out.close();
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    if (fis != null) {
                        try {
                            fis.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }

                }
            }
        }
    }



//
//    @ApiOperation(value = "更改章节状态", notes = "根据章节ID更改章节状态")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar")})
//    @PostMapping(value = "/updateChapterState")
//    public Result updateChapterState(CurUser curUser,@Valid BChapter bChapter,BindingResult br) {
//        int data = bChapterService.updateChapterState(bChapter);
//        return Result.builder(new PersistModel(data),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapter);
//    }

//    @ApiOperation(value = "章节向上调顺序", notes = "章节向上调顺序")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
//            @ApiImplicitParam(name = "sort", value = "当前顺序", required = true, dataType = "Integer")})
//    @PostMapping(value = "/updateTopChapter")
//    public Result updateTopChapter(CurUser curUser,@Valid BChapter bChapter,BindingResult br) {
//        int data = bChapterService.updateTopChapter(bChapter);
//        return Result.builder(new PersistModel(data),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapter);
//    }
//
//    @ApiOperation(value = "章节向下调顺序", notes = "章节向下调顺序")
//    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar"),
//            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
//            @ApiImplicitParam(name = "sort", value = "当前顺序", required = true, dataType = "Integer")})
//    @PostMapping(value = "/updateDownChapter")
//    public Result updateDownChapter(CurUser curUser,@Valid BChapter bChapter,BindingResult br) {
//        int data = bChapterService.updateDownChapter(bChapter);
//        return Result.builder(new PersistModel(data),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                bChapter);
//    }

}
