package org.rcisoft.business.bchapter.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bchapter.dto.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;
import tk.mybatis.mapper.common.Mapper;

import java.util.List;
import java.util.Map;

/**
 * Created with gwf on 2017-7-21 15:08:47.
 */
@Repository
public interface BChapterRepository extends BaseMapper<BChapter> {

    /**
     * 根据id逻辑删除章节
     *
     * @param businessId
     * @return
     */

    @Update("update b_chapter set del_flag = '1' where business_id=#{businessId}")
    int removeByPrimaryKey(@Param("businessId") String businessId);

    /**
     * 根据id更新添加视频的时长
     *
     * @param param
     * @return
     */

    @Update("update b_chapter set chapter_time = #{chapterTime} where business_id=#{businessId}")
    int insertChapterTime(Map<String, Object> param);


    /**
     * 根据条件查找
     *
     * @param param
     * @return
     */
    @Select("<script>select * from b_chapter  " +
            "where del_flag = 0 and flag = 1 and pid = -1 " +
            "   and lesson_id = #{lessonId} " +

            "and corp_id = #{corpId} " +

            "order by sort asc</script>")
    @ResultMap(value = "SupperChildListResultMap")
    List<QueryChapterListResDTO> queryBChapters(Map<String, Object> param);

    /**
     * 根据父节点查找子节点列表
     *
     * @param pid
     * @return
     */
    @Select("<script>SELECT" +
            " bc.business_id, " +
            " bc.chapter_name, " +
            " bc.chapter_level, " +
            " bc.pid, " +
            " bc.chapter_url, " +
            " bc.is_test, " +
            " bc.lesson_id, " +
            " bc.sort," +
            " bf.video_url AS videoUrl, " +
            " bf.file_name AS fileName " +
            " FROM " +
            " b_chapter bc " +
            " LEFT JOIN b_file bf ON bf.chapter_id = bc.business_id " +
            " WHERE " +
            " bc.del_flag = 0 " +
            " AND bc.pid = #{pid} " +
            " ORDER BY " +
            " bc.sort asc</script>")
    @ResultMap(value = "SupperChildListResultMap")
    List<QueryChapterListResDTO> queryBChaptersByPid(String pid);

    /**
     * 根据章节id查找该章节学生分数列表
     *
     * @param
     * @return
     */
    @Select("<script>SELECT " +
            " t1.chapter_name,t2.automatic,t5.class_name,t3.`code`,t6.`name`,t2.score,t3.business_id as sId,t2.business_id as scoreId " +
            "FROM " +
            " b_chapter t1 " +
            " LEFT JOIN b_r_student_chapter t2 ON t2.chapter_id = t1.`business_id` " +
            " LEFT JOIN b_student t3 ON t2.student_id = t3.`business_id` " +
            " LEFT JOIN b_r_class_student t4 ON t3.`code` = t4.student_code " +
            " LEFT JOIN b_class t5 ON t5.`code` = t4.class_code " +
            " LEFT JOIN s_user t6 ON t6.login_name = t3.`code` " +
            "WHERE t1.del_flag='0' and t1.flag='1' and t2.is_complete='1' " +
            " and t2.`score`  != -1" +
            " and t1.business_id = #{dto.chapterId} " +
            "<if test=\"dto.stuInfo !=null and dto.stuInfo !=''\">AND (t3.code like CONCAT('%',#{dto.stuInfo}.'%') or t6.name like CONCAT('%',#{dto.stuInfo}.'%')) </if>  " +
            "<if test=\"dto.automatic !=null and dto.automatic !=''\">AND t2.automatic = #{dto.stuInfo} </if>  " +
            "<if test=\"dto.classInfo !=null and dto.classInfo !=''\">AND t5.code like like CONCAT('%',#{dto.classInfo}.'%') </if>  " +
            "</script>")
    @ResultMap("QueryScoreListByChapterIdResultMap")
    List<StudentChapterScoreVO> queryScoreListByChapterId(@Param("dto") QueryScoreListDTO dto);

    /**
     * 根据学生ID和章节ID查询该节的学习情况
     * @param
     * @return
     */
    @Select("<script>SELECT " +
            " t1.is_complete as isComplete " +
            "FROM " +
            " b_r_student_chapter t1 " +
            "LEFT JOIN b_student t2 ON t1.student_id = t2.business_id " +
            "LEFT JOIN s_user t3 ON t3.login_name = t2.code " +
            "WHERE t1.chapter_id = #{chapterId} and t3.business_id = #{studentId}" +
            "</script>")
    ScoreInfoDTO queryChapInfoById(@Param("chapterId") String chapterId, @Param("studentId") String studentId);

    /**
     * 根据课程id和学生id查找该学生所学课程分数列表
     *
     * @param slid
     * @param stuid
     * @return
     */
    @Select("<script>SELECT " +
            " t2.chapter_id as chapterId, " +
            " t2.score " +
            "FROM " +
            " b_chapter t1 " +
            "LEFT JOIN b_r_student_chapter t2 ON t1.business_id = t2.chapter_id " +
            "LEFT JOIN b_student t4 ON t2.student_id = t4.business_id " +
            "LEFT JOIN s_user t5 ON t5.login_name = t4.code " +
            "WHERE " +
            " t3.business_id = #{slid} " +
            "AND t5.business_id = #{stuid} " +
            "AND t1.del_flag = 0 and t1.flag = 1</script>")
    List<StuScoreDTO> selectScoreListBySlIdAndStuId(@Param("slid") String slid, @Param("stuid") String stuid);

    @Delete("<script>delete from b_r_student_chapter where chapter_id = #{chapterId}</script>")
    int deleteStudentChapter(String chapterId);

    /**
     * 根据章节id和学生id获取主键
     *
     * @param chapterId
     * @param studentId
     * @return
     */
    @Select("<script>select business_id from b_r_student_chapter " +
            "where chapter_id = #{chapterId} and student_id = #{studentId}</script>")
    String selectIdByStudentIdAndChapterId(@Param("chapterId") String chapterId, @Param("studentId") String studentId);

    /**
     * 插入分数
     *
     * @param scoreInfoDTO
     * @return
     */
    @Insert("</script>insert into b_r_student_chapter(business_id,chapter_id,student_id,score,sl_id) values(#{businessId},#{chapterId},#{studentId},#{score},#{slId})</script>")
    int insertScoreInfo(ScoreInfoDTO scoreInfoDTO);

    /**
     * 更新分数
     *
     * @param scoreInfoDTO
     * @return
     */
    @Update("<script>update b_r_student_chapter set chapter_id=#{chapterId},automatic=#{automatic},student_id=#{studentId},score=#{score} where business_id = #{businessId}</script>")
    int updateScoreInfo(ScoreInfoDTO scoreInfoDTO);

    /**
     * 更新完成状态
     *
     * @param scoreInfoDTO
     * @return
     */
    @Insert("<script>insert into b_r_student_chapter(business_id,student_id,chapter_id,is_complete,sl_id,score) values (#{businessId},#{studentId},#{chapterId},#{isComplete},#{slId},#{score})</script>")
    int updateIsComplete(ScoreInfoDTO scoreInfoDTO);

    /**
     * 删除分数记录
     *
     * @param chapterId
     * @param studentId
     * @return
     */
    @Update("<script>update b_r_student_chapter set score = '999', is_complete = '0' where chapter_id = #{chapterId} and student_id = #{studentId}</script>")
    int updateScoerInfo(@Param("chapterId") String chapterId, @Param("studentId") String studentId);

    @Select("<script>select * from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and lesson_id = #{lessonId}</script>")
    @ResultMap(value = "BaseResultMap")
    List<BChapter> queryChapterByLessonId(String lessonId);

    @Insert("<script>INSERT INTO b_chapter " +
            "(create_date,create_by,update_date,update_by," +
            "del_flag,flag,business_id," +
            "sl_id,chapter_name,chapter_level,pid," +
            "chapter_url,md_file,is_test,lesson_or_sl,class_hour,sort,experiment_type)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.createDate},#{item.createBy},#{item.updateDate},#{item.updateBy}," +
            "#{item.delFlag},#{item.flag},#{item.businessId}," +
            "#{item.slId},#{item.chapterName},#{item.chapterLevel},#{item.pid}," +
            "#{item.chapterUrl},#{item.mdFile},#{item.isTest},#{item.lessonOrSl},#{item.classHour},#{item.sort},#{item.experimentType})" +
            "</foreach></script>")
    int insertChapterList(List<BChapter> slList);

    @Select("<script>select chapter_name from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{pid}</script>")
    String selectChapterNameByPid(String pid);

    @Select("<script>select state from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{businessId}</script>")
    String selectStateById(String businessId);

    @Update("<script>update b_chapter set state = #{state} where  business_id = #{businessId} or pid = #{businessId}</script>")
    int updateStateById(BChapter bChapter);

    @Update("<script>update b_chapter set state = #{state} where  business_id = #{pid}</script>")
    int updateStateByPid(@Param("pid") String pid, @Param("state") String state);

    @Select("<script>select DISTINCT state from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and sl_id = #{slId} order by state desc</script>")
    List<String> selectStateBySlId(String slId);

    @Update("<script>update b_chapter set state = #{state} where  sl_id = #{slId}</script>")
    int updateAllStateBySlId(@Param("slId") String slId, @Param("state") String state);

    @Select("<script>select DISTINCT IFNULL((select max(sort) from b_chapter where pid = #{pid} " +
            "and del_flag = 0 and flag = 1 " +
            "<if test=\"lessonId !=null and lessonId !=''\">AND lesson_id = #{lessonId} </if>),0)+1 as sort</script>")
    int selectMaxSort(@Param("pid") String pid,
                      @Param("lessonId") String lessonId);

    @Select("<script>select * from b_chapter where sort &lt; #{sort} " +
            "AND pid = #{pid} and del_flag = 0 and flag = 1 " +
            "<if test=\"lessonId !=null and lessonId !=''\">AND lesson_id = #{lessonId} </if> " +
            "ORDER BY sort desc LIMIT 1</script>")
    @ResultMap(value = "BaseResultMap")
    BChapter getlessChapter(BChapter bChapter);


    @Update("<script>update b_chapter set sort = #{sort} where business_id = #{businessId}</script>")
    int updateSortById(@Param("businessId") String businessId, @Param("sort") int sort);

    @Select("<script>select * from b_chapter where sort &gt; #{sort} " +
            "AND pid = #{pid} and del_flag = 0 and flag = 1 " +
            "<if test=\"lessonId !=null and lessonId !=''\">AND lesson_id = #{lessonId} </if> " +
            "ORDER BY sort asc LIMIT 1</script>")
    @ResultMap(value = "BaseResultMap")
    BChapter getBigChapter(BChapter bChapter);

    @Select("<script>select * from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and sl_id = #{slId}</script>")
    @ResultMap(value = "BaseResultMap")
    List<BChapter> queryChapterBySlId(String slId);

    @Select("<script>select automatic from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{businessId}</script>")
    String selectAutomatic(String businessId);

    @Update("<script>update b_chapter set automatic = #{automatic} where business_id = #{businessId}</script>")
    int updateAutomatic(@Param("automatic") String automatic, @Param("businessId") String businessId);

    @Select("<script>select md_file from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{businessId}</script>")
    String queryMdFileByChapterId(String businessId);

    @Select("select CONCAT((SELECT sort from b_chapter where business_id = #{pid}), " +
            "'.'," +
            "(SELECT sort from b_chapter where business_id = #{businessId})) AS sort  " +
            "from b_chapter where business_id = #{businessId} " +
            " and del_flag = 0 and flag = 1 ")
    String querySortByPidAndId(@Param("pid") String pid, @Param("businessId") String businessId);


    @Select("<script>select bc.*,su.business_id as userId,su.`name` as studentName,su.login_name as stuId from b_chapter bc  " +
            "LEFT JOIN b_r_student_chapter bsc on bc.business_id = bsc.chapter_id " +
            "LEFT JOIN b_student bs on bsc.student_id = bs.business_id " +
            "LEFT JOIN s_user su on su.login_name = bs.`code`  " +
            "where bc.business_id = #{chapterId} and bsc.score = '-1' and bsc.is_complete = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<BChapter> queryBchapterStudentById(String chapterId);




    @Select("<script>select business_id from b_student where code = #{code} and del_flag = 0 and flag = 1</script>")
    String queryStuIdByCode(String code);


    @Select(" SELECT business_id AS businessId " +
            " from b_r_student_chapter  " +
            " where student_id = #{stuId}  " +
            " and chapter_id = #{chapterId} LIMIT 1")
    ScoreInfoDTO queryStudentChapter(@Param("chapterId") String chapterId, @Param("stuId") String stuId);


    @Insert("<script>insert into b_r_student_chapter" +
            "(business_id,student_id,chapter_id,is_complete,sl_id,score,ppt_finish,pdf_finish,video_finish,paper_finish)" +
            " values " +
            "(#{businessId},#{studentId},#{chapterId},#{isComplete},#{slId},#{score},#{pptFinish},#{pdfFinish},#{videoFinish},#{paperFinish})</script>")
    int insertStudentChapter(ScoreInfoDTO scoreInfoDTO);

    @Select("SELECT bsc.is_complete FROM b_r_student_chapter bsc  " +
            "LEFT JOIN b_student bs on bs.business_id = bsc.student_id " +
            "LEFT JOIN s_user su on su.login_name = bs.`code` " +
            "WHERE su.business_id = #{userId} " +
            "and bsc.chapter_id = #{chapId}")
    String queryIsCompleteByUserIdAndChapId(@Param("userId") String userId, @Param("chapId") String chapId);


    @Select("<script>select DISTINCT state from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and pid = #{pid} order by state desc</script>")
    List<String> queryChapterStateByPid(String pid);

    @Update("update  b_lesson set class_hour =" +
            "(select count(*) from b_chapter where lesson_id = #{lessonId} and chapter_level = 2 and del_flag != 1 and flag =1) " +
            "where business_id = #{lessonId}")
    int updateClassHourInLesson(@Param("lessonId") String lessonId);


    @Select("<script>" +
            "SELECT DISTINCT bc.*, brc.current_location as currentLocation, bf.video_url as videoUrl, brc.progress as progress, bf.file_url as fileUrl " +
            "FROM " +
            " b_lesson_person bp " +
            " LEFT JOIN b_chapter bc ON bc.business_id = bp.chapter_id " +
            "left join b_r_student_chapter brc on brc.chapter_id = bc.business_id " +
            "left join b_file bf on bf.chapter_id = bc.business_id " +
            "WHERE " +
            " bp.chapter_id IS NOT NULL  " +
            " AND bp.chapter_id != ''  " +
            " AND bp.lesson_id = #{lessonId} " +
            " AND bp.person_id = #{userId} " +
            " AND brc.student_id = #{userId} " +

            "and bc.corp_id = #{corpId} " +
            "</script>")
    @ResultMap(value = "SupperChildListResultMap")
    QueryChapterListResDTO getWithCurrentChapterBylessonId(@Param("lessonId")String lessonId, @Param("userId")String userId, @Param("corpId")String corpId);

    /**
     * 根据business_id将第一章第一节的课程时长加到course_time
     * @param time
     * @param id
     * @return
     */
    @Update("UPDATE b_lesson set course_time = #{time} where business_id =#{id}")
    int updateVideoTimeInLesson(@Param("time") String time,@Param("id") String id);

    /**
     * 根据lessonid获取它第一节的章节id
     * @param lessonId
     * @return
     */
    @Select("select c1.business_id from b_chapter c1 " +
            "left join b_chapter c2 on c1.pid = c2.business_id " +
            "where c1.lesson_id = #{lessonId}  " +
            "and c1.chapter_level = 2 " +
            "and c1.del_flag != 1 " +
            "and c1.flag = 1 " +
            "and c2.del_flag != 1 " +
            "and c2.flag = 1 " +
            "ORDER BY c2.sort ,c1.sort limit 1")
    String getFirstChapter(@Param("lessonId") String lessonId);

    /**
     * 更新b_lesson 表中 course_time
     * @param time
     * @param id
     * @return
     */
    @Update("update b_lesson set course_time = #{time} where business_id = #{id}")
    int updateCourseTimeForLesson(@Param("time") String time ,@Param("id") String id);

    /**
     * 根据businessid 获取 该节 的课程时长
     * @param id
     * @return
     */
    @Select("select course_time from  b_chapter where business_id = #{id}")
    String getTimeByChapterId(@Param("id") String id);
    /**
     * 根据章节id查询所在课程id
     * @param businessId
     * @return
     */
    @Select("select lesson_id from b_chapter where business_id = #{businessId}")
    String getLessonIdBychaperId(@Param("businessId") String businessId);

    @Delete("delete from b_r_student_chapter where chapter_id = #{chapterId} and student_id = #{uid} ")
    int deleteBStudentChapter(@Param("chapterId") String chapterId,@Param("uid") String uid);

    @Select("select count(0) from b_chapter where lesson_id = #{lessonId} and chapter_level = 2 and del_flag != 1 and flag =1")
    int getlessonHours(@Param("lessonId") String lessonId);

    /**
     * 获取当前课程或章下所有的章节id
     * @return
     */
    @Select("select business_id from b_chapter where pid = #{pid} and lesson_id = #{lessonId} and del_flag != 1 and flag = 1 ORDER BY sort ")
    List<String> getChapterIds(@Param("pid")String pid,@Param("lessonId")String lessonId);

    @Update("<script>" +
            "<foreach collection=\"map\" item=\"item\" separator=\";\">" +
            "update b_chapter set sort = #{item.sort}  where business_id = #{item.businessId}"+
            "</foreach>" +
            "</script>")
    int updateSort(@Param("map") List<Map<String,Object>> map);
}

