package org.rcisoft.business.bchapter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bchapter.dto.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.AutomaticEnum;
import org.rcisoft.business.bchapter.enums.ChapterLevelEnum;
import org.rcisoft.business.bchapter.enums.HasMarkEnum;
import org.rcisoft.business.bchapter.enums.StateEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.business.bfile.dto.uploadDTO;
import org.rcisoft.business.bfile.util.GetVideoTime;
import org.rcisoft.business.blesson.dao.BCollectRepository;
import org.rcisoft.business.blesson.dao.BLessonPersonRepository;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.dto.AppointLessonDTO;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.bfile.dao.BFileRepository;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.blesson.service.impl.BLessonServiceImpl;
import org.rcisoft.business.blesson.util.QueryDepart;
import org.rcisoft.business.bmaterial.dao.BMaterialRepository;
import org.rcisoft.business.bmaterial.entity.BMaterial;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.business.brstudentchapter.dao.BRStudentChapterRepository;
import org.rcisoft.business.brstudentchapter.entity.BRStudentChapter;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.OfficeToPdf;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;


/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BChapterServiceImpl implements BChapterService {
    @Autowired
    private BChapterRepository bChapterRepository;
    //    @Autowired
//    private BSlRepository bSlRepository;
//    @Autowired
//    private BStudentRepository bStudentRepository;
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    private BFileRepository bFileRepository;
    @Autowired
    private BMaterialRepository bMaterialRepository;
    @Autowired
    private Global global;
    @Autowired
    BRStudentChapterRepository brStudentChapterRepository;
    @Autowired
    BLessonPersonRepository bLessonPersonRepository;
    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;
    @Autowired
    BCollectRepository bCollectRepository;
    @Autowired
    BLessonService bLessonService;
    @Autowired
    OfficeToPdf officeToPdf;
    @Autowired
    BLessonServiceImpl bLessonServiceImpl;

    @Override
    public BChapter selectOne(String businessId) {
        BChapter result = bChapterRepository.selectByPrimaryKey(businessId);
        if (result == null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);
        return result;
    }

    @Override
    public List<QueryChapterListResDTO> queryBChaptersByPagination(PageUtil pageUtil, QueryChapterListDTO model, String corpId) {
        return queryChapterListResDTO(model, corpId);
    }

    @Override
    public List<StudentChapterScoreVO> queryScoreListByChapterIdByPagination(PageUtil pageUtil, QueryScoreListDTO dto) {
        return bChapterRepository.queryScoreListByChapterId(dto);
    }

    @Override
    public List<QueryChapterListResDTO> queryBChapters(QueryChapterListDTO model, String corpId) {
        return queryChapterListResDTO(model, corpId);
    }

    @Override
    public Map queryBChaptersWithProgress(String lessonId, CurUser curUser) {
        QueryChapterListDTO model = new QueryChapterListDTO();
        model.setLessonId(lessonId);
        model.setStudentId(curUser.getUserId());
        List<QueryChapterListResDTO> queryChapterListResDTOS = queryChapterListResDTO(model, curUser.getCorpId());
        for (QueryChapterListResDTO queryChapterListResDTO : queryChapterListResDTOS) {
            for (QueryChapterListResDTO childList : queryChapterListResDTO.getChildList()) {
                // 遍历学生章节中间表 查询学生观看章节进度
                List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(childList.getBusinessId(), curUser.getUserId());
                if (null != brStudentChapterList && brStudentChapterList.size() > 0) {
                    childList.setProgress(brStudentChapterList.get(0).getProgress() * 100 + "%");
                    childList.setCurrentLocation(brStudentChapterList.get(0).getCurrentLocation());
                }
            }
        }
        Map map = new HashMap();
        DecimalFormat df = new DecimalFormat("0.00");//设置保留位数
        map.put("chapterList", queryChapterListResDTOS);
        BLessonPerson bLessonPerson = bLessonPersonRepository.getAppointInTraining(lessonId, curUser.getUserId());
        if (null != bLessonPerson) {
            map.put("totalProgress", bLessonPerson.getLearnProgress());
        }else {
            map.put("totalProgress", "0%");
        }
        BLesson bLesson = bLessonRepository.selectInfoById(lessonId,curUser);
        //判断课程是否存在
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
//        //判断课程是否被关闭
//        if ("4".equals(bLesson.getReleaseState())){
//            throw new ServiceException(ResultServiceEnums.LESSON_CLOSED);
//        }
        //判断该登录人是否可见该课程
        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
        int result = bLessonRepository.isInViewRange(curUser,lessonId,departs);
        if (result < 1){
            throw new ServiceException(ResultServiceEnums.NO_ACCESS);
        }

        //设置讲师部门
        List<String> lecturerIds = new ArrayList<>();
        lecturerIds.add(bLesson.getLecturerId());
        MTUserGetsReqDTO mtUserGetsReqDTOlecDept = new MTUserGetsReqDTO();
        mtUserGetsReqDTOlecDept.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTOlecDept.setIds(lecturerIds);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOLecDeptList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTOlecDept);
        if (mtUserInfoRspDTOLecDeptList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOLecDeptList.forEach(mtUserInfoRspDTO -> {
                if (mtUserInfoRspDTO.getDepts() != null && mtUserInfoRspDTO.getDepts().size() > 0 && mtUserInfoRspDTO.getId().equals(bLesson.getLecturerId())){
                    //设置部门名
                    bLesson.setLecturerDeptName(mtUserInfoRspDTO.getDepts().get(0).getName());
                }
            });
        }

        //查询该课程是否收藏
        int collectCount = bCollectRepository.selectCountbyLessonAndUser(curUser,bLesson.getBusinessId());
        if (collectCount>0){
            bLesson.setIsCollect("1");
        }else {
            bLesson.setIsCollect("0");
        }
        map.put("lessonInfo", bLesson);  // 课程信息

        QueryChapterListResDTO queryChapterListResDTO = bChapterRepository.getWithCurrentChapterBylessonId(lessonId, curUser.getUserId(), curUser.getCorpId());
        if (null != queryChapterListResDTO && StringUtils.isNotBlank(queryChapterListResDTO.getPid())) {
            BChapter bChapter = bChapterRepository.selectByPrimaryKey(queryChapterListResDTO.getPid());
            if (null != bChapter) {
                queryChapterListResDTO.setPSort(bChapter.getSort());
                queryChapterListResDTO.setProgress(Double.parseDouble(queryChapterListResDTO.getProgress())*100+"%");
            }
            map.put("currentChapter", queryChapterListResDTO);
        } else {
            map.put("currentChapter", "");
        }

        return map;
    }

    /**
     * dto查询章节信息
     *
     * @param model
     * @return
     */
    @Override
    public List<QueryChapterListResDTO> queryChapterListResDTO(QueryChapterListDTO model, String corpId) {
        Map param = new HashMap<String, Object>();
        if (model.getLessonId() != null && !"".equals(model.getLessonId())) {
            param.put("lessonId", model.getLessonId());
        }
        if (StringUtils.isNotBlank(model.getChapterId())) {
            param.put("chapterId", model.getChapterId());
        }
        param.put("corpId", corpId);
        return bChapterRepository.queryBChapters(param);
    }


    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel persist(ChapterDTO chapterDTO, String corpId, uploadDTO dto) {
        int line = 0;
        BFile bFile = new BFile();
        BChapter model = new BChapter();

        //将dto中的数据放入bchapter实体类中
        model.setBusinessId(chapterDTO.getBusinessId());
        model.setLessonId(chapterDTO.getLessonId());
        model.setChapterName(chapterDTO.getChapterName());
        model.setPid(chapterDTO.getPid());
        model.setChapterLevel(chapterDTO.getChapterLevel());
        model.setIsTest(chapterDTO.getIsTest());
        model.setState("1");

        bFile.setFileName(dto.getFileName());
        bFile.setVideoUrl(dto.getVideoUrl());
        bFile.setType(dto.getType());
        bFile.setFileSize(dto.getFileSize());

//        if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
//            dto.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));;
//        }

        if (model.getBusinessId() == null) {
            //新建章节
            UserUtil.setCurrentPersistOperation(model);
            if (model.getClassHour() == null || model.getClassHour().equals("")) {
                model.setClassHour("");
            }
            if (StringUtils.isEmpty(model.getPid())) {
                model.setPid("-1");
            }
            if (model.getIsTest() == null || model.getIsTest().equals("")) {
                model.setIsTest("");
            }
            model.setIsTest(dto.getType());
            int sort = bChapterRepository.selectMaxSort(model.getPid(), model.getLessonId());
            model.setSort(sort);

            if (model.getChapterLevel() == 1) {

                chapterDTO.setBusinessId(model.getBusinessId());
                chapterDTO.setPid(model.getPid());
                //添加章
                line = bChapterRepository.insertSelective(model);
            } else {
                model.setClassHour("1");
//            model.setCorpId(corpId);
                bFile.setChapterId(model.getBusinessId());
                bFile.setLessonId(model.getLessonId());

                bFile.setFileTime(dto.getFileTime());
                model.setCourseTime(dto.getFileTime());
                if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
                    bFile.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
                    model.setCourseTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
                }
                UserUtil.setCurrentPersistOperation(bFile);
                //新建节数据插入到chapter表
                chapterDTO.setIsTest(dto.getType());
                line = bChapterRepository.insertSelective(model);
                log.debug("----------------------------文件时间:"+model.getCourseTime()+"---------------------");

                //先判断该课程的第一节是否是本节  更新blesson表中course_time字段
                String cid = bChapterRepository.getFirstChapter(model.getLessonId());
                if (cid.equals(model.getBusinessId())) {
                    bChapterRepository.updateVideoTimeInLesson(model.getCourseTime(), model.getLessonId());
                    log.debug("-----------------------------更新lesson中time:"+model.getCourseTime()+"---------------------");
                }
                //上传文件数据插入到bfile表中
                int x = bFileRepository.insertUploadFile(bFile);
                bChapterRepository.updateClassHourInLesson(model.getLessonId());//更新b_lesson表中 class_hour
                // 更新课程总进度
                this.updateLessonProgress(chapterDTO.getLessonId(), true, 0);
                log.debug("-----------------------------更新课时成功-------------------");
            }
        } else {
            BFile bFileOld = bFileRepository.selectInfoByChapterId(chapterDTO.getBusinessId());
            UserUtil.setCurrentMergeOperation(model);
            bFile.setChapterId(model.getBusinessId());
            UserUtil.setCurrentMergeOperation(bFile);

            model.setCourseTime(dto.getFileTime());
            model.setIsTest(dto.getType());

            if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
                model.setCourseTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
                bFile.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
            }
            //先判断该课程的第一节是否是本节 更新blesson表中course_time字段
            String cid = bChapterRepository.getFirstChapter(model.getLessonId());
            if (model.getBusinessId().equals(cid)) {
                bChapterRepository.updateVideoTimeInLesson(model.getCourseTime(), model.getLessonId());
            }
            //判断修改前后pid是否发生改变
            BChapter bChapter = bChapterRepository.selectByPrimaryKey(model.getBusinessId());
            if(bChapter !=null && !bChapter.getPid().equals(model.getPid())){
                line = bChapterRepository.updateByPrimaryKeySelective(model);
                //更新修改之前章中节的sort
                List<String> chapterIds = bChapterRepository.getChapterIds(bChapter.getPid(),bChapter.getLessonId());
                if(chapterIds != null && chapterIds.size() > 0){
                    List<Map<String,Object>> params = new ArrayList<>();
                    for(int i = 0 ;i < chapterIds.size(); i++){
                        Map<String,Object> map = new HashedMap();
                        map.put("businessId",chapterIds.get(i));
                        map.put("sort",i+1);
                        params.add(map);
                    }
                    //更新表中sort
                    bChapterRepository.updateSort(params);
                }
                int sort = bChapterRepository.selectMaxSort(model.getPid(), model.getLessonId());
                model.setSort(sort);
                line = bChapterRepository.updateByPrimaryKeySelective(model);
                //更新修改之后章中节的sort
                List<String> chapterIds2 = bChapterRepository.getChapterIds(model.getPid(),model.getLessonId());
                if(chapterIds2 != null && chapterIds2.size() > 0){
                    List<Map<String,Object>> params = new ArrayList<>();
                    for(int i = 0 ;i < chapterIds2.size(); i++){
                        Map<String,Object> map = new HashedMap();
                        map.put("businessId",chapterIds2.get(i));
                        map.put("sort",i+1);
                        params.add(map);
                    }
                    //更新表中sort
                    bChapterRepository.updateSort(params);
                }
            }else{
                line = bChapterRepository.updateByPrimaryKeySelective(model);
            }

            if(model.getChapterLevel() == 1)  {
                return  new PersistModel(line);
            }
            int x = bFileRepository.updateUploadFile(bFile);
            // 判断 新文件 更新 课程学习进度（总进度 此章节进度）
            if (bFileOld != null && StringUtils.isNotEmpty(bFileOld.getVideoUrl()) && !(bFileOld.getVideoUrl().equals(dto.getVideoUrl())) ) {
                // 所有学习过此课程的 学生课程信息
                List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByLessonId(chapterDTO.getLessonId());
                BLesson bLesson = bLessonRepository.selectByPrimaryKey(chapterDTO.getLessonId());
                NumberFormat numberFormat = NumberFormat.getInstance();
                numberFormat.setMaximumFractionDigits(2);
                for (BLessonPerson lessonPerson: bLessonPersonList){
                    // 更新 学生针对此章节的中间表
                    List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(chapterDTO.getBusinessId(), "");
                    for (BRStudentChapter studentChapter: brStudentChapterList){
                        if (lessonPerson.getPersonId().equals(studentChapter.getStudentId())) {

                            double learnProgress = Double.parseDouble(lessonPerson.getLearnProgress().split("%")[0]) / 100;
                            double newLearnProgress = Double.parseDouble(numberFormat.format((float) (learnProgress * Integer.parseInt(bLesson.getClassHour()) - studentChapter.getProgress()) / Float.parseFloat(bLesson.getClassHour())));
                            if (newLearnProgress == 0) {
                                lessonPerson.setIsFinish("0");
                                lessonPerson.setFinishDate(null);
                            }
                            if (0 < newLearnProgress && newLearnProgress < 1) {
                                lessonPerson.setIsFinish("1");
                                lessonPerson.setFinishDate(null);
                            }
                            lessonPerson.setLearnProgress(newLearnProgress * 100 + "%");
                            bLessonPersonRepository.updateByPrimaryKeySelective(lessonPerson);
                            studentChapter.setProgress(0);
                            studentChapter.setCurrentLocation(0);
                            brStudentChapterRepository.updateById(studentChapter);
                        }
                    }
                }
            }
        }

        //不是为添加章时插入资料表
        if (StringUtils.isNotEmpty(model.getIsTest())) {
            BMaterial bMaterial = new BMaterial();
            UserUtil.setCurrentPersistOperation(bMaterial);
            bMaterial.setLessonId(chapterDTO.getLessonId());
            bMaterial.setChapterId(model.getBusinessId());
            bMaterial.setFileName(dto.getFileName());

            if (StringUtils.isNotEmpty(dto.getFileTime()) && dto.getFileTime().contains(".")) {
                bMaterial.setFileTime(dto.getFileTime().substring(0, dto.getFileTime().lastIndexOf(".")));
            }

            bMaterial.setFileUrl(dto.getVideoUrl());

            bMaterial.setType(model.getIsTest());
            bMaterial.setUploadType("1");
            bMaterial.setFileSize(dto.getFileSize());
            bMaterialRepository.insertMaterial(bMaterial);
        }

        return new PersistModel(line);
    }

    /**
     *
     * @param lessonId
     * @param add true: 新增 false 删除
     */
    // 新增章节 - 更新学生观看课程总进度
    private void updateLessonProgress(String lessonId, boolean add, double delProgress){
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMaximumFractionDigits(2);
        List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByLessonId(lessonId);
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(lessonId);
        for (BLessonPerson lessonPerson: bLessonPersonList){
            String newLearnProgress ;
            if(lessonPerson.getLearnProgress() == null){
                break;
            }
            double learnProgress = Double.parseDouble(lessonPerson.getLearnProgress().split("%")[0]) / 100 ;
            if (add){
                newLearnProgress =Integer.parseInt(bLesson.getClassHour()) -1 == 0? "0": numberFormat.format( learnProgress * (Integer.parseInt(bLesson.getClassHour()) -1) / (Double.parseDouble(bLesson.getClassHour())) );
            }else {
                newLearnProgress = numberFormat.format(learnProgress  - delProgress/ (Integer.parseInt(bLesson.getClassHour()) + 1));
//                newLearnProgress = numberFormat.format(((learnProgress * Integer.parseInt(bLesson.getClassHour() + 1) - delProgress) / Double.parseDouble(bLesson.getClassHour()) ));
            }
            if (0 == Double.parseDouble(newLearnProgress)){
                lessonPerson.setIsFinish("0");
                lessonPerson.setFinishDate(null);
            }else {
                if (learnProgress > 0 && Double.parseDouble(newLearnProgress) < 0.01) {
                    newLearnProgress = "0.01";
                }
                if ("2".equals(lessonPerson.getIsFinish()) && Double.parseDouble(newLearnProgress) < 1) {
                    lessonPerson.setIsFinish("1");
                    lessonPerson.setFinishDate(null);
                }
                if (Double.parseDouble(newLearnProgress) >= 1) {
                    newLearnProgress = "1";
                    lessonPerson.setIsFinish("2");
                }
            }
            //TODO 判断 曾经是否学完过 奖励积分
            if ( "2".equals(lessonPerson.getIsFinish()) && !"1".equals(lessonPerson.getEverFinished())){
                //查询课程信息
                BPersonValue bPersonValue = new BPersonValue(lessonPerson.getPersonId(),"学完课程",bLesson.getValueGain(),"0");
                bLessonServiceImpl.addValueEvent(lessonPerson.getPersonId(),bPersonValue,null);
            }
            lessonPerson.setEverFinished("1");


            lessonPerson.setLearnProgress(Float.parseFloat(newLearnProgress)*100+"%");
            bLessonPersonRepository.updateByPrimaryKeySelective(lessonPerson);

        }
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel update(BChapter model) {
        int line = bChapterRepository.updateByPrimaryKeySelective(model);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel removeBChapter(String id,String userId) {
        BChapter bChapter = bChapterRepository.selectByPrimaryKey(id);
        String lessonId = bChapterRepository.getLessonIdBychaperId(id);
        if (bChapter == null)
            throw new ServiceException(ResultServiceEnums.CHAPTER_NOT_EXISTS);

        int chapterCount =bChapterRepository.getlessonHours(lessonId);
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(lessonId);
        if(("2".equals(bLesson.getReleaseState()) || "4".equals(bLesson.getReleaseState())) && chapterCount == 1 && bChapter.getChapterLevel() != 1){
            throw new ServiceException(ResultServiceEnums.CHAPTER_ONLY_ONE);
        }
        List<QueryChapterListResDTO> childList = bChapterRepository.queryBChaptersByPid(id);
        boolean hasChild = childList != null && childList.size() > 0;
        boolean isChapter = bChapter.getChapterLevel() == ChapterLevelEnum.CHAPTER.getCode();
        if (isChapter && hasChild) {
            throw new ServiceException(ResultServiceEnums.CHAPTER_HAS_CHILD);
        }
        //获取第一节课的id 与当前id对比
        String firstId =  bChapterRepository.getFirstChapter(lessonId);
        int line = bChapterRepository.removeByPrimaryKey(id);
        bChapterRepository.updateClassHourInLesson(lessonId);
        if(StringUtils.isNotEmpty(id) && id.equals(firstId)){
            //当前节为第一节 获取下一节的课程时长
            String secondId = bChapterRepository.getFirstChapter(lessonId);
            String time = bChapterRepository.getTimeByChapterId(secondId);
            //更新到b_lesson 表
            bChapterRepository.updateCourseTimeForLesson(time,lessonId);
        }
        //更新表中sort
        List<String> chapterIds = bChapterRepository.getChapterIds(bChapter.getPid(),bChapter.getLessonId());
        if(chapterIds != null && chapterIds.size() > 0){
            List<Map<String,Object>> params = new ArrayList<>();
            for(int i = 0 ;i < chapterIds.size(); i++){
                Map<String,Object> map = new HashedMap();
                map.put("businessId",chapterIds.get(i));
                map.put("sort",i+1);
                params.add(map);
            }
            //更新表中sort
            bChapterRepository.updateSort(params);
        }
        List<BRStudentChapter> brStudentChapterList = brStudentChapterRepository.queryByStuIdAndChapter(id, userId);
        double oldProgress = 0;
        if (null !=brStudentChapterList && brStudentChapterList.size()>0){
           oldProgress = brStudentChapterList.get(0).getProgress();
        }
        // 更新学生 对应章节总进度
        this.updateLessonProgress(lessonId, false, oldProgress);
        //删除student_chapter表中数据
        bChapterRepository.deleteBStudentChapter(id,userId);

        return new PersistModel(line);
    }

    @Override
    public String changeFileToPdf(String chapterId) {
        if (StringUtils.isEmpty(chapterId)){
            return null;
        }
        //1.  获取文件信息
        BFile bFile = bFileRepository.selectInfoByChapterId(chapterId);
        return changeFile(bFile);
    }

    @Override
    public String changeFileToPdfAtView(BFile bFile) {
        if (StringUtils.isEmpty(bFile.getVideoUrl()) || StringUtils.isEmpty(bFile.getLessonId()) || StringUtils.isEmpty(bFile.getType())){
            return null;
//            throw new ServiceException(ResultServiceEnums.PARAM_NOT_NULL);
        }
        log.debug("----------dfc----------");
        return changeFile(bFile);
    }

    @Override
    public String getDownLoadUrl(BFile bFile) {
        if (StringUtils.isEmpty(bFile.getVideoUrl()) || StringUtils.isEmpty(bFile.getLessonId())){
            return null;
//            throw new ServiceException(ResultServiceEnums.PARAM_NOT_NULL);
        }
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        String returnBasePath = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        File file = new File(filePath);
        File fileOld = new File(filePath + File.separator + name);
        if (fileOld.exists()) {
            return returnBasePath + File.separator + name;
        } else {
            String downLoadName = downLoadFile(bFile, file, filePath);
            return returnBasePath + File.separator + downLoadName;
        }
    }


    public String changeFile(BFile bFile) {
        log.debug("****************request + begin***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);
        String filePath = global.getBASE_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        String returnBasePath = global.getRETURN_UPLOAD_SERVER_LOCATION() + global.getCOURSE_LOCATION() + File.separator + bFile.getLessonId();
        File file = new File(filePath);
        //2. pdf 本地是否存在， word ppt 转换后的pdf是否存在
        log.debug("------------------z1-----------------------------" + filePath);
        File fileOld = new File(filePath + File.separator + name);
        log.debug("------------------z2-----------------------------" + fileOld);
        String newName = name.substring(0, name.indexOf(".")) + "-pdf.pdf";
        log.debug("------------------z3-----------------------------" + newName);
        //转换后的路径
        File fileNew = new File(filePath + File.separator + newName);
        log.debug("------------------z4-----------------------------" + fileNew.getPath());
        if ("3".equals(bFile.getType())) {
            log.debug("------------------z5-----------------------------");
            if (fileOld.exists()) {
                log.debug("------------------z6-----------------------------" + fileOld);
                return returnBasePath + File.separator + name;
            } else {
                String downLoadName = downLoadFile(bFile, file, filePath);
                log.debug("------------------z7-----------------------------" + downLoadName);
                return returnBasePath + File.separator + downLoadName;
            }
        }
        //3. pdf不存在，下载  ，  删除word ppt，重新下载，转换
        if ("2".equals(bFile.getType()) || "4".equals(bFile.getType())) {
            log.debug("------------------z8-----------------------------");
            if (fileNew.exists()) {
                log.debug("------------------z9-----------------------------");
                return returnBasePath + File.separator + newName;
            } else {
                log.debug("------------------z10-----------------------------");
                fileOld.delete();
                log.debug("------------------z11-----------------------------");
                String downLoadFileUrl = downLoadFile(bFile, file, filePath);
                log.debug("------------------z12-----------------------------downLoadFileUrl：" + filePath + File.separator + downLoadFileUrl);
                log.debug("------------------z12-----------------------------filePath：" + filePath + File.separator + newName);
                int result = officeToPdf.transformToPdf(filePath + File.separator + downLoadFileUrl, filePath + File.separator + newName);
                log.debug("------------------z13-----------------------------");
                if (result < 1) {
                    log.error("文件转换异常");
                    throw new ServiceException(ResultServiceEnums.CHANGE_FILE_ERROR);
                }
                //4. 更新在 bfile 表    ， ppt和word  存一下转换之后的url filePath
                if (StringUtils.isNotEmpty(bFile.getChapterId())) {
                    log.debug("------------------z14-----------------------------");
                    bFile.setFileUrl(filePath + File.separator + newName);
                    log.debug("------------------z15-----------------------------");
                    bFileRepository.updateByPrimaryKeySelective(bFile);
                }
                log.debug("------------------z16-----------------------------");
                log.debug("****************request + end***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));

                return returnBasePath + File.separator + newName;
            }
        } else {
            return null;
        }
    }


    //下载
    public String downLoadFile(BFile bFile, File file, String filePath) {
        log.debug("****************download + begin***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
        String name = bFile.getVideoUrl().substring(bFile.getVideoUrl().lastIndexOf("/") + 1);

        log.debug("------------------D1-----------------------------" + filePath);
        //判断文件夹是否存在
        if (!file.exists()) {
            log.debug("------------------D2-----------------------------");
            //如果文件夹不存在，则创建新的的文件夹
            file.mkdirs();
        }
        FileOutputStream fileOut = null;
        HttpURLConnection conn = null;
        InputStream inputStream = null;
        try {
            // 建立链接
            URL httpUrl = new URL(bFile.getVideoUrl());
            log.debug("------------------D3-----------------------------");
            conn = (HttpURLConnection) httpUrl.openConnection();
            log.debug("------------------D4-----------------------------");
            //以Post方式提交表单，默认get方式
            conn.setRequestMethod("GET");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            log.debug("------------------D5-----------------------------");
            // post方式不能使用缓存
            conn.setUseCaches(false);
            log.debug("------------------D6-----------------------------");
            //连接指定的资源
            conn.connect();
            //获取网络输入流
            inputStream = conn.getInputStream();
            log.debug("------------------D7-----------------------------");
            BufferedInputStream bis = new BufferedInputStream(inputStream);
            log.debug("------------------D8-----------------------------");
            //判断文件的保存路径后面是否以/结尾
            if (!filePath.endsWith(File.separator)) {
                log.debug("------------------D9-----------------------------", filePath);
                filePath += File.separator;
            }
            //写入到文件（注意文件保存路径的后面一定要加上文件的名称）
            fileOut = new FileOutputStream(filePath + name);
            log.debug("------------------D10-----------------------------");
            BufferedOutputStream bos = new BufferedOutputStream(fileOut);
            log.debug("------------------D11-----------------------------");
            byte[] buf = new byte[1024 * 512];
            int length = bis.read(buf);
            //保存文件
            while (length != -1) {
                bos.write(buf, 0, length);
                length = bis.read(buf);
            }
            bos.close();
            bis.close();
            conn.disconnect();
            log.debug("------------------D12-----------------------------");
            log.debug("****************download + end***********" + DateFormatUtils.format(new Date(), "HH:mm:ss"));
        } catch (Exception e) {
            log.error(e.getMessage() + "下载文件异常");
            throw new ServiceException(ResultServiceEnums.DOWNLOAD_ERROR);
        }
        log.debug("------------------D13-----------------------------" + name);
        return name;
    }

    @Override
    @Transactional
    public void markScore(List<ScoreInfoDTO> scoreInfoDTOS) {
        for (ScoreInfoDTO scoreInfoDTO : scoreInfoDTOS) {
            String id = bChapterRepository.selectIdByStudentIdAndChapterId(scoreInfoDTO.getChapterId(), scoreInfoDTO.getStudentId());
            if (scoreInfoDTO.getScore() == null || scoreInfoDTO.getScore() < 0 || scoreInfoDTO.getScore() > 100)  //分数不为空且在0到100之间
                throw new ServiceException(ResultServiceEnums.SCORE_ERROR);
            if (id == null) {   //查看该学生该章节是否已经完成学习
                throw new ServiceException(ResultServiceEnums.UN_COMPLETE);
            } else {   //已经学习则修改
                scoreInfoDTO.setBusinessId(id);
                scoreInfoDTO.setAutomatic("0");
                UserUtil.setCurrentMergeOperation(scoreInfoDTO);
                bChapterRepository.updateScoreInfo(scoreInfoDTO);
            }
        }
    }

    @Override
    public List<StuScoreDTO> queryScoreListBySlIdAndStuId(String slid, String stuid) {
        return bChapterRepository.selectScoreListBySlIdAndStuId(slid, stuid);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public void deleteScoerInfo(String chapterId, String studentCode) {
//        String studentId = bStudentRepository.queryStudentIdByCode(studentCode);
//        int result = bChapterRepository.updateScoerInfo(chapterId,studentId);
//        if(result<1)
        throw new ServiceException(ResultServiceEnums.SCORE_INFO_NOT_EXISTS);
    }


    @Override
    public ScoreInfoDTO getChapInfoById(ScoreInfoDTO scoreInfoDTO) throws Exception {

        return bChapterRepository.queryChapInfoById(scoreInfoDTO.getChapterId(), scoreInfoDTO.getStudentId());
    }


    /**
     * 递归方法复制文件夹及子目录
     *
     * @param oldPath
     * @param newPath
     */
    private void copyFileToSl(String oldPath, String newPath) throws IOException {

        File oldFile = new File(oldPath);
        File[] fs = oldFile.listFiles();
        File newFile = new File(newPath);
        if (!newFile.exists()) {
            newFile.mkdirs();
        }
        for (File f : fs) {
            if (f.isFile()) {
                Path link = Paths.get(newPath + File.separator + f.getName());
                Path existing = Paths.get(f.getPath());
                Files.createLink(link, existing);
                //fileCopy(f.getPath(),newPath+File.separator+f.getName()); //调用文件拷贝的方法
            } else if (f.isDirectory()) {
                copyFileToSl(f.getPath(), newPath + File.separator + f.getName());
            }
        }
    }

    private void fileCopy(String oldPath, String newPath) {
        BufferedReader br = null;
        PrintStream ps = null;
        try {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(oldPath)));
            ps = new PrintStream(new FileOutputStream(newPath));
            String s = null;
            while ((s = br.readLine()) != null) {
                ps.println(s);
                ps.flush();
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (br != null) br.close();
                if (ps != null) ps.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateChapterState(BChapter bChapter) {
        if (bChapter.getState().equals(StateEnum.OPEN.getCode())) {
            bChapter.setState(StateEnum.CLOSE.getCode());
        } else
            bChapter.setState(StateEnum.OPEN.getCode());
        int result = bChapterRepository.updateStateById(bChapter);

        if (!bChapter.getPid().equals("-1")) {
            List<String> state = bChapterRepository.queryChapterStateByPid(bChapter.getPid());
            if (state.size() == 1 && state.get(0).equals(StateEnum.CLOSE.getCode())) {
                bChapterRepository.updateStateByPid(bChapter.getPid(), StateEnum.CLOSE.getCode());
            } else {
                bChapterRepository.updateStateByPid(bChapter.getPid(), StateEnum.OPEN.getCode());
            }
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateAllChapterState(String slId) {
        List<String> list = bChapterRepository.selectStateBySlId(slId);
        String state = StateEnum.OPEN.getCode();
        if (list.size() == 1 && list.get(0).equals(state)) {
            state = StateEnum.CLOSE.getCode();
        }
        int result = bChapterRepository.updateAllStateBySlId(slId, state);
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateTopChapter(BChapter bChapter) {
        int result = 0;
        BChapter lessChapter = bChapterRepository.getlessChapter(bChapter);
        int sort = bChapter.getSort();
        int lessSort = lessChapter.getSort();
        int data = bChapterRepository.updateSortById(bChapter.getBusinessId(), lessSort);
        int data2 = bChapterRepository.updateSortById(lessChapter.getBusinessId(), sort);
        if (data == data2) {
            result = 1;
        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateDownChapter(BChapter bChapter) {
        int result = 0;
        BChapter bigChapter = bChapterRepository.getBigChapter(bChapter);
        int sort = bChapter.getSort();
        int bigSort = bigChapter.getSort();
        int data = bChapterRepository.updateSortById(bChapter.getBusinessId(), bigSort);
        int data2 = bChapterRepository.updateSortById(bigChapter.getBusinessId(), sort);
        if (data == data2) {
            result = 1;
        }
        return result;
    }


}
