package org.rcisoft.business.bcourse.service.impl;

import com.alibaba.fastjson.JSONArray;
import org.apache.commons.lang.StringUtils;
import org.rcisoft.business.bcourse.dao.BCourseRepository;
import org.rcisoft.business.bcourse.dto.AllCourseDTO;
import org.rcisoft.business.bcourse.dto.QueryCourseResDTO;
import org.rcisoft.business.bcourse.entity.BCourse;
import org.rcisoft.business.bcourse.service.BCourseService;
import org.rcisoft.business.blesson.util.Recursion;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BCourseServiceImpl implements BCourseService {

    @Autowired
    private BCourseRepository bCourseRepository;


    @Override
    public List<QueryCourseResDTO> queryBCourse(BCourse model) {
        Map param = new HashMap<String, Object>();
        if (model.getCName() != null && !"".equals(model.getCName())) {
            param.put("cName", model.getCName());
        }
        param.put("corpId", model.getCorpId());
        return bCourseRepository.queryCourses(param);
    }

    @Override
    public List<QueryCourseResDTO> queryBCourseByPagination(PageUtil pageUtil, BCourse model) {
        Map param = new HashMap<String, Object>();
        if (model.getCName() != null && !"".equals(model.getCName())) {
            param.put("cName", model.getCName());
        }
        param.put("corpId", model.getCorpId());
        return bCourseRepository.queryCourses(param);
    }

    @Override
    public PersistModel insertCourse(BCourse bCourse) {

        if (bCourse.getCName() == null || bCourse.getCName().equals("")) {
            throw new ServiceException("课程分类名不能为空");
        }
//        List<BCourse> list = bCourseRepository.selectOneByName(bCourse.getCName(), bCourse.getCorpId());
//        if (list.size() > 0) {
//            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
//        }

        if (bCourse.getPid() == null || "".equals(bCourse.getPid())) {
            bCourse.setPid("00000000");
            bCourse.setCourseLevel("1");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else if ("00000000".equals(bCourse.getPid())) {
            bCourse.setCourseLevel("1");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else if (bCourseRepository.queryParentLevel(bCourse.getPid(), bCourse.getCorpId()) == 1) {//通过getpid 获取上一级的level
            bCourse.setCourseLevel("2");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else if (bCourseRepository.queryParentLevel(bCourse.getPid(), bCourse.getCorpId()) == 2) {
            bCourse.setCourseLevel("3");
            int sort = bCourseRepository.getSort(bCourse.getPid(), bCourse.getCorpId());
            bCourse.setSort(sort);
        } else {
            throw new ServiceException(ResultServiceEnums.COURSE_ERROR);
        }
        UserUtil.setCurrentPersistOperation(bCourse);
        return new PersistModel(bCourseRepository.insert(bCourse), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public PersistModel removeCourse(String corpId, String id) {

        List<AllCourseDTO> allCourse = null;

        if (StringUtils.isNotEmpty(id)) {
            allCourse = this.findAllCourse(corpId);
        }
        List<String> courseIds = null;
        if (allCourse != null) {
            //查询出所有子分类
            courseIds = Recursion.FindSons(id, allCourse);
            //将一级分类放入分类集合中
            courseIds.add(id);
        }
        return new PersistModel(
                bCourseRepository.removeCourse(courseIds), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public PersistModel updateCourse(BCourse bCourse) {
        if (bCourse.getCName() == null || bCourse.getCName().equals("")) {
            throw new ServiceException("课程分类名不能为空");
        }
//        List<BCourse> list = bCourseRepository.selectOneByName(bCourse.getCName(), bCourse.getCorpId());
//        if (list.size() > 0) {
//            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
//        }
        UserUtil.setCurrentMergeOperation(bCourse);
        return new PersistModel(bCourseRepository.updateCourse(bCourse), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    public PersistModel sortCourse(String json) {
//        Map<String,Object> maps = (Map) JSON.parse(json);
//        List<BCourse> bCourses = new ArrayList<>();
//        for (String obj : maps.keySet()) {
//            BCourse bCourse = new BCourse();
//            String value = maps.get(obj).toString();
//            bCourse.setBusinessId(obj.toString());
//            String[] split = StringUtils.split(value, ",");
//            bCourse.setPid(split[0]);
//            bCourse.setSort(Integer.parseInt(split[1]));
//            bCourse.setCourseLevel(split[2]);
//            bCourses.add(bCourse);
//        }
//        int line = bCourseRepository.sortCourser(bCourses);
        List<Map<String, String>> listObjectFir = (List<Map<String, String>>) JSONArray.parse(json);
        List<BCourse> bCourses = new ArrayList<>();
        for (Map<String, String> mapList : listObjectFir) {
            BCourse bCourse = new BCourse();

            for (Map.Entry<String, String> str : mapList.entrySet()) {
                if (str.getKey().equals("key")) {
                    bCourse.setBusinessId(String.valueOf(str.getValue()));
                }
                if (str.getKey().equals("pid")) {
                    bCourse.setPid(String.valueOf(str.getValue()));
                }
                if (str.getKey().equals("sort")) {
                    bCourse.setSort(Integer.parseInt(String.valueOf(str.getValue())));
                }
                if (str.getKey().equals("courseLevel")) {
                    bCourse.setCourseLevel(String.valueOf(str.getValue()));
                }
                if (str.getKey().equals("children")) {

                    String child = String.valueOf(str.getValue());
                    List<Map<String, String>> list1 = (List<Map<String, String>>) JSONArray.parse(child);
                    for (Map<String, String> mapList1 : list1) {

                        BCourse bCourse1 = new BCourse();
                        for (Map.Entry<String, String> str1 : mapList1.entrySet()) {
                            if (str1.getKey().equals("key")) {
                                bCourse1.setBusinessId(str1.getValue());
                            }
                            if (str1.getKey().equals("pid")) {
                                bCourse1.setPid(str1.getValue());
                            }
                            if (str1.getKey().equals("sort")) {
                                bCourse1.setSort(Integer.parseInt(String.valueOf(str1.getValue())));
                            }
                            if (str1.getKey().equals("courseLevel")) {
                                bCourse1.setCourseLevel(String.valueOf(str1.getValue()));
                            }
                            if (str1.getKey().equals("children")) {

                                String child2 = String.valueOf(str1.getValue());
                                List<Map<String, String>> list2 = (List<Map<String, String>>) JSONArray.parse(child2);
                                for (Map<String, String> mapList2 : list2) {
                                    BCourse bCourse2 = new BCourse();
                                    for (Map.Entry<String, String> str2 : mapList2.entrySet()) {
                                        if (str2.getKey().equals("key")) {
                                            bCourse2.setBusinessId(str2.getValue());
                                        }
                                        if (str2.getKey().equals("pid")) {
                                            bCourse2.setPid(str2.getValue());
                                        }
                                        if (str2.getKey().equals("sort")) {
                                            bCourse2.setSort(Integer.parseInt(String.valueOf(str2.getValue())));
                                        }
                                        if (str2.getKey().equals("courseLevel")) {
                                            bCourse2.setCourseLevel(String.valueOf(str2.getValue()));
                                        }
                                    }
                                    bCourses.add(bCourse2);
                                }
                            }
                        }
                        bCourses.add(bCourse1);
                    }
                }
            }
            bCourses.add(bCourse);
        }
        int line = bCourseRepository.sortCourser(bCourses);
        return new PersistModel(line);
    }

    @Override
    public List<AllCourseDTO> findAllCourse(String corpId) {
        return bCourseRepository.findAllCourse(corpId);
    }

}
