package org.rcisoft.business.bdictionary.service.impl;

import org.rcisoft.business.bdictionary.dto.ColumConfigDTO;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bdictionary.dao.BDictionaryRepository;
import org.rcisoft.business.bdictionary.entity.BDictionary;
import org.rcisoft.business.bdictionary.service.BDictionaryService;


import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-9-29 16:21:48.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BDictionaryServiceImpl implements BDictionaryService {

    @Autowired
    private BDictionaryRepository bDictionaryRepository;


    /**
    * 保存 bDictionary
    * @param bDictionary
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BDictionary bDictionary){
        //增加操作
        UserUtil.setCurrentPersistOperation(bDictionary);
        int line = bDictionaryRepository.insertSelective(bDictionary);
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bDictionary
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BDictionary bDictionary){
        UserUtil.setCurrentMergeOperation(bDictionary);
        bDictionary.setDeleted();
        int line = bDictionaryRepository.logicalDelete(bDictionary);
        return new PersistModel(line);
    }

//    /**
//    * 修改 bDictionary
//    * @param bDictionary
//    * @return
//    */
//    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
//    @Override
//    public PersistModel merge(BDictionary bDictionary){
//        UserUtil.setCurrentMergeOperation(bDictionary);
//        int line = bDictionaryRepository.updateByPrimaryKeySelective(bDictionary);
//        log.info(UserUtil.getUserInfoProp(bDictionary.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
//            bDictionary.getBusinessId()+"的信息");
//        return new PersistModel(line);
//    }
//
//    /**
//    * 根据id查询 bDictionary
//    * @param id
//    * @return
//    */
//    public BDictionary findById(String id){
//        return bDictionaryRepository.selectByPrimaryKey(id);
//    }

    /**
    * 分页查询 bDictionary
    * @param curUser
    * @return
    */
    public List<BDictionary> findAllColumnConfig(CurUser curUser){
        List<String> configList = new ArrayList<>();
        List<BDictionary> DictList = bDictionaryRepository.findAllColumnConfig(curUser,configList);
        return DictList;
    }

    @Override
    public int updateConfiguration(ColumConfigDTO param) {

        BDictionary model  = bDictionaryRepository.selectByPrimaryKey(param.getBusinessId());

        String value = param.getValue();
        if ("GZRS".equals(model.getType())||"ZXRS".equals(model.getType())){
            try {
                if (Integer.parseInt(value) < 0){
                    throw new ServiceException(ResultServiceEnums.INPUT_FORMAT_ERROR);
                }
            } catch (NumberFormatException e) {
                log.error(e.getMessage()+" 输入的格式不正确 ");
                throw new ServiceException(ResultServiceEnums.INPUT_FORMAT_ERROR);
            }
        }
        model.setValue(value);
        UserUtil.setCurrentMergeOperation(model);
        return bDictionaryRepository.updateByPrimaryKeySelective(model);
    }

}
