package org.rcisoft.business.bdiscuss.service.impl;

import org.rcisoft.business.bdiscuss.dto.BDiscussDto;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bdiscuss.dao.BDiscussRepository;
import org.rcisoft.business.bdiscuss.entity.BDiscuss;
import org.rcisoft.business.bdiscuss.service.BDiscussService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-9-25 9:03:09.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BDiscussServiceImpl implements BDiscussService {

    @Autowired
    private BDiscussRepository bDiscussRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    /**
    * 保存 bDiscuss
    * @param bDiscuss
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BDiscuss bDiscuss){
        //增加操作
        UserUtil.setCurrentPersistOperation(bDiscuss);
        int line = bDiscussRepository.insertSelective(bDiscuss);
//        log.info(UserUtil.getUserInfoProp(bDiscuss.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
//            bDiscuss.getBusinessId()+"的信息");
        return new PersistModel(line);
    }


    /**
    * 逻辑删除
    * @param bDiscuss
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BDiscuss bDiscuss){
        UserUtil.setCurrentMergeOperation(bDiscuss);
        bDiscuss.setDeleted();
        int line = bDiscussRepository.logicalDelete(bDiscuss);
//        log.info(UserUtil.getUserInfoProp(bDiscuss.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
//        bDiscuss.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bDiscuss
    * @param bDiscuss
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BDiscuss bDiscuss){
        UserUtil.setCurrentMergeOperation(bDiscuss);
        int line = bDiscussRepository.updateByPrimaryKeySelective(bDiscuss);
//        log.info(UserUtil.getUserInfoProp(bDiscuss.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
//            bDiscuss.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bDiscuss
    * @param id
    * @return
    */
    public BDiscuss findById(String id){
        return bDiscussRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bDiscuss
    * @param bDiscuss
    * @return
    */
    public List<BDiscuss> findAllByPagination(PageUtil<BDiscuss> paginationUtility,
        BDiscuss bDiscuss){
        bDiscuss.setStart();
        bDiscuss.setNotDeleted();
        return bDiscussRepository.queryBDiscusss(bDiscuss);
    }

    @Override
    public List<BDiscussDto> getDiscussByLessonId(String lessonId,String corpId) {

        List<String> ids = new ArrayList<>();

        List<BDiscussDto> bDiscussList = bDiscussRepository.getDiscussByLessonId(lessonId,"",corpId);
        bDiscussList.stream().forEach(bDiscuss -> {
            bDiscuss.setBDiscussChildList(bDiscussRepository.getDiscussByLessonId(lessonId,bDiscuss.getBusinessId(),corpId));
            ids.add(bDiscuss.getStudentId());
        });

        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(corpId);
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                bDiscussList.forEach(bDiscussDto -> {
                    if (mtUserInfoRspDTO.getId().equals(bDiscussDto.getStudentId())) {
                        //设置一级评论头像
//                        myInfoDTO.setName(mtUserInfoRspDTO.getName());
                        bDiscussDto.setHeadPic(mtUserInfoRspDTO.getAvatar());
                    }
                });
            });
        }

        return bDiscussList;
    }

}
