package org.rcisoft.business.blabel.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.blabel.dao.BLabelRepository;
import org.rcisoft.business.blabel.entity.BLabel;
import org.rcisoft.business.blabel.service.BLabelService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
@Service
public class BLabelServiceImpl implements BLabelService {

    @Autowired
    private BLabelRepository bLabelRepository;

    @Override
    public List<BLabel> queryBlabelsByPagination(PageUtil pageUtil, String lName,String corpId) {
        return bLabelRepository.queryBlabels(lName,corpId);
    }

    @Override
    public List<BLabel> queryBlabels(String lName,String corpId) {
        return bLabelRepository.queryBlabels(lName,corpId);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel insertLabel(BLabel bLabel, CurUser curUser) {
        if(StringUtils.isBlank(bLabel.getLName())){
            throw new ServiceException("标签名称不能为空");
        }
        int line = bLabelRepository.checknameByName(bLabel.getLName(),bLabel.getBusinessId(),curUser.getCorpId());
        if(line > 0){
            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
        }
        UserUtil.setCurrentPersistOperation(bLabel);
        return new PersistModel(bLabelRepository.insert(bLabel),MessageConstant.MESSAGE_ALERT_SUCCESS) ;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel updateLabel(BLabel bLabel, CurUser curUser) {
        if(StringUtils.isBlank(bLabel.getLName())){
            throw new ServiceException("标签名称不能为空");
        }
        int line = bLabelRepository.checknameByName(bLabel.getLName(),bLabel.getBusinessId(),curUser.getCorpId());
        if(line > 0){
            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
        }
        UserUtil.setCurrentMergeOperation(bLabel);
        return new PersistModel(bLabelRepository.updateByPrimaryKeySelective(bLabel), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public PersistModel removeLabel(String id) {
        int line = bLabelRepository.removeLabel(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }
}
