package org.rcisoft.business.blesson.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bcourse.dto.AllCourseDTO;
import org.rcisoft.business.bcourse.service.BCourseService;
import org.rcisoft.business.blesson.dto.*;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.business.blesson.service.BLessonPersonService;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.sys.role.service.SysRoleService;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Api(tags = "06 课程")
@Slf4j
@RestController
@RequestMapping("/BLesson")
public class BLessonController extends PaginationController<BLesson> {
    @Autowired
    private BLessonService bLessonService;

    @Autowired
    private BCourseService bCourseService;

    @Autowired
    private BLessonPersonService bLessonPersonService;

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private Global global;


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "601 添加/编辑课程", notes = "businessId为空时是添加方法，不为空时是修改方法")
    @PostMapping(value = "/add")
    public Result add(CurUser curUser, @Valid AddLessonDTO addLessonDTO, BindingResult bindingResult) {
        PersistModel data = bLessonService.persist(addLessonDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getResult());
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "602 删除课程", notes = "根据ID删除一条记录")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(CurUser curUser, String businessId, BindingResult bindingResult) {
        PersistModel data = bLessonService.removeBLesson(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "603 唯一查找", notes = "根据ID查找一条记录", response = BLesson.class)
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/one")
    public Result queryOne(CurUser curUser, String businessId, BindingResult bindingResult) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.selectOne(curUser, businessId));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "604 分页查询我发布的课程(我主讲的)", notes = "分页查询我发布的课程", response = BLesson.class)
    @ApiImplicitParam(name = "releaseState", value = "发布状态（0：待发布 1：审核中 2：已发布 3：已驳回 4：已关闭）", required = false, dataType = "varchar")
    @GetMapping(value = "/queryBLessonsByPagination")
    public Result queryBLessonsByPagination(CurUser curUser, MyReleaseDTO myReleaseDTO, BindingResult bindingResult) {
        String userId = curUser.getUserId();
        bLessonService.queryBLessonsByPagination(getPaginationUtility(), myReleaseDTO);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "605 根据条件分页查询全部课程", notes = "根据条件分页查询全部课程", response = BLesson.class)
    @GetMapping(value = "/queryAllBLessonsByPagination")
    public Result queryAllBLessonsByPagination(CurUser curUser, @Valid FindAllLessonDTO findAllLessonDTO, BindingResult bindingResult) {
        findAllLessonDTO.setCropId(curUser.getCorpId());
        bLessonService.queryAllBLessonsByPagination(getPaginationUtility(), findAllLessonDTO);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "606 分页查询大家都在学", notes = "分页查询大家都在学", response = BLesson.class)
    @GetMapping(value = "/queryPersonMoreByPagination")
    public Result queryPersonMoreByPagination(CurUser curUser, BindingResult bindingResult) {
        bLessonService.queryPersonMoreByPagination(getPaginationUtility(), curUser);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "607 分页查询企业推荐", notes = "分页查询企业推荐", response = BLesson.class)
    @GetMapping(value = "/queryRecommendByPagination")
    public Result queryRecommendByPagination(CurUser curUser, BindingResult bindingResult) {
        bLessonService.queryRecommendByPagination(getPaginationUtility(), curUser);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "608 分页查询最受关注", notes = "分页查询最受关注", response = BLesson.class)
    @GetMapping(value = "/queryConcernByPagination")
    public Result queryConcernByPagination(CurUser curUser, BindingResult bindingResult) {
        bLessonService.queryConcernByPagination(getPaginationUtility(), curUser);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "609 分页查询可能感兴趣", notes = "分页查询可能感兴趣", response = BLesson.class)
    @GetMapping(value = "/queryInterestedByPagination")
    public Result queryInterestedByPagination(CurUser curUser, BindingResult bindingResult) {
        bLessonService.queryInterestedByPagination(getPaginationUtility(), curUser);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "610 首页分页查询全部课程", notes = "根据条件分页查询全部课程", response = BLesson.class)
    @GetMapping(value = "/queryHomeBLessonsByPagination")
    public Result queryHomeBLessonsByPagination(CurUser curUser, @Valid FirstPageQueryDTO firstPageQueryDTO, BindingResult bindingResult) {
        List<AllCourseDTO> allCourse = null;
        if (StringUtils.isNotEmpty(firstPageQueryDTO.getCourseLevelOne())) {
            allCourse = bCourseService.findAllCourse(curUser.getCorpId());
        }
        bLessonService.queryHomeBLessonsByPagination(getPaginationUtility(), firstPageQueryDTO, allCourse);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "611 关闭课程", notes = "根据ID关闭一个课程")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/close")
    public Result close(CurUser curUser, String businessId, BindingResult bindingResult) {
        PersistModel data = bLessonService.closeBLesson(businessId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "612 课程培训条件查询", notes = "课程培训条件查询(Banner内部资源)", response = BLesson.class)
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @GetMapping(value = "/findLessonAtBanner")
    public Result findLessonAtBanner(CurUser curUser, @Valid LessonTrainAtBannerDTO lessonTrainDTO, BindingResult bindingResult) {
//        bLessonService.findLessonAtBannerByPagination(getPaginationUtility(),lessonTrainDTO);
//        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.findLessonAtBanner(lessonTrainDTO));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "613 我的课程总数（学习和发布）", notes = "我的课程总数")
    @GetMapping(value = "/iLessonCount")
    public Result ILessonCountDTO(CurUser curUser, BindingResult bindingResult) {
        String userId = curUser.getUserId();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.iLessonCount(curUser));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "614 退出课程", notes = "根据ID停用一条记录")
    @ApiImplicitParam(name = "businessId", value = "退出课程的选课id", required = true, dataType = "varchar")
    @PostMapping(value = "/quit")
    public Result quit(CurUser curUser, @Valid QuitLessonDTO quitLessonDTO, BindingResult bindingResult) {
        PersistModel data = bLessonPersonService.quit(quitLessonDTO, curUser);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                quitLessonDTO);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value="615 分页查询我学习", notes="分页查询我学习", response = BLessonPerson.class)
    @GetMapping(value = "/queryLearnBLessonsByPagination")
    public Result queryLearnBLessonsByPagination(CurUser curUser, @Valid ILearnLessonDTO param, BindingResult bindingResult) {
        String userId = curUser.getUserId();
        bLessonPersonService.queryLearnBLessonsByPagination(getPaginationUtility(), param);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "616 列表页分页查询课程", notes = "分页查询列表页全部", response = BLesson.class)
    @GetMapping(value = "/queryListAllLessonByPagination")
    public Result queryListAllLessonByPagination(CurUser curUser, @Valid FindListLessonDTO param, BindingResult bindingResult) {
        List<AllCourseDTO> allCourse = null;
        if (StringUtils.isNotEmpty(param.getCourse())) {
            allCourse = bCourseService.findAllCourse(curUser.getCorpId());
        }
        bLessonService.queryListAllLessonByPagination(getPaginationUtility(), param, allCourse);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }


    /**
     * @param
     * @return
     * @author: jiangpengpeng
     * @description: TODO
     * @date: 2019/9/27 14:37
     */
    @ApiOperation(value = "617课程管理显示的全部课程", notes = "课程管理显示的全部课程", response = BLesson.class)
    @GetMapping(value = "theLessonManage")
    public Result theLessonManage(CurUser curUser, @Valid BLesson bLesson, BindingResult bindingResult) {
        List<RoleTypeDepartDTO> roleList = sysRoleService.queryRoleTypeByUserId(curUser);
        List<BLesson> bLessons = bLessonService.theLessonMangageByPagination(getPaginationUtility(), curUser, bLesson, roleList);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "618 发布课程", notes = "根据ID发布一个课程")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/release")
    public Result release(CurUser curUser, String businessId, BindingResult bindingResult) {
        PersistModel data = bLessonService.releaseBLesson(businessId, curUser);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "619 线上课程学员跟踪图表", notes = "线上课程学员跟踪图表")
    @GetMapping(value = "/studentTrackingChart")
    public Result studentTrackingChart(CurUser curUser, @Valid StudentTrackingChartDTO studentTrackingChartDTO, BindingResult bindingResult) {
        StudentTrackingChartRspDTO model = bLessonPersonService.studentTrackingChart(studentTrackingChartDTO);

        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                model);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "620 学员跟踪", notes = "线上课程级线下培训学员跟踪", response = StudentTrackingRspDTO.class)
    @GetMapping(value = "/studentTrackingByPagination")
    public Result studentTrackingByPagination(CurUser curUser, @Valid StudentTrackingDTO studentTrackingDTO, BindingResult bindingResult) {
        bLessonPersonService.studentTrackingByPagination(getPaginationUtility(), studentTrackingDTO);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "621 取消指派", notes = "取消指派")
    @PostMapping(value = "/cancelAppoint")
    public Result cancelAppoint(CurUser curUser, @Valid CancelAppointDTO cancelAppointDTO, BindingResult bindingResult) {
        PersistModel data = bLessonPersonService.cancelAppoint(cancelAppointDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                cancelAppointDTO);
    }


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "622 推荐课程", notes = "根据ID推荐课程")
    @PostMapping(value = "/recommend")
    public Result recommend(CurUser curUser, @Valid RecommendLessonDTO recommendLessonDTO, BindingResult bindingResult) {
        PersistModel data = bLessonService.recommendBLesson(recommendLessonDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                recommendLessonDTO);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "623 根据课程Id 查询该课程推荐范围", notes = "根据课程id 查询课程推荐范围（人和部门）", response = FindLessonRecommendRspDTO.class)
    @GetMapping(value = "/queryRecommendByLessonId")
    public Result queryRecommendByLessonId(CurUser curUser, @Valid FindLessonRecommendDTO param, BindingResult bindingResult) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.queryRecommendByLessonId(param));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "624 指派课程", notes = "根据ID指派课程")
    @PostMapping(value = "/appointLessonToPerson")
    public Result appointLessonToPerson(CurUser curUser, @Valid AppointLessonDTO appointLessonDTO, BindingResult bindingResult) {
        PersistModel data = bLessonService.appointLessonToPerson(appointLessonDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                appointLessonDTO);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "625 添加我学习的课程", notes = "添加我学习的课程")
    @PostMapping(value = "/addMyLearnLesson")
    public Result addMyLearnLesson(CurUser curUser, @Valid AddMyLearnLessonDTO addMyLearnLessonDTO, BindingResult bindingResult) {
        PersistModel data = bLessonPersonService.addMyLearnLesson(addMyLearnLessonDTO,curUser);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                addMyLearnLessonDTO);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "626 删除推荐", notes = "删除推荐")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/deleteRecommend")
    public Result deleteRecommend(CurUser curUser, @Valid String lessonId, BindingResult bindingResult) {
        PersistModel data = bLessonService.deleteRecommend(lessonId);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                lessonId);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "627 设置为出席/缺席", notes = "设置为出席/缺席")
    @PostMapping(value = "/setTrainIsSign")
    public Result setTrainIsSign(CurUser curUser, @Valid SetTrainIsSignDTO setTrainIsSignDTO, BindingResult bindingResult) {
        PersistModel data = bLessonService.setTrainIsSign(setTrainIsSignDTO);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                setTrainIsSignDTO);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "628 我的培训总数（学习和教学）", notes = "我的培训总数")
    @GetMapping(value = "/iTrainCount")
    public Result iTrainCount(CurUser curUser, BindingResult bindingResult) {
        String userId = curUser.getUserId();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.iTrainCount(curUser));
    }


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "629 培训学员总数（已报名 已到场 未到场）", notes = "培训学员总数（已报名 已到场 未到场）")
    @GetMapping(value = "/trainPersonCount")
    public Result trainPersonCount(CurUser curUser, String lessonId, BindingResult bindingResult) {
        String userId = curUser.getUserId();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.trainPersonCount(lessonId,curUser));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "630 培训学员", notes = "培训学员", response = BLessonPerson.class)
    @GetMapping(value = "/trainPersons")
    public Result trainPersons(CurUser curUser, @Valid FindTrainPersonDTO param, BindingResult bindingResult) {

        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.trainPersons(param));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "631 报名培训", notes = "报名培训")
    @ApiImplicitParam(name = "lessonId", value = "培训课id", required = true, dataType = "varchar")
    @GetMapping(value = "/toTraining")
    public Result toTraining(CurUser curUser, String lessonId) {
        return Result.builder(new PersistModel(bLessonService.toTraining(curUser, lessonId)),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                lessonId);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "632 单一查询我参加培训的信息", notes = "单一查询我参加培训的信息", response = BLesson.class)
    @ApiImplicitParam(name = "businessId", value = "培训课id", required = true, dataType = "varchar")
    @GetMapping(value = "/findOneMyTrain")
    public Result findOneMyTrain(CurUser curUser, String lessonId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.findOneMyTrain(curUser, lessonId));
    }


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "633 单一查询我主讲培训的信息", notes = "单一查询我主讲培训的信息", response = BLesson.class)
    @ApiImplicitParam(name = "lessonId", value = "培训课id", required = true, dataType = "varchar")
    @GetMapping(value = "/findOneMyTeachTrain")
    public Result findOneMyTeachTrain(CurUser curUser, String lessonId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.findOneMyTeachTrain(curUser, lessonId));
    }


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value="634 查询指派人员", notes="查询指派人员",response = SysUser.class)
    @ApiImplicitParam(name = "lessonId", value = "培训/课程id", required = true, dataType = "varchar")
    @GetMapping(value = "/findAppointPerson")
    public Result findAppointPerson(CurUser curUser,String lessonId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.findAppointPerson(curUser,lessonId));
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "635 收藏课程或培训", notes = "收藏课程或培训")
    @ApiImplicitParam(name = "lessonId", value = "培训/课程id", required = true, dataType = "varchar")
    @GetMapping(value = "/collectLesson")
    public Result collectLesson(CurUser curUser, String lessonId) {
        return Result.builder(bLessonService.colletLesson(curUser, lessonId),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                curUser.getUserId());
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "636 取消收藏课程或培训", notes = "取消收藏课程或培训")
    @ApiImplicitParam(name = "lessonId", value = "培训/课程id", required = true, dataType = "varchar")
    @GetMapping(value = "/notCollectLesson")
    public Result notCollectLesson(CurUser curUser, String lessonId) {
        return Result.builder(bLessonService.notColletLesson(curUser, lessonId),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                curUser.getUserId());
    }



    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value="637 分页查询我关注", notes="分页查询我关注", response = BLessonPerson.class)
    @ApiImplicitParam(name = "lessonType", value = "课程类型 0 课程 1培训", required = false, dataType = "varchar")
    @GetMapping(value = "/queryCollectByPagination")
    public Result queryCollectByPagination(CurUser curUser, @Valid String lessonType, BindingResult bindingResult) {
        bLessonService.queryCollectByPagination(getPaginationUtility(), lessonType, curUser);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "638 获取课程或培训的删除(关闭)状态", notes = "获取课程或培训的删除(关闭)状态")
    @ApiImplicitParam(name = "lessonId", value = "培训/课程id", required = true, dataType = "varchar")
    @GetMapping(value = "/getLessonState")
    public Result getLessonState(CurUser curUser, String lessonId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLessonService.getLessonState(lessonId));
    }


    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "639 判断可见权限 ", notes = "判断可见权限 ")
    @PostMapping(value = "/judgeVisibleAuthority")
    public Result judgeVisibleAuthority(CurUser curUser, @Valid String lessonId, BindingResult bindingResult) {
        Map data = bLessonService.judgeVisibleAuthority(curUser,  lessonId);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data);
    }

    /**
     * @param
     * @return
     * @author: zhangqingle
     */
    @ApiOperation(value = "640 用户管理 - 课程情况 ", notes = "用户管理 - 课程情况 ", response = StudentTrackingRspDTO.class)
    @GetMapping(value = "/userManageLessonByPagination")
    public Result userManageLessonByPagination(CurUser curUser, @Valid UserLessonDTO userLessonDTO, BindingResult bindingResult) {
        bLessonService.userManageLessonByPagination(getPaginationUtility(), userLessonDTO);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value = "641 用户管理 - 考试情况 ", notes = "用户管理 - 考试情况 " ,response = ExamDto.class)
    @GetMapping(value = "/userManageExamByPagination")
    public Result userManageExamByPagination(CurUser curUser, @Valid ExamQueryDto dto, BindingResult bindingResult) {
        bLessonService.userManageExamByPagination(getPaginationUtility(), dto);
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

}
