package org.rcisoft.business.blesson.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.blabel.dto.QueryLabelResDTO;
import org.rcisoft.business.blesson.dto.*;
import org.rcisoft.business.blesson.entity.*;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * Created with CodeGenerator on .
 */
@Repository
public interface BLessonRepository extends BaseMapper<BLesson> {

    /**
     * 根据条件查找我发布的
     *
     * @param param
     * @return
     */
    @Select("<script>select b.*,su.name lecturerName " +
            " from b_lesson b " +
            " left join s_user su on b.lecturer_id = su.business_id " +
            " where b.del_flag != 1 and b.flag = 1 " +
//            " and su.del_flag != 1 and su.flag = 1 " +
            "and b.lecturer_id = #{param.userId}" +

            " and b.corp_id = #{param.corpId} "+
//            " and su.corp_id = #{param.corpId} "+

            " <if test=\"param.releaseState!=null and param.releaseState != ''\">and b.release_state = #{param.releaseState} </if>" +
            " <if test=\"param.lessonType!=null and param.lessonType != ''\">and b.lesson_type = #{param.lessonType} </if>" +
            " <if test=\"param.releaseState!=null and param.releaseState == 0\"> order by b.create_date desc </if>" +
            " <if test=\"param.releaseState!=null and param.releaseState == 2\"> order by b.release_date desc </if>" +
            " <if test=\"param.releaseState!=null and param.releaseState == 4\"> order by b.close_date desc </if>" +
//            "order by b.update_date desc" +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryBLessons(@Param("param") MyReleaseDTO param);


    /**
     * 课程新增 添加标签
     *
     * @param bLessonLabels
     * @return
     */
    @Insert("<script>INSERT INTO b_lesson_label " +
            " (business_id,label_id,lesson_id,sort)VALUES " +
            " <foreach collection=\"list\" item=\"item\" separator=\",\"> " +
            " (#{item.businessId},#{item.labelId},#{item.lessonId},#{item.sort}) " +
            " </foreach> </script> ")
    int insertLessonLabel(List<BLessonLabel> bLessonLabels);


    /**
     * 根据课程id删除课程-标签表
     *
     * @param model
     * @return
     */
    @Delete("<script>delete from b_lesson_label where lesson_id = #{businessId}</script>")
    int removeLabelByLessonId(BLesson model);

    /**
     * 单一查询课程信息
     *
     * @param businessId
     * @return
     */
    @Select(" <script> select bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName" +
            " ,blp.is_appoint isAppoint, " +
            "CONCAT( " +
            "IFNULL(CONCAT(b3.c_name,\" / \"),\"\"), " +
            "IFNULL(CONCAT(b2.c_name,\" / \"),\"\"), " +
            "  IFNULL(b1.c_name,\"\") " +
            ") AS courseName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +
            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +
            " left join b_course b1 on b1.business_id = bl.course_id " +
            "and b1.del_flag != 1 and b1.flag = 1 "+
            " left join b_lesson_person blp on blp.lesson_id = bl.business_id " +
            "and blp.del_flag != 1 and blp.flag = 1 "+
            "and blp.person_id = #{curUser.userId} "+
            "LEFT JOIN b_course b2 ON b2.business_id = b1.pid " +
            "and b2.del_flag != 1 and b2.flag = 1 "+
            "LEFT JOIN b_course b3 ON b3.business_id = b2.pid " +
            "and b3.del_flag != 1 and b3.flag = 1 "+
            " where bl.del_flag != 1 and bl.flag = 1 " +
            " and bl.business_id = #{businessId} " +
            "</script>")
    @ResultMap(value = "TrainBaseResultMap")
    BLesson selectInfoById(@Param("businessId") String businessId , @Param("curUser") CurUser curUser);


    /**
     * 根据条件查询课程并案发布时间排序
     *
     * @param param
     * @return
     */
    //分类
    @Select(" <script> select bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{cropId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{cropId} "+

            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{cropId} "+

            " <if test= \" code != null and code != ''\">and bl.code like CONCAT('%',#{code},'%') </if> " +
            " <if test= \" lessonName != null and lessonName != ''\">and bl.lesson_name like CONCAT('%',#{lessonName},'%') </if> " +
            " <if test= \" releaseState !=null and releaseState != ''\">and bl.release_state = #{releaseState} </if> " +
            " <if test= \" createByName != null and createByName != ''\">and suc.name like CONCAT('%',#{createByName},'%') </if> " +
            " <if test= \" courseId !=null and courseId != ''\">and bl.course_id = #{courseId} </if> " +
            " <if test= \" viewRange !=null and viewRange != ''\">and bl.view_range = #{viewRange} </if> " +
            " <if test= \" lessonType !=null and lessonType != ''\">and bl.lesson_type = #{lessonType} </if> " +
            " <if test= \" recommend !=null and recommend != ''\">and bl.recommend = #{recommend} </if> " +
            " <if test= \" lecturerName !=null and lecturerName != ''\">and bl.lesson_name like CONCAT('%',#{lecturerName},'%')  </if> " +
            " order by bl.release_date desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryAllBLesson(FindAllLessonDTO param);

    /**
     * 按学习人数降序查询
     *
     * @param
     * @return
     */
    @Select(" <script> select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{curUser.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{curUser.corpId} "+

            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{curUser.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
//            " left join b_course bc on bc.business_id = bl.course_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{curUser.corpId} "+

            " and bl.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{curUser.userId}) " +
            "      or (bl.lecturer_id = #{curUser.userId}) " +
            "    <if test=\"departs != null and departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " and bl.person_number >= (select `value` from b_dictionary where type = 'ZXRS' and corp_id = #{curUser.corpId}) + 0 " + //ZXRS 在学人数
            " order by CAST(bl.person_number AS UNSIGNED) desc,bl.release_date desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryPersonMore(@Param("curUser") CurUser curUser, @Param("departs") List<String> departs);

    /**
     * 按学习人数降序查询推荐
     *
     * @param
     * @return
     */
    @Select(" <script> select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{curUser.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{curUser.corpId} "+

            " left join b_recommend br on bl.business_id = br.lesson_id " +
            " and br.del_flag != 1 and br.flag = 1  " +

            " and br.corp_id = #{curUser.corpId} "+

            " left join b_recommend_son brs on br.business_id = brs.recommend_id " +
//            " left join b_course bc on bc.business_id = bl.course_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{curUser.corpId} "+

            " and bl.release_state = 2 " +
            " and ((brs.type = 0 and brs.target_id = #{curUser.userId}) " +
            "    <if test=\"departs != null and departs.size() > 0 \"> " +
            "      or (brs.type = 1 and brs.target_id in " +
            "      <foreach item='item' index='index' collection='departs' open='(' separator=',' close=')'> #{item} </foreach>) " +
            "    </if>) " +
            " order by br.update_date desc" +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryRecommend(@Param("curUser") CurUser curUser,@Param("departs") List<String> departs);

    /**
     * 按学习人数降序查询
     *
     * @param
     * @return
     */
    @Select(" <script> select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{curUser.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +


            " and suc.corp_id = #{curUser.corpId} "+

            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{curUser.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
//            " left join b_course bc on bc.business_id = bl.course_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{curUser.corpId} "+

            " and bl.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{curUser.userId}) " +
            "      or (bl.lecturer_id = #{curUser.userId}) " +
            "    <if test=\"departs != null and departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " and bl.collect_number >= (select `value` from b_dictionary where type = 'GZRS' and corp_id = #{curUser.corpId}) + 0 " + //GZRS 关注人数
            " order by CAST(bl.collect_number AS UNSIGNED) desc,bl.release_date desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryConcern(@Param("curUser") CurUser curUser, @Param("departs") List<String> departs);


    @Select(" <script> SELECT DISTINCT b.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " FROM " +
            " b_lesson b " +
            " LEFT JOIN b_lesson_label bl ON b.business_id = bl.lesson_id" +
            " inner JOIN ( " +
            "   SELECT " +
            "   bla.business_id AS bussinessId, " +
            "   count( 1 ) AS bsum  " +
            "   FROM " +
            "   b_label bla " +
            "   LEFT JOIN b_lesson_label bl ON bla.business_id = bl.label_id " +
            "   LEFT JOIN b_lesson b ON bl.lesson_id = b.business_id " +
            "   LEFT JOIN b_lesson_person bp ON b.business_id = bp.lesson_id " +
            "   WHERE " +
            "   bp.person_id = #{curUser.userId} " +
            "   and b.del_flag != 1 and b.flag = 1  " +
            "   and bp.del_flag != 1 and bp.flag = 1  " +

            "   and b.corp_id = #{curUser.corpId} "+
//            "   and bp.corp_id = #{curUser.corpId} "+

            "   GROUP BY " +
            "   bla.business_id " +
            " ) T ON T.bussinessId = bl.label_id " +
            " left join s_user su on su.business_id = b.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{curUser.corpId} "+

            " left join s_user suc on suc.business_id = b.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{curUser.corpId} "+

            " left join b_viewrange bv on b.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{curUser.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " where b.del_flag != 1 and b.flag = 1 " +

            " and b.corp_id = #{curUser.corpId} "+

            " and b.business_id not in (SELECT DISTINCT b.business_id from " +
            " b_lesson b left join b_lesson_person  blp on b.business_id = blp.lesson_id " +
            " where blp.del_flag != 1 and blp.flag = 1  " +
            " and blp.person_id = #{curUser.userId} )  " +
            " and b.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{curUser.userId}) " +
            "      or (b.lecturer_id = #{curUser.userId}) " +
            "    <if test=\"departs != null and departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " order by  CAST(T.bsum AS UNSIGNED) desc, CAST( b.hot_number AS UNSIGNED) desc, b.release_date desc" +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryInterested(@Param("curUser") CurUser curUser, @Param("departs") List<String> departs);

    //查询课程中的标签
    @Select(" <script> select blb.business_id businessId,blb.l_name lName" +
            " from b_lesson_label bll " +
            " left join b_label blb on bll.label_id = blb.business_id " +
            " left join b_lesson bls on bls.business_id = bll.lesson_id " +
            " where  blb.del_flag != 1 and blb.flag = 1  " +
            " and bls.del_flag != 1 and bls.flag = 1  " +
            " and bls.business_id = #{lessonId} " +
            " order by CAST( bll.sort AS UNSIGNED) asc " +
            "</script>")
    List<QueryLabelResDTO> queryLabelByLessonId(String lessonId);

    /**
     * 首页分页查询全部课程
     * @param firstPageQueryDTO
     * @param courseIds
     * @return
     */
    @Select(" <script> select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{param.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{param.corpId} "+

            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{param.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{param.corpId} "+

            " and bl.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{param.userId}) " +
            "      or (bl.lecturer_id = #{param.userId}) " +
            "    <if test=\"param.departs != null and param.departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='param.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " <if test= \" param.viewParam !=null and param.viewParam != ''\">and (bl.lesson_name like CONCAT('%',#{param.viewParam},'%') " +
            "   or su.name like CONCAT('%',#{param.viewParam},'%'))</if> " +
            " <if test= \" courseIds !=null and courseIds.size() > 0 \">and course_id in " +
            "   <foreach item='item' index='index' collection='courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
            " </if> " +
            " order by  CAST( bl.person_number AS UNSIGNED) desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryHomeBLesson(@Param("param") FirstPageQueryDTO firstPageQueryDTO,@Param("courseIds") List<String> courseIds);

    /**
     * 根据课程id关闭课程
     *
     * @param model
     * @return
     */
    @Update({"<script>",
            "update b_lesson",
            "  <set>",
            "    <if test='updateBy != null'>update_by=#{updateBy},</if>",
            "    <if test='updateDate != null'>update_date=#{updateDate},</if>",
            "    <if test='closeDate != null'>close_date=#{closeDate},</if>",
            "   release_state = '4'",
            "  </set>",
            "where business_id=#{businessId}",
            "</script>"})
    int closeLesson(BLesson model);

    /**
     * 查询课程状态
     * @param id
     * @return
     */
    @Select(" <script> select release_state from b_lesson where business_id = #{id} </script>")
    int findLessonReleaseState(String id);

    /**
     * 根据课程id发布课程
     *
     * @param model
     * @return
     */
    @Update({"<script>",
            "update b_lesson",
            "  <set>",
            "    <if test='updateBy != null'>update_by=#{updateBy},</if>",
            "    <if test='updateDate != null'>update_date=#{updateDate},</if>",
            "    <if test='releaseDate != null'>release_date=#{releaseDate},</if>",
            "   release_state = '2'",
            "  </set>",
            "where business_id=#{businessId}",
            "</script>"})
    int releaseLesson(BLesson model);

    /**
     * 首页分页查询全部课程
     * @param param
     * @return
     */
    @Select(" <script> select bl.* " +
//            " ,su.name lecturerName " +
//            " ,suc.name createByName " +
            " from b_lesson bl " +
//            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " left join s_user suc on suc.business_id = bl.create_by " +
//            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
//            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
//            " left join b_course bc on bc.business_id = bl.course_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +
//            " and su.del_flag != 1 and su.flag = 1  " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +
//            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bl.corp_id = #{param.corpId} "+
//            " and su.corp_id = #{param.corpId} "+
//            " and suc.corp_id = #{param.corpId} "+
//            " and bv.corp_id = #{param.corpId} "+

            " and bl.release_state = 2 " +
//            " and ((bvs.type = 0 and bvs.target_id = #{param.userId}) " +
//            "    <if test=\"param.departs != null and param.departs.size() > 0 \"> " +
//            "      or (bvs.type = 1 and bvs.target_id in " +
//            "      <foreach item='item' index='index' collection='param.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
//            "    </if>) " +
            " <if test= \" param.lessonName !=null and param.lessonName != ''\">and lesson_name like CONCAT('%',#{param.lessonName},'%') " +
//            "   or su.name like CONCAT('%',#{param.viewParam},'%'))" +
            " </if> " +
            " <if test= \" param.lessonType !=null and param.lessonType != ''\">and lesson_type = #{param.lessonType} " +
            " </if> " +
//            " <if test= \" courseIds !=null and courseIds.size() > 0 \">and course_id in " +
//            "   <foreach item='item' index='index' collection='courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
//            " </if> " +
            " order by bl.release_date desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> findLessonAtBanner(@Param("param") LessonTrainAtBannerDTO param);

    /**
     * 查询我发布和我学习课程数量
     * @param curUser
     * @return
     */
    @Select({"<script>",
            " select (select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 0 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state is not null) publishAllCount, " +
                    "(select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 0 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state = '0') notPublishCount, " +
                    "(select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 0 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state = '2') publishedCount, " +
                    "(select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 0 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state = '4') closedCount, "+

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 0 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId} and blp.is_finish is not null ) learnAllCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 0 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId}  and blp.is_finish = '2') finishCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 0 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId} and blp.is_finish = '1') notFinishCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 0 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId}  and blp.is_finish = '0') notStartCount " +
            "</script>"})
    ILessonCountDTO iLessonCount(@Param("curUser") CurUser curUser);

    /**
     * 查询我教学和我学习培训数量
     * @param curUser
     * @return
     */
    @Select({"<script>",
            " select (select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 1 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state is not null) publishAllCount, " +
                    "(select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 1 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state = '0') notPublishCount, " +
                    "(select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 1 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state = '2') publishedCount, " +
                    "(select COUNT(1)  from b_lesson where del_flag != 1 and flag = 1 " +
                    " and lesson_type = 1 " +
                    " and corp_id = #{curUser.corpId} " +
                    " and lecturer_id = #{curUser.userId} and release_state = '4') closedCount, "+

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId} and blp.train_is_sign is not null ) learnAllCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId}  and blp.train_is_sign = '2') finishCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId} and blp.train_is_sign = '1') notFinishCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and bl.corp_id = #{curUser.corpId} " +
                    " and blp.person_id = #{curUser.userId}  and blp.train_is_sign = '0') notStartCount " +
            "</script>"})
    ILessonCountDTO iTrainCount(@Param("curUser") CurUser curUser);

    /**
     * 查询我教学和我学习培训数量
     * @param lessonId
     * @return
     */
    @Select({"<script>",
            " select "+
                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " left join s_user su on blp.person_id = su.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
//                    " and su.del_flag != 1 and su.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and blp.lesson_id = #{lessonId} and blp.train_is_sign is not null ) learnAllCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " left join s_user su on blp.person_id = su.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
//                    " and su.del_flag != 1 and su.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and blp.lesson_id = #{lessonId}  and blp.train_is_sign = '2') finishCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " left join s_user su on blp.person_id = su.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
//                    " and su.del_flag != 1 and su.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and blp.lesson_id = #{lessonId} and blp.train_is_sign = '1') notFinishCount, " +

                    "(select COUNT(1)  from b_lesson_person blp " +
                    " LEFT join b_lesson bl on blp.lesson_id = bl.business_id " +
                    " left join s_user su on blp.person_id = su.business_id " +
                    " where blp.del_flag != 1 and blp.flag = 1 " +
                    " and bl.del_flag != 1 and bl.flag = 1 " +
//                    " and su.del_flag != 1 and su.flag = 1 " +
                    " and bl.lesson_type = 1 " +
                    " and blp.lesson_id = #{lessonId}  and blp.train_is_sign = '0') notStartCount " +
                    "</script>"})
    ILessonCountDTO trainPersonCount(String lessonId);

    /**
     * 重新统计在学人数
     * @param lessonId
     * @return
     */
    @Update("<script>update b_lesson a set a.person_number = (" +
            "select b.bc from (select count(1) bc from b_lesson_person blp " +
            " left join b_lesson bl on blp.lesson_id = bl.business_id " +
            " where 1=1 " +
            " and blp.del_flag != 1 and blp.flag = 1 " +
            " and bl.lesson_type = #{type}" +
            " <if test=\"type!=null and type != '' and type == 0\">and blp.is_finish != 0 </if>" +
            " and blp.lesson_id = #{lessonId}) as b )" +
            " where business_id = #{lessonId} </script>" )
    int personNumberReCount(@Param("lessonId") String lessonId, @Param("type") String lessonType);

    /**
     * 在学人数
     * @param lessonId
     * @return
     */
    @Select("<script>select count(1) from b_lesson_person where 1=1 " +
            " and del_flag != 1 and flag = 1 " +
            " and lesson_id = #{lessonId} </script>" )
    int personNumberCount(String lessonId);
    /**
     * 在学人数
     * @param lessonId
     * @return
     */
    @Select("<script>select count(1) from b_lesson_person where 1=1 " +
            " and del_flag != 1 and flag = 1 " +
            " and lesson_id = #{lessonId} </script>" )
    String lessonPersonCount(String lessonId);

    /**
     * 重新统计收藏人数
     * @param lessonId
     * @return
     */
    @Update("update b_lesson set collect_number = " +
            "(select count(1) from b_collect where 1=1  " +
            "and lesson_id = #{lessonId}) " +
            "where business_id = #{lessonId}" )
    int collectNumberReCount(String lessonId);
    /**
     * 重新统计热度
     * @param lessonId
     * @return
     */
    @Update("<script>update b_lesson set hot_number = " +
            "(select count(1) from b_hot where 1=1 " +
            " and lesson_id = #{lessonId}) where 1=1 " +
            " and business_id = #{lessonId} " +
            " </script>" )
    int hotNumberReCount(String lessonId);

    /**
     * 列表页分页查询全部课程
     * @param model
     * @return
     */
    @Select(" <script>  select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{model.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{model.corpId} "+

            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{model.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " left join b_lesson_label bll on bl.business_id = bll.lesson_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{model.corpId} "+

            " <if test=\"model.labelId !=null and model.labelId != ''\">and bll.label_id = #{model.labelId}</if>" +
            " <if test=\"model.lessonType !=null and model.lessonType != ''\">and bl.lesson_type = #{model.lessonType}</if>" +
            " and bl.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{model.userId}) " +
            "      or (bl.lecturer_id = #{model.userId}) " +
            "    <if test=\"model.departs != null and model.departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='model.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " <if test= \" model.viewParam !=null and model.viewParam != ''\">and (bl.lesson_name like CONCAT('%',#{model.viewParam},'%') " +
            "   or su.name like CONCAT('%',#{model.viewParam},'%'))</if> " +
            " <if test= \" model.courseIds !=null and model.courseIds.size() > 0 \">and course_id in " +
            "   <foreach item='item' index='index' collection='model.courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
            " </if> " +
            " <if test=\"model.releaseDateSort !=null and model.releaseDateSort !='' and model.releaseDateSort =='0'.toString()\">  ORDER BY bl.release_date DESC  </if> "+
            " <if test=\"model.releaseDateSort !=null and model.releaseDateSort !='' and model.releaseDateSort =='1'.toString()\">  ORDER BY bl.release_date ASC "+
            " </if> "+
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryListAllLessonByPagination(@Param("model") FindListLessonDTO model);

    /**
     * 列表页分页查询推荐课程
     * @param model
     * @return
     */
    @Select(" <script>  select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{model.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{model.corpId} "+

            " left join b_recommend br on bl.business_id = br.lesson_id " +
            " and br.del_flag != 1 and br.flag = 1  " +

            " and br.corp_id = #{model.corpId} "+

            " left join b_recommend_son brs on br.business_id = brs.recommend_id " +
            " left join b_lesson_label bll on bl.business_id = bll.lesson_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{model.corpId} "+

            " <if test=\"model.labelId !=null and model.labelId != ''\">and bll.label_id = #{model.labelId}</if>" +
            " <if test=\"model.lessonType !=null and model.lessonType != ''\">and bl.lesson_type = #{model.lessonType}</if>" +
            " and bl.release_state = 2 " +
            " and ((brs.type = 0 and brs.target_id = #{model.userId}) " +
            "      or (brs.type = 1 and brs.target_id in " +
            "    <if test=\"model.departs != null and model.departs.size() > 0 \"> " +
            "      <foreach item='item' index='index' collection='model.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if>) " +
            " <if test= \" model.viewParam !=null and model.viewParam != ''\">and (bl.lesson_name like CONCAT('%',#{model.viewParam},'%') " +
            "   or su.name like CONCAT('%',#{model.viewParam},'%'))</if> " +
            " <if test= \" model.courseIds !=null and model.courseIds.size() > 0 \">and course_id in " +
            "   <foreach item='item' index='index' collection='model.courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
            " </if> " +
            " order by br.update_date desc" +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryRecommendListByPagination(@Param("model") FindListLessonDTO model);



    /**
     * 列表页分页查询在学课程
     * @param model
     * @return
     */
    @Select(" <script> select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{model.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{model.corpId} "+

            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{model.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " left join b_lesson_label bll on bl.business_id = bll.lesson_id " +
//            " left join b_course bc on bc.business_id = bl.course_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{model.corpId} "+

            " <if test=\"model.labelId !=null and model.labelId != ''\">and bll.label_id = #{model.labelId}</if>" +
            " <if test=\"model.lessonType !=null and model.lessonType != ''\">and bl.lesson_type = #{model.lessonType}</if>" +
            " and bl.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{model.userId}) " +
            "      or (bl.lecturer_id = #{model.userId}) " +
            "    <if test=\"model.departs != null and model.departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='model.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " and bl.person_number >= (select `value` from b_dictionary where type = 'ZXRS' and corp_id = #{model.corpId}) + 0 " + //ZXRS 在学人数
            " <if test= \" model.viewParam !=null and model.viewParam != ''\">and (bl.lesson_name like CONCAT('%',#{model.viewParam},'%') " +
            "   or su.name like CONCAT('%',#{model.viewParam},'%'))</if> " +
            " <if test= \" model.courseIds !=null and model.courseIds.size() > 0 \">and course_id in " +
            "   <foreach item='item' index='index' collection='model.courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
            " </if> " +
            " order by CAST( bl.person_number AS UNSIGNED) desc,bl.release_date desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryLearnListByPagination(@Param("model") FindListLessonDTO model);

    /**
     * 列表页分页查询收藏课程
     * @param model
     * @return
     */
    @Select(" <script>  select distinct bl.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " from  b_lesson bl " +
            " left join s_user su on su.business_id = bl.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{model.corpId} "+

            " left join s_user suc on suc.business_id = bl.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{model.corpId} "+

            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{model.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " left join b_lesson_label bll on bl.business_id = bll.lesson_id " +
            " where bl.del_flag != 1 and bl.flag = 1 " +

            " and bl.corp_id = #{model.corpId} "+

            " <if test=\"model.labelId !=null and model.labelId != ''\">and bll.label_id = #{model.labelId}</if>" +
            " <if test=\"model.lessonType !=null and model.lessonType != ''\">and bl.lesson_type = #{model.lessonType}</if>" +
            " and bl.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{model.userId}) " +
            "      or (bl.lecturer_id = #{model.userId}) " +
            "    <if test=\"model.departs != null and model.departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='model.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " and bl.collect_number >= (select `value` from b_dictionary where type = 'GZRS' and corp_id = #{model.corpId}) + 0 " + //GZRS 关注人数
            " <if test= \" model.viewParam !=null and model.viewParam != ''\">and (bl.lesson_name like CONCAT('%',#{model.viewParam},'%') " +
            "   or su.name like CONCAT('%',#{model.viewParam},'%'))</if> " +
            " <if test= \" model.courseIds !=null and model.courseIds.size() > 0 \">and course_id in " +
            "   <foreach item='item' index='index' collection='model.courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
            " </if> " +
            " order by CAST( bl.collect_number AS UNSIGNED) desc,bl.release_date desc " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryCollectListByPagination(@Param("model") FindListLessonDTO model);


    @Select(" <script> SELECT DISTINCT b.* " +
            " ,su.name lecturerName " +
            " ,suc.name createByName " +
            " FROM " +
            " b_lesson b " +
            " LEFT JOIN b_lesson_label bl ON b.business_id = bl.lesson_id" +
            " inner JOIN ( " +
            "   SELECT " +
            "   bla.business_id AS bussinessId, " +
            "   count( 1 ) AS bsum  " +
            "   FROM " +
            "   b_label bla " +
            "   LEFT JOIN b_lesson_label bl ON bla.business_id = bl.label_id " +
            "   LEFT JOIN b_lesson b ON bl.lesson_id = b.business_id " +
            "   LEFT JOIN b_lesson_person bp ON b.business_id = bp.lesson_id " +
            "   WHERE " +
            "   bp.person_id = #{model.userId} " +
            "   and b.del_flag != 1 and b.flag = 1  " +
            "   and bp.del_flag != 1 and bp.flag = 1  " +

            "   and b.corp_id = #{model.corpId} "+

            "   GROUP BY " +
            "   bla.business_id " +
            " ) T ON T.bussinessId = bl.label_id " +
            " left join s_user su on su.business_id = b.lecturer_id " +
//            " and su.del_flag != 1 and su.flag = 1  " +

            " and su.corp_id = #{model.corpId} "+

            " left join s_user suc on suc.business_id = b.create_by " +
//            " and suc.del_flag != 1 and suc.flag = 1  " +

            " and suc.corp_id = #{model.corpId} "+

            " left join b_viewrange bv on b.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{model.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " left join b_lesson_label bll on bl.business_id = bll.lesson_id " +
            " where b.del_flag != 1 and b.flag = 1 " +

            " and b.corp_id = #{model.corpId} "+

            " and b.business_id not in (SELECT DISTINCT b.business_id from " +
            " b_lesson b left join b_lesson_person  blp on b.business_id = blp.lesson_id " +
            " where blp.del_flag != 1 and blp.flag = 1  " +
            " and blp.person_id = #{model.userId} )  " +
            " <if test=\"model.labelId !=null and model.labelId != ''\">and bll.label_id = #{model.labelId}</if>" +
            " <if test=\"model.lessonType !=null and model.lessonType != ''\">and b.lesson_type = #{model.lessonType}</if>" +
            " and b.release_state = 2 " +
            " and ((bvs.type = 0 and bvs.target_id = #{model.userId}) " +
            "      or (b.lecturer_id = #{model.userId}) " +
            "    <if test=\"model.departs != null and model.departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='model.departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL))) " +
            " <if test= \" model.viewParam !=null and model.viewParam != ''\">and (bl.lesson_name like CONCAT('%',#{model.viewParam},'%') " +
            "   or su.name like CONCAT('%',#{model.viewParam},'%'))</if> " +
            " <if test= \" model.courseIds !=null and model.courseIds.size() > 0 \">and course_id in " +
            "   <foreach item='item' index='index' collection='model.courseIds' open='(' separator=',' close=')'> #{item} </foreach> " +
            " </if> " +
            " order by  CAST( T.bsum AS UNSIGNED) desc,  CAST( b.collect_number AS UNSIGNED) desc" +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> queryInterestedListByPagination(@Param("model") FindListLessonDTO model);

    //查询管理的全部课程
    //isAdmin  <if test= " ！isAdmin ">
    //                teacher in (uid)
    //         </IF>
    @Select("<script>" +
            "SELECT " +
            "les.*,  " +
            "CONCAT( " +
            "IFNULL(CONCAT(b3.c_name,\"•\"),\"\"), " +
            "IFNULL(CONCAT(b2.c_name,\"•\"),\"\"), " +
            "  IFNULL(b1.c_name,\"\") " +
            ") AS courseName ," +
            "u1.name AS createByName " +
            "FROM " +
            "b_lesson les " +
            "LEFT JOIN b_course b1 ON b1.business_id = les.course_id " +
            "AND b1.corp_id = #{corpId} " +
            "and b1.del_flag != 1 and b1.flag = 1 "+
            "LEFT JOIN b_course b2 ON b2.business_id = b1.pid " +
            "AND b2.corp_id = #{corpId} " +
            "and b2.del_flag != 1 and b2.flag = 1 "+
            "LEFT JOIN b_course b3 ON b3.business_id = b2.pid " +
            "AND b3.corp_id = #{corpId} " +
            "and b3.del_flag != 1 and b3.flag = 1 "+
            "LEFT JOIN s_user u1 ON u1.business_id = les.create_by " +
            "AND u1.corp_id = #{corpId}" +
            "and u1.del_flag != 1 and u1.flag = 1 "+
            "WHERE 1=1 " +
            "and les.del_flag != 1 and les.flag = 1 "+
            "AND les.corp_id = #{corpId}" +
            "<if test='!isAdmin'>" +
            "AND les.create_by IN" +
            "<foreach item='item' index='index' collection='CurUserSet' open='(' close=')' separator=','> " +
            "#{item} " +
            "</foreach>" +
            "</if> " +
            "<if test=\"bLesson.lessonName!=null and bLesson.lessonName!=''\">AND les.lesson_name like '%${bLesson.lessonName}%' </if>" +
            "<if test=\"bLesson.lessonType!=null and bLesson.lessonType!=''\">AND les.lesson_type  = #{bLesson.lessonType} </if>" +
            "<if test=\"bLesson.lecturerId!=null and bLesson.lecturerId!=''\">AND u1.name like '%${bLesson.lecturerId}%' </if>" +
            "<if test=\"bLesson.releaseState!=null and bLesson.releaseState!=''\">AND les.release_state = #{bLesson.releaseState} </if>" +
            "</script>")
    @ResultMap(value = "AllManageResultMap")
    List<BLesson> selectAllManageLesson(@Param("CurUserSet") Set<String> CurUserSet, @Param("bLesson") BLesson bLesson, @Param("isAdmin") boolean isAdmin, @Param("corpId") String corpId) ;


    @Select("<script>" +
            "SELECT * FROM `b_recommend` where type = 0   and corp_id = #{corpId}  and lesson_id = #{lessonId} " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<BRecommend> queryRecommendByLessonId(@Param("corpId") String corpId, @Param("lessonId")String lessonId);


    @Select("<script>select distinct bl.*, " +
            " blp.is_apply isApply,blp.is_appoint isAppoint,blp.train_is_sign trainIsSign,blp.sign_date signDate,blp.business_id lessonPersonId, " +
            "CONCAT( " +
            "IFNULL(CONCAT(b3.c_name,\" / \"),\"\"), " +
            "IFNULL(CONCAT(b2.c_name,\" / \"),\"\"), " +
            "  IFNULL(b1.c_name,\"\") " +
            ") AS courseName, " +
            " su.name lecturerName " +
            " from b_lesson bl left join b_lesson_person blp  on bl.business_id = blp.lesson_id " +
            " and blp.del_flag = 0 and blp.flag = 1 " +
            " and blp.person_id = #{curUser.userId} " +
            " left join s_user su on bl.lecturer_id = su.business_id " +
            " left join b_course b1 on b1.business_id = bl.course_id " +
            "and b1.del_flag != 1 and b1.flag = 1 "+
            "LEFT JOIN b_course b2 ON b2.business_id = b1.pid " +
            "and b2.del_flag != 1 and b2.flag = 1 "+
            "LEFT JOIN b_course b3 ON b3.business_id = b2.pid " +
            "and b3.del_flag != 1 and b3.flag = 1 "+
            " where 1=1 " +
            "and bl.del_flag = 0 and bl.flag = 1 " +
            "and bl.business_id = #{lessonId}</script>")
    @ResultMap(value = "TrainBaseResultMap")
    BLesson findOneMyTrain(@Param("curUser") CurUser curUser, @Param("lessonId") String lessonId);

    @Select("<script>select distinct bl.*," +
            "CONCAT( " +
            "IFNULL(CONCAT(b3.c_name,\" / \"),\"\"), " +
            "IFNULL(CONCAT(b2.c_name,\" / \"),\"\"), " +
            "  IFNULL(b1.c_name,\"\") " +
            ") AS courseName, " +
            " su.name lecturerName " +
            " from b_lesson bl  " +
            " left join s_user su on bl.lecturer_id = su.business_id " +
            " left join b_course b1 on b1.business_id = bl.course_id " +
            "and b1.del_flag != 1 and b1.flag = 1 "+
            "LEFT JOIN b_course b2 ON b2.business_id = b1.pid " +
            "and b2.del_flag != 1 and b2.flag = 1 "+
            "LEFT JOIN b_course b3 ON b3.business_id = b2.pid " +
            "and b3.del_flag != 1 and b3.flag = 1 "+
            " where 1=1 " +
            "and bl.del_flag = 0 and bl.flag = 1 " +
            "and bl.business_id = #{lessonId} </script>")
    @ResultMap(value = "TrainBaseResultMap")
    BLesson findOneMyTeachTrain(@Param("curUser") CurUser curUser, @Param("lessonId") String lessonId);


    @Select("<script>" +
            " select count(1) from b_lesson_person " +
            " where 1=1 " +
            " and del_flag = 0 and flag = 1 " +
            " and lesson_id = #{lessonId} " +
            "</script>")
    int trainApplyCount(String lessonId);


    /**
     * 分页查询我收藏
     */
    @Select("<script>select bl.*,su.name lecturerName, suc.name createByName," +
            "  if(bl.lecturer_id = #{curUser.userId}, '1', '0') userIsLecturer " +
            " from b_lesson bl " +
            "left join b_collect bc on bl.business_id = bc.lesson_id " +
            "left join s_user su on bl.lecturer_id = su.business_id " +
            "left join s_user suc on bl.create_by = suc.business_id " +
            "where 1=1 " +
            "and bl.del_flag = 0 and bl.flag = 1 " +
            "and bl.corp_id = #{curUser.corpId} " +
            " <if test=\"lessonType!=null and lessonType != ''\">and bl.lesson_type = #{lessonType}</if>" +
            "AND bc.person_id = #{curUser.userId} </script>")
    @ResultMap(value = "BaseResultMap")
    List<BLesson> selectMyCollect(@Param("lessonType") String lessonType, @Param("curUser")CurUser curUser);

    /**
     * 查询所有
     */
    @Select("<script>select business_id from b_lesson " +
            "where 1=1 " +
            "and del_flag = 0 and flag = 1 " +
            "and lesson_type = 1 " +
            "AND #{nowDate} between train_over_date and SUBDATE(train_over_date,interval -#{sec} second) </script>")
    List<String> selectByTrainNowBeforeOver(@Param("nowDate")Date newDate ,@Param("sec")String sec);

    @Select("select * from b_lesson where business_id = #{id}")
    @ResultMap(value = "BaseResultMap")
    BLesson getLessonState(@Param("id") String id);

    /**
     * 判断该人是否在该课程可见范围内
     * @return
     */
    @Select("<script>select count(1) from b_lesson bl " +
            " left join b_viewrange bv on bl.business_id = bv.lesson_id " +
            " and bv.del_flag != 1 and bv.flag = 1  " +

            " and bv.corp_id = #{curUser.corpId} "+

            " left join b_viewrange_son bvs on bv.business_id = bvs.viewrange_id " +
            " where 1=1 " +
            " and bl.business_id = #{lessonId} "+
            " and ((bvs.type = 0 and bvs.target_id = #{curUser.userId}) " +
            "      or (bl.lecturer_id = #{curUser.userId}) " +
            "    <if test=\"departs != null and departs.size() > 0 \"> " +
            "      or (bvs.type = 1 and bvs.target_id in " +
            "      <foreach item='item' index='index' collection='departs' open='(' separator=',' close=')'> #{item} </foreach>)" +
            "    </if> " +
            "  or (bl.business_id not in (select lesson_id from b_viewrange where del_flag != 1 and flag = 1 AND lesson_id IS NOT NULL ))) " +
            " </script>")
    int isInViewRange(@Param("curUser") CurUser curUser, @Param("lessonId") String lessonId,  @Param("departs") List<String> departs);

}

