package org.rcisoft.business.blesson.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;
import org.rcisoft.business.bfile.entity.BTrainFile;
import org.rcisoft.business.blabel.dto.QueryLabelResDTO;
import org.rcisoft.business.blesson.dto.BViewRangeSonDTO;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.entity.IdEntity;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotBlank;
import java.util.Date;
import java.util.List;

/**
 * Created with CodeGenerator on .
 */
@Entity
@Table(name = "b_lesson")
@Data
@NoArgsConstructor
@AllArgsConstructor
public class BLesson extends IdEntity<BLesson> {
    private static final long serialVersionUID = -4327021174023243830L;


//    @Transient
//    @Autowired
//    private Global global;

    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    @ApiModelProperty(value = "课程编号")
    private String code;

    @NotBlank
    @ApiModelProperty(value = "课程名称")
    @Length(min = 1,max = 200,message = "长度最小为1,最大为150")
    private String lessonName;

    @ApiModelProperty(value = "*封面图片url")
    private String defaultUrl;

    @ApiModelProperty(value = "课时")
//    @Length(min = 1,max = 8,message = "长度最小为1,最大为5")
    private String classHour;

    @ApiModelProperty(value = "讲师id")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String lecturerId;

    @ApiModelProperty(value = "发布状态（0：待发布 1：审核中 2：已发布 3：已驳回 4：已关闭）")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String releaseState;

    @ApiModelProperty(value = "课程分类")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String courseId;

    @ApiModelProperty(value = "选课人数")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String personNumber;

    @ApiModelProperty(value = "课程时长（当课时为1时显示）")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String courseTime;

    @ApiModelProperty(value = "课程描述")
    @Length(min = 1,max = 255,message = "长度最小为1,最大为200")
    private String courseDescription;

    @ApiModelProperty(value = "可见范围")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String viewRange;

    @ApiModelProperty(value = "发布时间")
    //@Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private Date releaseDate;

    @ApiModelProperty(value = "关闭时间")
    //@Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private Date closeDate;

    @ApiModelProperty(value = "课程类型（0：课程 1：培训）")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String lessonType;

    @ApiModelProperty(value = "评论数")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String discussNumber;

    @ApiModelProperty(value = "推荐（0：不推荐 1：推荐）")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String recommend;

    @ApiModelProperty(value = "关注数")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String collectNumber;

    @ApiModelProperty(value = "热度")
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String hotNumber;

    @ApiModelProperty(value = "消耗积分")
    private Integer valueConsume;

    @ApiModelProperty(value = "获得积分")
    private Integer valueGain;

    @ApiModelProperty(value = "更新积分日期")
    private Date valueUpdateDate;

//    @ApiModelProperty(value = "积分")
//    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
//    private String value;

    //---------------培训------------------
    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "报名开始时间")
    private Date applyStartDate;

    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "报名结束时间")
    private Date applyOverDate;

    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "培训开始时间")
    private Date trainStartDate;

    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "培训结束时间")
    private Date trainOverDate;

    @ApiModelProperty(value = "最大报名人数")
    private String maxApplyPerson;

    @ApiModelProperty(value = "培训地址")
    @Length(min = 1,max = 255,message = "长度最小为1,最大为255")
    private String trainAddress;

    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "培训签到时间（培训开始前 分钟数）")
    @Length(min = 1,max = 15,message = "长度最小为1,最大为15")
    private String trainSignTime;

    @ApiModelProperty(value = "二维码")
    @Length(min = 1,max = 255,message = "长度最小为1,最大为255")
    private String qrCode;

    @ApiModelProperty(value = "文件集合")
    @Transient
    private List<BTrainFile> bTrainFileList;

    @ApiModelProperty(value = "时间状态 0 未开始签到 1 已开始签到 2超过签到时间")
    @Transient
    private String trainDateState;


    //-------------------------------------

    @ApiModelProperty(value = "学习进度")
    @Transient
    private String learnProgress;

    @ApiModelProperty(value = "讲师姓名")
    @Transient
    private String lecturerName;

    @ApiModelProperty(value = "接收标签集合 用‘,’分隔")
    @Transient
    private String labels;

    @ApiModelProperty(value = "创建人姓名")
    @Transient
    private String createByName;

    @ApiModelProperty(value = "返回标签集合")
    @Transient
    private List<QueryLabelResDTO> labelList;

    @ApiModelProperty(value = "分类名")
    @Transient
    private String courseName;

    @ApiModelProperty(value = "可见范围（人）多人之间用','隔开")
    @Transient
    private String viewRangePerson;

    @ApiModelProperty(value = "可见范围（人）返回集合")
    @Transient
    private List<BViewRangeSonDTO> viewRangePersonList;

    @ApiModelProperty(value = "可见范围（部门）多人之间用','隔开")
    @Transient
    private String viewRangeDepart;

    @ApiModelProperty(value = "可见范围（部门）返回集合")
    @Transient
    private List<BViewRangeSonDTO> viewRangeDepartList;



    @ApiModelProperty(value = "是否报名（0未报名 1已报名）")
    @Transient
    private String isApply;

    @ApiModelProperty(value = "知否必修 0 选修 1必修")
    @Transient
    private String isAppoint;

    @ApiModelProperty(value = "是否签到")
    @Transient
    private String trainIsSign;

    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "签到日期")
    @Transient
    private String signDate;

    @ApiModelProperty(value = "是否收藏")
    @Transient
    private String isCollect;

    @ApiModelProperty(value = "报名是否已满(1 已满 2未满)")
    @Transient
    private String applyIsFull;

    @ApiModelProperty(value = "当前是否为报名时间0 未到报名时间 1已到报名时间 2已过报名时间 ")
    @Transient
    private String isApplyDate;

    @DateTimeFormat(pattern="yyyy/MM/dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy/MM/dd HH:mm:ss")
    @ApiModelProperty(value = "报名时间")
    @Transient
    private Date applyDate;

    @ApiModelProperty(value = "讲师部门")
    @Transient
    private String lecturerDeptName;

    @ApiModelProperty(value = "登录人是否为讲师0 不是 1 是")
    @Transient
    private String userIsLecturer;

    @ApiModelProperty(value = "选课主键businessId")
    @Transient
    private String lessonPersonId;


    public void initModel(){
//        this.setDefaultUrl(global.getDEFAULT_COURSE_LOCATION());
        UserUtil.setCurrentPersistOperation(this);
        //设置状态为待发布
        this.setReleaseState("0");
        //设置讲师为登录人
        this.setLecturerId(this.getCreateBy());
        //设置初始报名人数为0
        this.setPersonNumber("0");
        //设置初始评论人数为0
        this.setDiscussNumber("0");
        //设置初始课时数为0
        this.setClassHour("0");
        //设置热度为0
        this.setHotNumber("0");
        //设置初始不推荐
        this.setRecommend("0");
        //设置初始关注数为0
        this.setCollectNumber("0");
    }

    public void initTrainModel(){
//        this.setDefaultUrl(global.getDEFAULT_COURSE_LOCATION());
        UserUtil.setCurrentPersistOperation(this);
        //设置状态为待发布
        this.setReleaseState("0");
//        //设置讲师为登录人
//        this.setLecturerId(this.getCreateBy());
//        设置初始报名人数为0
        this.setPersonNumber("0");
        //设置初始评论人数为0
        this.setDiscussNumber("0");
//        //设置初始课时数为0
//        this.setClassHour("0");
        //设置热度为0
        this.setHotNumber("0");
        //设置初始不推荐
        this.setRecommend("0");
        //设置初始关注数为0
        this.setCollectNumber("0");
    }

}

