package org.rcisoft.business.blesson.entity;


import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import org.rcisoft.core.entity.IdEntity;
import org.rcisoft.core.util.UserUtil;

import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotBlank;
import java.util.Date;

@Entity
@Table(name = "b_lesson_person")
@Data
@NoArgsConstructor
@AllArgsConstructor
public class BLessonPerson extends IdEntity<BLessonPerson> {

    @ApiModelProperty(value = "学生id")
    @NotBlank
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String personId;

    @ApiModelProperty(value = "课程id")
    @NotBlank
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String lessonId;

    @ApiModelProperty(value = "学习进度")
    @NotBlank
    @Length(min = 1,max = 64,message = "长度最小为1,最大为50")
    private String learnProgress;

    @ApiModelProperty(value = "是否已学完（0：未开始 1：已开始 2： 已学完）")
    @NotBlank
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String isFinish;

    @ApiModelProperty(value = "是否收藏 0:未收藏 1:已收藏")
    @NotBlank
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String isCollect;


    @ApiModelProperty(value = "指派id")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String appointId;

    @ApiModelProperty(value = "章節id")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String chapterId;

    @ApiModelProperty(value = "是否为指派（0自选（选修） 1指派（必修））")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String isAppoint;

    @ApiModelProperty(value = "曾经学完过(0未学完 1已学完)")
    private String everFinished;
    //-----------培训---------------
    @ApiModelProperty(value = "培训是否报名（0 未报名 1已报名））")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String isApply;

    @ApiModelProperty(value = "培训状态（0 待参加 1 缺勤 2 签到 ）")
    @Length(min = 1,max = 1,message = "长度最小为1,最大为1")
    private String trainIsSign;

    @ApiModelProperty(value = "报名日期")
    private Date applyDate;

    @ApiModelProperty(value = "学完成时间")
    private Date finishDate;

    @ApiModelProperty(value = "签到日期")
    private Date signDate;
    //-----------------------------

    @ApiModelProperty(value = "课程名称")
    @Transient
    private String lessonName;

    @ApiModelProperty(value = "图片url")
    @Transient
    private String defaultUrl;

    @ApiModelProperty(value = "讲师id")
    @Transient
    private String lecturerId;

    @ApiModelProperty(value = "讲师姓名")
    @Transient
    private String lecturerName;

    @ApiModelProperty(value = "该课程人数")
    @Transient
    private String personNumber;

    @ApiModelProperty(value = "课时")
    @Transient
    private String classHour;

    @ApiModelProperty(value = "时长")
    @Transient
    private String courseTime;

    @ApiModelProperty(value = "分类")
    @Transient
    private String courseId;

    @ApiModelProperty(value = "评论数")
    @Transient
    private String discussNumber;

    @ApiModelProperty(value = "课程类型 0 课程 1 培训")
    @Transient
    private String lessonType;

    @ApiModelProperty(value = " 发布状态（0：待发布 1：审核中 2：已发布 3：已驳回 4：已关闭）")
    @Transient
    private String releaseState;

    @Transient
    private String corpId;

    @ApiModelProperty(value = "姓名")
    @Transient
    private String PersonName;

    @ApiModelProperty(value = "头像")
    @Transient
    private String headPic;

    @ApiModelProperty(value = "部门名")
    @Transient
    private String departName;

    @ApiModelProperty(value = "详细部门")
    @Transient
    private String fullDept;

    //初始化
    public void initModel(){
        UserUtil.setCurrentPersistOperation(this);
        //学习进度为0
        this.setLearnProgress("0%");
        //设置正在学
        this.setIsFinish("1");
        //设置不收藏
        this.setIsCollect("0");
        //设置不指派
        this.setIsAppoint("0");

        this.setRemarks("");
        //设置加入学习时间为当前时间
        this.setApplyDate(new Date());
    }
    //初始化
    public void initTrainModel(){
        UserUtil.setCurrentPersistOperation(this);
        //设置不收藏
        this.setIsCollect("0");
        //设置不指派
        this.setIsAppoint("0");

        this.setRemarks("");
        //设置待开始
        this.setTrainIsSign("0");
    }

}
