package org.rcisoft.business.blesson.service;

import org.rcisoft.business.bcourse.dto.AllCourseDTO;
import org.rcisoft.business.blesson.dto.*;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.entity.SysUser;

import java.util.List;
import java.util.Map;

/**
 * Created by CodeGenerator on 2017/07/12.
 */
public interface BLessonService{

    /**
     * 根据主键唯一查找
     * @author: zhangqingle
     * @param businessId
     * @return
     */
    BLesson selectOne(CurUser curUser,String businessId);

    /**
     * 根据bLesson对象查询
     */
    BLesson selectOne(BLesson bLesson);
    /**
     * 根据bLesson对象更新
     */
    int updateByPrimaryKeySelective(BLesson bLesson);
    /**
     * 根据条件分页查找
     * @author: zhangqingle
     * @param pageUtil
     * @param myReleaseDTO
     * @return
     */
    List<BLesson> queryBLessonsByPagination(PageUtil pageUtil, MyReleaseDTO myReleaseDTO);

    /**
     * 根据条件分页查找
     * @author: zhangqingle
     * @param pageUtil
     * @param findAllLessonDTO
     * @return
     */
    List<BLesson> queryAllBLessonsByPagination(PageUtil pageUtil, FindAllLessonDTO findAllLessonDTO);

    /**
     * 分页查找大家都在学
     * @author: zhangqingle
     * @param pageUtil
     * @param
     * @return
     */
    List<BLesson> queryPersonMoreByPagination(PageUtil pageUtil,CurUser curUser);

    /**
     * 分页查找推荐
     * @author: zhangqingle
     * @param pageUtil
     * @param
     * @return
     */
    List<BLesson> queryRecommendByPagination(PageUtil pageUtil,CurUser curUser);


    /**
     * 分页查找最受关注
     * @author: zhangqingle
     * @param pageUtil
     * @param
     * @return
     */
    List<BLesson> queryConcernByPagination(PageUtil pageUtil,CurUser curUser);

    /**
     * 分页查找可能感兴趣的
     * @author: zhangqingle
     * @param pageUtil
     * @param
     * @return
     */
    List<BLesson> queryInterestedByPagination(PageUtil pageUtil,CurUser curUser);

    /**
     * 插入/更新课程
     * @author: zhangqingle
     * @param model
     * @return
     */
    PersistModel persist(AddLessonDTO model);

    /**
     * @author: zhangqingle
     * 逻辑删除
     * @param id
     * @return
     */
    PersistModel removeBLesson(String id);

    /**
     * @author: zhangqingle
     * 首页条件查询全部课程
     * @return
     */
    List<BLesson> queryHomeBLessonsByPagination(PageUtil pageUtil, FirstPageQueryDTO firstPageQueryDTO,List<AllCourseDTO> allCourse);

    /**
     * 关闭课程
     * @author: zhangqingle
     * @param id
     * @return
     */
    PersistModel closeBLesson(String id);

    /**
     * 发布课程
     * @author: zhangqingle
     * @param id
     * @return
     */
   PersistModel releaseBLesson(String id, CurUser curUser);

    /**
     * 推荐课程
     * @author: zhangqingle
     * @param recommendLessonDTO
     * @return
     */
   PersistModel recommendBLesson(RecommendLessonDTO recommendLessonDTO);

   /**
     * 根据课程Id 查询该课程推荐范围
     * @author: zhangqingle
     * @param param
     * @return
     */
   List<FindLessonRecommendRspDTO> queryRecommendByLessonId(FindLessonRecommendDTO param);

    /**
     * 指派课程
     * @param appointLessonDTO
     * @return
     */
    PersistModel appointLessonToPerson(AppointLessonDTO appointLessonDTO);

    /**
     * 课程培训条件查询
     * @author: zhangqingle
     * @param lessonTrainDTO
     * @return
     */
    List<BLesson> findLessonAtBanner(LessonTrainAtBannerDTO lessonTrainDTO);

    /**
     * 我发布的总数
     * @author: zhangqingle
     * @param curUser
     * @return
     */
    ILessonCountDTO iLessonCount(CurUser curUser);
    /**
     * 我发布的总数
     * @author: zhangqingle
     * @param curUser
     * @return
     */
    ILessonCountDTO iTrainCount(CurUser curUser);

    /**
     * 课程学员总数
     * @author: zhangqingle
     * @param lessonId
     * @return
     */
    ILessonCountDTO trainPersonCount(String lessonId, CurUser curUser);

    /**
     * 培训学员查询
     * @author: zhangqingle
     * @param param
     * @return
     */
    List<BLessonPerson> trainPersons(FindTrainPersonDTO param);

    /**
     * 分页查询列表页全部
     * @author: zhangqingle
     * @param pageUtil
     * @param model
     * @return
     */
    List<BLesson> queryListAllLessonByPagination(PageUtil pageUtil, FindListLessonDTO model, List<AllCourseDTO> allCourse);

     /**
      * @author: jiangpengpeng
      * @description: 分页查找课程管理数据
      * @date: 2019/9/27 14:44
      * @param
      * @return
      */
      List<BLesson> theLessonMangageByPagination(PageUtil pageUtil,CurUser curUser,BLesson bLesson,List<RoleTypeDepartDTO> roleList);

    /**
     * 取消推荐
     * @author: zhangqingle
     * @param lessonId
     * @return
     */
    PersistModel deleteRecommend(String lessonId);
    /**
     * 设置培训状态
     * @author: zhangqingle
     * @param setTrainIsSignDTO
     * @return
     */
    PersistModel setTrainIsSign(SetTrainIsSignDTO setTrainIsSignDTO);

    /**
     * 分页查询列表页全部
     * @author: zhangqingle
     * @param curUser
     * @param lessonId
     * @return
     */
    PersistModel addCollect(CurUser curUser,String lessonId);

    /**
     * 报名培训
     * @return
     */
    int toTraining(CurUser curUser,String lessonId);

    /**
     * 收藏课程或课程
     * @param curUser
     * @param lessonId
     * @return
     */
    PersistModel colletLesson(CurUser curUser,String lessonId);

    PersistModel notColletLesson(CurUser curUser,String lessonId);

    Map<String,Object> getLessonState(String id);

    /**
     * 单一查询我参加培训的信息
     * @param curUser
     * @param lessonId
     * @return
     */
    BLesson findOneMyTrain(CurUser curUser,String lessonId);


    /**
     * 单一查询我参加培训的信息
     * @param curUser
     * @param lessonId
     * @return
     */
    BLesson findOneMyTeachTrain(CurUser curUser,String lessonId);


    /**
     * 单一查询我参加培训的信息
     * @param curUser
     * @param lessonId
     * @return
     */
    List<SysUser> findAppointPerson(CurUser curUser, String lessonId);

    /**
     *分页查询我收藏
     * @param lessonType
     * @param curUser
     * @return
     */
    List<BLesson> queryCollectByPagination(PageUtil pageUtil, String lessonType , CurUser curUser);

    /**
     * 判断可见权限
     * @param curUser
     * @param lessonId
     * @return
     */
    Map<String,Object> judgeVisibleAuthority(CurUser curUser, String lessonId);

    /**
     * 用户管理-课程情况
     * @param pageUtil
     * @param userLessonDTO
     * @return
     */
    List<StudentTrackingRspDTO>  userManageLessonByPagination(PageUtil pageUtil, UserLessonDTO userLessonDTO);

    List<ExamDto>  userManageExamByPagination(PageUtil pageUtil, ExamQueryDto dto);

}
