package org.rcisoft.business.blesson.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.blesson.dao.BAppointRepository;
import org.rcisoft.business.blesson.dao.BHotRepository;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.dto.*;
import org.rcisoft.business.blesson.entity.BAppoint;
import org.rcisoft.business.blesson.entity.BHot;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.dao.BLessonPersonRepository;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.business.blesson.service.BLessonPersonService;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.bpersonvalue.dao.BPersonValueRepository;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.common.util.feignDto.*;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultExceptionEnum;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.dto.MyInfoDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;


/**
 * @author
 * @date 2019/9/10
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BLessonPersonServiceImpl implements BLessonPersonService {

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private BLessonPersonRepository bLessonPersonRepository;

    @Autowired
    private BAppointRepository bAppointRepository;

    @Autowired
    private BHotRepository bHotRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    BPersonValueRepository bPersonValueRepository;

    @Autowired
    BLessonServiceImpl bLessonServiceImpl;


    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel quit(QuitLessonDTO quitLessonDTO,CurUser curUser) {
        BLessonPerson bLessonPerson = new BLessonPerson();
        if (StringUtils.isNotEmpty(quitLessonDTO.getBusinessId())){
            bLessonPerson = bLessonPersonRepository.selectById(quitLessonDTO.getBusinessId());
        }else {
            AddMyLearnLessonDTO addMyLearnLessonDTO = new AddMyLearnLessonDTO(quitLessonDTO.getLessonId(),curUser.getCorpId(),curUser.getUserId());
            List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByPersonAndLesson(addMyLearnLessonDTO);
            if (bLessonPersonList != null && bLessonPersonList.get(0) != null){
                bLessonPerson = bLessonPersonList.get(0);
            }
        }

        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(quitLessonDTO.getLessonId());
        bLesson = bLessonRepository.selectByPrimaryKey(bLesson);
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.NOT_LESSON_ERROR);
        }
        if (bLessonPerson == null && "0".equals(bLesson.getLessonType())){
            throw new ServiceException(ResultServiceEnums.APPOINT_NOT_CLOSE);
        }
        if (bLessonPerson == null && "1".equals(bLesson.getLessonType())){
            throw new ServiceException(ResultServiceEnums.LEARN_LESSON_NOT_EXISTS);
        }


        if ("1".equals(bLessonPerson.getIsAppoint())){
            throw new ServiceException(ResultServiceEnums.APPLY_TRAIN_NOT_EXISTS);
        }
        bLessonPerson.setDelFlag("1");
        //将加入学习时间置空
        bLessonPerson.setApplyDate(null);

        UserUtil.setCurrentMergeOperation(bLessonPerson);
//        int line = bLessonPersonRepository.quit(quitLessonDTO.getBusinessId());
        int line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        //重新统计在学人数
        bLessonRepository.personNumberReCount(quitLessonDTO.getLessonId(),bLesson.getLessonType());
        return new PersistModel(line);
    }

    @Override
    public List<BLesson> queryLearnBLessonsByPagination(PageUtil pageUtil, ILearnLessonDTO model) {
        return bLessonPersonRepository.queryLearnBLessons(model);
    }

    @Override
    public StudentTrackingChartRspDTO studentTrackingChart(StudentTrackingChartDTO studentTrackingChartDTO) {
        return bLessonPersonRepository.studentTrackingChart(studentTrackingChartDTO);
    }

    @Override
    public List<StudentTrackingRspDTO> studentTrackingByPagination(PageUtil pageUtil, StudentTrackingDTO studentTrackingDTO) {
//        List<Long> deptIds = new ArrayList<>();
        if(StringUtils.isNotEmpty(studentTrackingDTO.getDepartId())){

            Set<String> deptIds = new HashSet<>();
            deptIds.add(studentTrackingDTO.getDepartId());
            Set<String> deptSets = new HashSet<>(deptIds);
            List<String> datas = mtCotactApiRequestClient.getSubUserIdsByDeptIds(studentTrackingDTO.getCorpId(), deptSets);
//            List<UserGetStudentTrackingRspDTO> datas = mtCotactApiRequestClient.allUserListByDeptAtStuTrack(studentTrackingDTO.getCorpId(),studentTrackingDTO.getDepartId());
            if (datas != null && datas.size()>0){
                List<String> userIdsInDepart = new ArrayList<>(datas);
//                datas.forEach(data->{
//                    userIdsInDepart.addAll(data);
//                });
                studentTrackingDTO.setUserIdsInDepart(userIdsInDepart);
            }
            if (studentTrackingDTO.getUserIdsInDepart() == null || studentTrackingDTO.getUserIdsInDepart().size() < 1){
                return new ArrayList<>();
            }
        }
        List<StudentTrackingRspDTO> studentTrackingRspDTOs = bLessonPersonRepository.studentTrackingByPagination(studentTrackingDTO);
        List<String> uids = new ArrayList<>();
        if (studentTrackingRspDTOs != null && studentTrackingRspDTOs.size() > 0){
            studentTrackingRspDTOs.forEach(studentTrackingRspDTO->{
                uids.add(studentTrackingRspDTO.getPersonId());
            });
        }
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(studentTrackingDTO.getCorpId());
        mtUserGetsReqDTO.setIds(uids);
        //遍历获取部门名
        List<MTUserInfoRspDTO> departs = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (studentTrackingRspDTOs != null && studentTrackingRspDTOs.size() > 0 && departs != null && departs.size() > 0){
            for (StudentTrackingRspDTO studentTrackingRspDTO : studentTrackingRspDTOs) {
                for (MTUserInfoRspDTO depart : departs) {
                    if (studentTrackingRspDTO.getPersonId().equals(depart.getId()) && depart.getDepts() != null && depart.getDepts().size() > 0){
                        studentTrackingRspDTO.setDepartName(depart.getDepts().get(0).getName());
                    }
                }
            }
        }
        return studentTrackingRspDTOs;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel cancelAppoint(CancelAppointDTO cancelAppointDTO) {
        String bAppointId = bLessonPersonRepository.selectAppointIdById(cancelAppointDTO);
        if (StringUtils.isEmpty(bAppointId)){
            throw new ServiceException(ResultServiceEnums.APPOINT_NOT_CANCEL);
        }

        BAppoint bAppoint = bAppointRepository.selectById(bAppointId);
        //改变bAppoint 值
        if (bAppoint == null){
            throw new ServiceException(ResultServiceEnums.NOT_APPOINT_INFO);
        }
        String designatedIds = bAppoint.getDesignatedId();
        String[] designatedIdList = designatedIds.split(",");
        String designatedIdRes = "";
        for (String s : designatedIdList) {
            if (!s.equals(cancelAppointDTO.getPersonId())){
                designatedIdRes += s + ",";
            }
        }
        //去除最后逗号
        if (designatedIdRes.length() > 0) {
            designatedIdRes = designatedIdRes.substring(0, designatedIdRes.length() - 1);
        }
        //删除b_appoint表
//        UserUtil.setCurrentMergeOperation(bAppoint);
//        bAppoint.setDesignatedId(designatedIdRes);

        //查询是否在学习
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(cancelAppointDTO.getLessonId());
//        String isFinish = bLessonPersonRepository.selectIsFinish(cancelAppointDTO);
        BLessonPerson bLessonPerson = bLessonPersonRepository.selectById(cancelAppointDTO.getBusinessId());
        int result = 0;
        //课程
        if (bLesson != null && "0".equals(bLesson.getLessonType()) && bLessonPerson != null && "0".equals(bLessonPerson.getIsFinish())){
            //未开始执行删除
            bLessonPerson.setDelFlag("1");
            //将加入学习时间置空
            bLessonPerson.setApplyDate(null);
            bLessonPerson.setAppointId("");
            bLessonPerson.setIsAppoint("0");
            result = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }
        if (bLesson != null && "0".equals(bLesson.getLessonType()) && bLessonPerson != null && ("1".equals(bLessonPerson.getIsFinish()) || "2".equals(bLessonPerson.getIsFinish()))){
            //已开始执行更新
            UserUtil.setCurrentMergeOperation(bLessonPerson);
            bLessonPerson.setIsAppoint("0");
            bLessonPerson.setAppointId("");
            result = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }
        //培训
        if (bLesson != null && "1".equals(bLesson.getLessonType()) && bLessonPerson != null){

            UserUtil.setCurrentMergeOperation(bLessonPerson);
            bLessonPerson.setIsAppoint("0");
            bLessonPerson.setAppointId("");
            result = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }

        //没有指派人员 删除指派表
        if (StringUtils.isEmpty(designatedIdRes)){
            bAppointRepository.deleteByPrimaryKey(bAppoint);
        }else {
            //更新b_appoint表
            UserUtil.setCurrentMergeOperation(bAppoint);
            bAppoint.setDesignatedId(designatedIdRes);
            bAppointRepository.updateByPrimaryKeySelective(bAppoint);
        }

        return new PersistModel(1,result);
    }

    @Override
    public PersistModel addMyLearnLesson(AddMyLearnLessonDTO addMyLearnLessonDTO, CurUser curUser) {

        BLesson bLesson = bLessonRepository.selectInfoById(addMyLearnLessonDTO.getLessonId(),curUser);
        //判断课程是否存在
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        //判断课程是否被关闭
        if ("4".equals(bLesson.getReleaseState())){
            throw new ServiceException(ResultServiceEnums.LESSON_CLOSED);
        }
        if (!"0".equals(bLesson.getLessonType())){
            throw new ServiceException(ResultServiceEnums.NOT_TRAIN_TYPE);
        }

        //添加至b_hot表 并重新统计热度
        int count = bHotRepository.selectCountByPersonId(addMyLearnLessonDTO.getUserId(),addMyLearnLessonDTO.getLessonId());
        if (count < 1){
            bHotRepository.insertSelective(new BHot(IdGen.uuid(),addMyLearnLessonDTO.getUserId(),addMyLearnLessonDTO.getLessonId()));
            bLessonRepository.hotNumberReCount(addMyLearnLessonDTO.getLessonId());
        }

        List<BLessonPerson> bLessonPersons = bLessonPersonRepository.selectByPersonAndLesson(addMyLearnLessonDTO);
        BLessonPerson bLessonPerson;
        int line = 0;
        //被指派未开始
        if (bLessonPersons != null && bLessonPersons.size()>0 && "0".equals(bLessonPersons.get(0).getIsFinish()) ){
            bLessonPerson = bLessonPersons.get(0);
            bLessonPerson.setIsFinish("1");
            line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
            //重新统计在学人数
            bLessonRepository.personNumberReCount(addMyLearnLessonDTO.getLessonId(),"0");
            return new PersistModel(line);
        }
        if (bLessonPersons == null || bLessonPersons.size() < 1){    //未指派
            //判断用户是否有足够积分
            MyInfoDTO userInfo =  sysUserMapper.getNameById(curUser.getUserId());
            if (userInfo == null){
                throw new ServiceException(ResultServiceEnums.COMPANY_NOT_EXISTS);
            }
            if (bLesson.getValueConsume() != null && (StringUtils.isEmpty(userInfo.getValue()) ||  Long.valueOf(userInfo.getValue()) < Long.valueOf(bLesson.getValueConsume()))){
                throw new ServiceException(ResultServiceEnums.VALUE_NOT_ENOUGH);
            }
            //查询是否已有已删除
            List<BLessonPerson> bLessonPersonReals = bLessonPersonRepository.selectRealByPersonAndLesson(addMyLearnLessonDTO);
            if (bLessonPersonReals != null && bLessonPersonReals.size()>0){
                BLessonPerson bLessonPersonReal = bLessonPersonReals.get(0);
                bLessonPersonReal.setFlag("1");
                bLessonPersonReal.setDelFlag("0");
                //更新报名时间
                bLessonPersonReal.setApplyDate(new Date());

                UserUtil.setCurrentMergeOperation(bLessonPersonReal);
                line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPersonReal);
                //扣除对应积分 添加用户积分详情 更新用户积分
                //增加用户积分详情
                if (bLesson.getValueConsume() != null && bLesson.getValueConsume() != 0){
                    BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"学习课程",bLesson.getValueConsume(),"1");
                    bLessonServiceImpl.addValueEvent(curUser.getUserId(),bPersonValue,userInfo.getLockNum());
                }
                bLessonRepository.personNumberReCount(addMyLearnLessonDTO.getLessonId(),"0");
                return new PersistModel(line);
            }
            bLessonPerson = new BLessonPerson();
            bLessonPerson.initModel();
            bLessonPerson.setIsFinish("1");
            //添加报名时间
            bLessonPerson.setApplyDate(new Date());
            bLessonPerson.setLessonId(addMyLearnLessonDTO.getLessonId());
            bLessonPerson.setPersonId(addMyLearnLessonDTO.getUserId());
            line = bLessonPersonRepository.insertBLessonPerson(bLessonPerson);
            //扣除对应积分 添加用户积分详情 更新用户积分
            //增加用户积分详情
            //积分变化后 s_user表中value字段重新统计
            if (bLesson.getValueConsume() != null && bLesson.getValueConsume() != 0){
                BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"学习课程",bLesson.getValueConsume(),"1");
                bLessonServiceImpl.addValueEvent(curUser.getUserId(),bPersonValue,userInfo.getLockNum());
            }
            //重新统计在学人数
            bLessonRepository.personNumberReCount(addMyLearnLessonDTO.getLessonId(),"0");
            return new PersistModel(line);
        }
//        if (bLessonPersons != null && bLessonPersons.size()>0){
//            line = 1;
//            return new PersistModel(line);
//        }

        return  new PersistModel(1);
    }
}
