package org.rcisoft.business.blesson.service.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bchapter.dao.BChapterRepository;
import org.rcisoft.business.bcourse.dao.BCourseRepository;
import org.rcisoft.business.bcourse.dto.AllCourseDTO;
import org.rcisoft.business.bfile.dao.BTrainFileRepository;
import org.rcisoft.business.bfile.entity.BTrainFile;
import org.rcisoft.business.blesson.dao.*;
import org.rcisoft.business.blesson.dto.*;
import org.rcisoft.business.blesson.entity.*;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.blesson.util.QueryDepart;
import org.rcisoft.business.blesson.util.Recursion;
import org.rcisoft.business.blesson.dao.BViewrangeRepository;
import org.rcisoft.business.bmaterial.dao.BMaterialRepository;
import org.rcisoft.business.bmaterial.entity.BMaterial;
import org.rcisoft.business.bpersonvalue.dao.BPersonValueRepository;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.business.breleasevalue.dao.BReleaseValueRepository;
import org.rcisoft.business.breleasevalue.entity.BReleaseValue;
import org.rcisoft.common.util.feignDto.*;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.dto.MyInfoDTO;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.beans.Transient;
import java.util.ArrayList;
import java.util.List;
import java.util.*;

import static java.util.Arrays.asList;


/**
 * Created by CodeGenerator on 2017/07/12.
 */
@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BLessonServiceImpl implements BLessonService {
    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private BLessonPersonRepository bLessonPersonRepository;

    @Autowired
    private BCourseRepository bCourseRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private BViewrangeRepository bViewrangeRepository;

    @Autowired
    private BRecommendRepository bRecommendRepository;

    @Autowired
    private BAppointRepository bAppointRepository;

    @Autowired
    private BCollectRepository bCollectRepository;

    @Autowired
    private BTrainFileRepository bTrainFileRepository;

    @Autowired
    private BHotRepository bHotRepository;

    @Autowired
    private BMaterialRepository bMaterialRepository;

    @Autowired
    private BChapterRepository bChapterRepository;

    @Autowired
    MTCotactApiRequestClient cotactApiRequestClient;

    @Autowired
    BReleaseValueRepository bReleaseValueRepository;

    @Autowired
    BPersonValueRepository bPersonValueRepository;

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public BLesson selectOne(CurUser curUser,String lessonId) {
        BLesson bLesson = bLessonRepository.selectInfoById(lessonId,curUser);
        if (bLesson == null ){
            throw new ServiceException(ResultServiceEnums.NOT_LESSON_ERROR);
        }
        //获取讲师部门
        List<String> ids = new ArrayList<>();
        ids.add(bLesson.getLecturerId());
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                if (mtUserInfoRspDTO.getDepts() != null && mtUserInfoRspDTO.getDepts().size() > 0 && mtUserInfoRspDTO.getId().equals(bLesson.getLecturerId())){
                    //设置部门名
                    bLesson.setLecturerDeptName(mtUserInfoRspDTO.getDepts().get(0).getName());
                }
            });
        }

        if ("1".equals(bLesson.getLessonType())){
            List<BTrainFile> bTrainFileList = bTrainFileRepository.queryByLessonId(bLesson.getBusinessId());
            bLesson.setBTrainFileList(bTrainFileList);
        }

        List<FindDepartByNameDTO> departs = cotactApiRequestClient.deptListDeptInfoByName(curUser.getCorpId(),"");

        //查询该课程推荐的所有人
        List<BViewRangeSonDTO> selectPersonByLessonIds = bViewrangeRepository.selectPersonByLessonId(curUser.getCorpId(),lessonId);
        bLesson.setViewRangePersonList(selectPersonByLessonIds);
        //查询该课程推荐的所有部门
        List<BViewRangeSonDTO> selectDepartByLessonIds = bViewrangeRepository.selectDepartByLessonId(curUser.getCorpId(),lessonId);
        if ( selectDepartByLessonIds != null && selectDepartByLessonIds.size() > 0){
            for (FindDepartByNameDTO depart : departs) {
                for (BViewRangeSonDTO selectDepartByLessonId : selectDepartByLessonIds) {
                    if ( depart.getId().equals(selectDepartByLessonId.getTargetId())){
                        selectDepartByLessonId.setTargetName(depart.getName());
                    }
                }
            }
            bLesson.setViewRangeDepartList(selectDepartByLessonIds);
        }

        // 每次点击插入热度表 并重新统计课程热度
        int count = bHotRepository.selectCountByPersonId(curUser.getUserId(),lessonId);
        if (count < 1){
            bHotRepository.insertSelective(new BHot(IdGen.uuid(),curUser.getUserId(),bLesson.getBusinessId()));
            bLessonRepository.hotNumberReCount(bLesson.getBusinessId());
        }

        return bLesson;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public BLesson selectOne(BLesson bLesson) {
        return bLessonRepository.selectOne(bLesson);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public int updateByPrimaryKeySelective(BLesson bLesson) {
        return bLessonRepository.updateByPrimaryKeySelective(bLesson);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public List<BLesson> queryBLessonsByPagination(PageUtil pageUtil, MyReleaseDTO myReleaseDTO) {
        return bLessonRepository.queryBLessons(myReleaseDTO);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public List<BLesson> queryAllBLessonsByPagination(PageUtil pageUtil, FindAllLessonDTO findAllLessonDTO) {
        return bLessonRepository.queryAllBLesson(findAllLessonDTO);
    }



    @Override
    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    public List<BLesson> queryPersonMoreByPagination(PageUtil pageUtil, CurUser curUser) {
        //---------获取部门id------------
//        List<String> departs = getDeparts();

        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
        //------------------------------
        return bLessonRepository.queryPersonMore(curUser, departs);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> queryRecommendByPagination(PageUtil pageUtil, CurUser curUser) {
//        List<String> departs = getDeparts();
        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
        return bLessonRepository.queryRecommend(curUser, departs);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> queryConcernByPagination(PageUtil pageUtil, CurUser curUser) {
        //---------获取部门id------------
//        List<String> departs = getDeparts();
        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
        //------------------------------
        return bLessonRepository.queryConcern(curUser, departs);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> queryInterestedByPagination(PageUtil pageUtil, CurUser curUser) {
        //---------获取部门id------------
//        List<String> departs = getDeparts();
        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
        //------------------------------
        return bLessonRepository.queryInterested(curUser, departs);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel persist(AddLessonDTO addLessonDTO) {
        if ("1".equals(addLessonDTO.getLessonType())){
            try {
                if (Integer.parseInt(addLessonDTO.getMaxApplyPerson()) < 0){
                    throw new ServiceException(ResultServiceEnums.INPUT_FORMAT_ERROR);
                }
            } catch (NumberFormatException e) {
                log.error(e.getMessage()+" 输入的格式不正确 ");
                throw new ServiceException(ResultServiceEnums.INPUT_FORMAT_ERROR);
            }
        }

        BLesson model = new BLesson();
        BeanUtils.copyProperties(addLessonDTO, model);

        //取出文件集合创建BMaterial集合
//        List<BMaterial> bMaterialList = new ArrayList<>();
//        if (trainFileList!=null && trainFileList.size()>0){
//            BMaterial bMaterial = new BMaterial();
//        }

        if (StringUtils.isNotEmpty(addLessonDTO.getDefaultUrl()) && !"http".equalsIgnoreCase(addLessonDTO.getDefaultUrl().substring(0,4))){
            model.setDefaultUrl(null);
        }

        int line = 0;

        if (StringUtils.isEmpty(model.getBusinessId())) {
            if (StringUtils.isEmpty(model.getLessonType())) {
                throw new ServiceException(ResultServiceEnums.MUST_NOT_FILLED);
            }
            //初始化
            if ("0".equals(model.getLessonType())) {
                model.initModel();
            } else if ("1".equals(model.getLessonType())) {
                model.initTrainModel();
                if ("1".equals(addLessonDTO.getIsClient())) {
                    model.setLecturerId(model.getCreateBy());
                }
            }
            //添加课程表
            line = bLessonRepository.insertSelective(model);
            //添加可见范围
            if (StringUtils.isNotEmpty(model.getViewRangePerson()) || StringUtils.isNotEmpty(model.getViewRangeDepart())) {
                BViewrange bViewrange = setBViewrangeInfo(model,null);
                //插入可见范围表
                bViewrangeRepository.insertSelective(bViewrange);
                //插入可见范围子表
                List<BViewrangeSon> bViewrangeSons = new ArrayList<>();
                //model.getViewRangePerson()不为空 新建bViewrangeSon对象放入bViewrangeSons中
                if (StringUtils.isNotEmpty(model.getViewRangePerson())) {
                    String[] viewRangePerson = model.getViewRangePerson().split(",");
                    bViewrangeSons = addBViewrangeSon(bViewrangeSons, viewRangePerson, "0", bViewrange.getBusinessId());
                }
                if (StringUtils.isNotEmpty(model.getViewRangeDepart())) {
                    String[] viewRangeDepart = model.getViewRangeDepart().split(",");
                    bViewrangeSons = addBViewrangeSon(bViewrangeSons, viewRangeDepart, "1", bViewrange.getBusinessId());
                }
                //插入子表
                if (bViewrangeSons != null && bViewrangeSons.size()>0){
                    bViewrangeRepository.insertBViewrangeSons(bViewrangeSons);
                }
            }

            //添加课程标签表
            if (StringUtils.isNotEmpty(model.getLabels())) {
                //标签集合
                List<BLessonLabel> bLessonLabels = addLabel(model);
                bLessonRepository.insertLessonLabel(bLessonLabels);
            }
        } else {
            UserUtil.setCurrentMergeOperation(model);
            line = bLessonRepository.updateByPrimaryKeySelective(model);

            if (StringUtils.isNotEmpty(model.getViewRangePerson()) || StringUtils.isNotEmpty(model.getViewRangeDepart())) {
                BViewrange bViewrange = new BViewrange();
                List<BViewrange> bViewranges = bViewrangeRepository.selectByLessonId(model.getBusinessId());
                if (bViewranges != null && bViewranges.size() > 0){
                    bViewrange = bViewranges.get(0);
                    bViewrange = setBViewrangeInfo(model,bViewrange);
                    //更新可见范围表
                    bViewrangeRepository.updateByLessonId(bViewrange);
                    //删除可见范围子表
                    bViewrangeRepository.deleteViewrangeSon(bViewrange);
                }
                if (bViewranges == null || bViewranges.size() < 1){
                    bViewrange = setBViewrangeInfo(model,null);
                    //添加可见范围表
                    bViewrangeRepository.insertSelective(bViewrange);
                }

                //获取所有可见人员和部门
                List<BViewrangeSon> bViewrangeSons = new ArrayList<>();

                //model.getViewRangePerson()不为空 新建bViewrangeSon对象放入bViewrangeSons中 type：0人员 1部门
                if (StringUtils.isNotEmpty(model.getViewRangePerson())) {
                    String[] viewRangePerson = model.getViewRangePerson().split(",");
                    bViewrangeSons = addBViewrangeSon(bViewrangeSons, viewRangePerson, "0", bViewrange.getBusinessId());
                }
                if (StringUtils.isNotEmpty(model.getViewRangeDepart())) {
                    String[] viewRangeDepart = model.getViewRangeDepart().split(",");
                    bViewrangeSons = addBViewrangeSon(bViewrangeSons, viewRangeDepart, "1", bViewrange.getBusinessId());
                }
                //插入子表
                if (bViewrangeSons != null && bViewrangeSons.size()>0){
                    bViewrangeRepository.insertBViewrangeSons(bViewrangeSons);
                }
            }else {
                //可见范围设为空  所有人员均可见
                List<BViewrange> bViewranges = bViewrangeRepository.selectByLessonId(model.getBusinessId());
                //删除可见范围子表
                if (bViewranges != null && bViewranges.size()>0){
                    bViewrangeRepository.deleteByPrimaryKey(bViewranges.get(0));
                    bViewrangeRepository.deleteViewrangeSon(bViewranges.get(0));
                }
            }

            bLessonRepository.removeLabelByLessonId(model);
            //删除原标签添，加课程标签表
            if (StringUtils.isNotEmpty(model.getLabels())) {
                //标签集合
                List<BLessonLabel> bLessonLabels = addLabel(model);
                bLessonRepository.insertLessonLabel(bLessonLabels);
            }
            //删除原有文件
            bTrainFileRepository.deleteByLessonId(model.getBusinessId());
        }
        //转换json
        if (StringUtils.isNotEmpty(addLessonDTO.getTrainFileJson())){
            List<BTrainFile> bTrainFiles = JSONObject.parseArray(addLessonDTO.getTrainFileJson(), BTrainFile.class);
            addLessonDTO.setTrainFileList(bTrainFiles);
        }

        //插入b_train_file表
        //取出trainFileList
        List<BTrainFile> trainFileList = addLessonDTO.getTrainFileList();
        if (trainFileList != null && trainFileList.size() > 0) {
            List<BMaterial> bMaterialList = new ArrayList<>();
            int sortNum = 7;
            for (BTrainFile bTrainFile : trainFileList) {
                bTrainFile.setLessonId(model.getBusinessId());
                UserUtil.setCurrentPersistOperation(bTrainFile);
                //设置sort  用于返回为上传时顺序
                bTrainFile.setSort(String.valueOf(sortNum));
                sortNum++;
                //设置资料表值
                BMaterial bMaterial = new BMaterial();
                UserUtil.setCurrentPersistOperation(bMaterial);
                bMaterial.setLessonName(model.getLessonName());
                bMaterial.setLessonId(model.getBusinessId());
                bMaterial.setFileName(bTrainFile.getFileName());
                bMaterial.setFileUrl(bTrainFile.getVideoUrl());
                bMaterial.setType(bTrainFile.getType());
                bMaterial.setUploadType("1");
                bMaterial.setFileSize(bTrainFile.getFileSize());
                bMaterialList.add(bMaterial);
            }
            bTrainFileRepository.insertTrainFileList(trainFileList);

            List<BMaterial> inBMaterialList = bMaterialRepository.materialHaveUrl(trainFileList,"1");
            //遍历  移除已存在的文件 将已存在的文件放进List 中
            if (bMaterialList!=null && bMaterialList.size()>0 && inBMaterialList != null &&inBMaterialList.size()>0){
                List<BMaterial> pubBMaterial = new ArrayList<>();
                for (BMaterial bMaterial : bMaterialList) {
                    for (BMaterial inBMaterial : inBMaterialList) {
                        if (inBMaterial.getFileUrl().equals(bMaterial.getFileUrl())){
//                        inBMaterialList.add(bMaterial);
                            pubBMaterial.add(bMaterial);
//                            bMaterialList.remove(bMaterial);
                        }
                    }
                }
                if (pubBMaterial!=null && pubBMaterial.size()>0){
                    bMaterialList.removeAll(pubBMaterial);
                }
            }
            //插入资料表
            if (bMaterialList!=null && bMaterialList.size()>0){
                bMaterialRepository.insertMaterialList(bMaterialList);
            }

        }
        return new PersistModel(line, model);
    }


    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel removeBLesson(String id) {
//        if(bSlRepository.selectslNumByLesson(id)>0) {
//            throw new ServiceException(ResultServiceEnums.LESSON_HAS_SL);
//        }
//        if(bSlRepository.isUse(id) > 0){
//            throw new ServiceException(ResultServiceEnums.LESSON_HAS_SL);
//        }
        BLesson bl = new BLesson();
        bl.setBusinessId(id);
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(bl);
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.NOT_LESSON_ERROR);
        }
        bLesson.setDeleted();
        UserUtil.setCurrentMergeOperation(bLesson);
        int line = bLessonRepository.logicalDelete(bLesson);
        //int line = bLessonRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);

    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> queryHomeBLessonsByPagination(PageUtil pageUtil, FirstPageQueryDTO firstPageQueryDTO, List<AllCourseDTO> allCourse) {
//        List<String> departs = getDeparts();
        List<String> ids = Arrays.asList(new String[]{firstPageQueryDTO.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(firstPageQueryDTO.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
        firstPageQueryDTO.setDeparts(departs);
        List<String> courseIds = null;
        if (allCourse != null) {
            //查询出所有子分类
            courseIds = Recursion.FindSons(firstPageQueryDTO.getCourseLevelOne(), allCourse);
            //将一级分类放入分类集合中
            courseIds.add(firstPageQueryDTO.getCourseLevelOne());
        }
        return bLessonRepository.queryHomeBLesson(firstPageQueryDTO, courseIds);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel closeBLesson(String id) {
        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(id);
        bLesson.setCloseDate(new Date());
        UserUtil.setCurrentMergeOperation(bLesson);
        int line = bLessonRepository.closeLesson(bLesson);
        //int line = bLessonRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel releaseBLesson(String id, CurUser curUser) {
        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(id);
        bLesson = bLessonRepository.selectByPrimaryKey(bLesson);

        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.NOT_LESSON_ERROR);
        }

        if("0".equals(bLesson.getLessonType()) && bChapterRepository.getlessonHours(id) == 0){
            throw new ServiceException(ResultServiceEnums.LESSON_NO_CHAPTER);
        }

        if ("2".equals(bLesson.getReleaseState())) {
            throw new ServiceException(ResultServiceEnums.STATE_NOT_RELEASE);
        }

        if ("0".equals(bLesson.getLessonType()) && ("0".equals(bLesson.getClassHour()) || StringUtils.isEmpty(bLesson.getClassHour()))) {
            throw new ServiceException(ResultServiceEnums.NOT_CLASS_HOUR);
        }

        bLesson.setReleaseDate(new Date());
        UserUtil.setCurrentMergeOperation(bLesson);
        int line = bLessonRepository.releaseLesson(bLesson);
        //查询该讲师发布课程获得积分数
        List<BReleaseValue> bReleaseValueList = bReleaseValueRepository.selectReleaseValueByPersonId(curUser);
        if(bReleaseValueList != null && bReleaseValueList.size() > 0 && bReleaseValueList.get(0) != null){
            //增加用户积分详情
            BPersonValue bPersonValue = new BPersonValue();
            UserUtil.setCurrentPersistOperation(bPersonValue);
            bPersonValue.setPersonId(bLesson.getLecturerId());
            bPersonValue.setType("0");
            if ("0".equals(bLesson.getLessonType())) {
                bPersonValue.setEvent("发布课程");
                bPersonValue.setValue(bReleaseValueList.get(0).getLessonValue());
            }
            if ("1".equals(bLesson.getLessonType())){
                bPersonValue.setEvent("发布培训");
                bPersonValue.setValue(bReleaseValueList.get(0).getTrainValue());
            }
            bPersonValueRepository.add(bPersonValue);
            //积分变化后 s_user表中value字段重新统计
            bPersonValueRepository.updateUserValue(bLesson.getLecturerId(),null);
        }
        //int line = bLessonRepository.deleteByPrimaryKey(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel recommendBLesson(RecommendLessonDTO recommendLessonDTO) {
        BLesson bLessonExi = new BLesson();
        //按id查询存在的课程
        bLessonExi.setBusinessId(recommendLessonDTO.getLessonId());
        bLessonExi = bLessonRepository.selectByPrimaryKey(bLessonExi);
        //判断课程是否存在
        if (bLessonExi == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        //判断课程是否已关闭
        if ("4".equals(bLessonExi.getReleaseState())){
            throw new ServiceException(ResultServiceEnums.LESSON_CLOSED);
        }

        BRecommend bRecommend = new BRecommend();
        bRecommend.setLessonId(recommendLessonDTO.getLessonId());
        if (StringUtils.isNotEmpty(recommendLessonDTO.getRecommendPerson()) || StringUtils.isNotEmpty(recommendLessonDTO.getRecommendDepart())) {
            bRecommend.setRecommendPerson(recommendLessonDTO.getUserId());
            //设置推荐类型（0仅人 1仅部门 2人和部门）
            if (StringUtils.isNotEmpty(recommendLessonDTO.getRecommendPerson()) && StringUtils.isEmpty(recommendLessonDTO.getRecommendDepart())) {
                bRecommend.setType("0");
            } else if (StringUtils.isEmpty(recommendLessonDTO.getRecommendPerson()) && StringUtils.isNotEmpty(recommendLessonDTO.getRecommendDepart())) {
                bRecommend.setType("1");
            } else if (StringUtils.isNotEmpty(recommendLessonDTO.getRecommendPerson()) && StringUtils.isNotEmpty(recommendLessonDTO.getRecommendDepart())) {
                bRecommend.setType("2");
            }
            //查询是否存在该课程推荐
            List<String> recommendIds = bRecommendRepository.selectIdByLessonId(bRecommend.getLessonId());
            if (recommendIds == null || recommendIds.size() < 1) {
                //更新课程表recommend字段
                BLesson bLesson = new BLesson();
                UserUtil.setCurrentMergeOperation(bLesson);
                bLesson.setBusinessId(recommendLessonDTO.getLessonId());
                bLesson.setRecommend("1");
                bLessonRepository.updateByPrimaryKeySelective(bLesson);
                //插入推荐表
                UserUtil.setCurrentPersistOperation(bRecommend);
                bRecommendRepository.insertSelective(bRecommend);
            } else {
                //修改推荐表
                UserUtil.setCurrentMergeOperation(bRecommend);
                bRecommend.setBusinessId(recommendIds.get(0));
                bRecommendRepository.updateByPrimaryKeySelective(bRecommend);
                //删除推荐子表内容
                bRecommendRepository.deleteRecommendSon(bRecommend);
            }
            //插入推荐子表
            List<BRecommendSon> bRecommendSons = new ArrayList<>();
            //model.getViewRangePerson()不为空 新建bViewrangeSon对象放入bViewrangeSons中
            if (StringUtils.isNotEmpty(recommendLessonDTO.getRecommendPerson())) {
                String[] recommendPerson = recommendLessonDTO.getRecommendPerson().split(",");
                bRecommendSons = addBRecommendSon(bRecommendSons, recommendPerson, "0", bRecommend.getBusinessId());
            }
            if (StringUtils.isNotEmpty(recommendLessonDTO.getRecommendDepart())) {
                String[] recommendDepart = recommendLessonDTO.getRecommendDepart().split(",");
                bRecommendSons = addBRecommendSon(bRecommendSons, recommendDepart, "1", bRecommend.getBusinessId());
            }
            //插入子表
            int line = bRecommendRepository.insertBRecommendSons(bRecommendSons);
            return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
        }
        //若推荐为空，删除推荐表和推荐子表内容
        List<String> recommendIds = bRecommendRepository.selectIdByLessonId(recommendLessonDTO.getLessonId());
//            throw new ServiceException(ResultServiceEnums.INVALID_PARAMETER_VALUE);
        //若原本未推荐且本次推荐为空直接返回
        if(recommendIds == null || recommendIds.size() < 1){
            return new PersistModel(1, MessageConstant.MESSAGE_ALERT_SUCCESS);
        }
        //若原本推荐，本次推荐删除推荐表和推荐子表,更新课程推荐字段
        bRecommend.setBusinessId(recommendIds.get(0));
        int line = bRecommendRepository.deleteByPrimaryKey(bRecommend);
        bRecommendRepository.deleteRecommendSon(bRecommend);

        bLessonExi.setRecommend("0");
        bLessonRepository.updateByPrimaryKeySelective(bLessonExi);

        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<FindLessonRecommendRspDTO> queryRecommendByLessonId(FindLessonRecommendDTO param) {
        //按部门名模糊查询部门id（list）------
//        if (StringUtils.isEmpty(param.getViewParam())){
//            param.setViewParam("");
//        }
        String viewParam = param.getViewParam();
        if (StringUtils.isEmpty(viewParam)){
            viewParam = "";
        }
        List<FindDepartByNameDTO> departs = cotactApiRequestClient.deptListDeptInfoByName(param.getCorpId(), viewParam);
        //查询结果放入DTO中
        param.setDeparts(departs);
        //查询该课程推荐的所有人
        List<FindLessonRecommendRspDTO> findLessonPersonRecommend = new ArrayList<>();
        findLessonPersonRecommend = bRecommendRepository.queryRecommendPersonByLessonId(param);
        //查询该课程推荐的所有部门
        List<FindLessonRecommendRspDTO> findLessonDepartRecommend = new ArrayList<>();
        findLessonDepartRecommend = bRecommendRepository.queryRecommendDepartByLessonId(param);

        if (departs != null && departs.size() > 0 && findLessonDepartRecommend != null && findLessonDepartRecommend.size() > 0) {
            for (FindDepartByNameDTO depart : departs) {
                for (FindLessonRecommendRspDTO findLessonRecommendRspDTO : findLessonDepartRecommend) {
                    if (depart.getId().equals(findLessonRecommendRspDTO.getTargetId())) {
                        findLessonRecommendRspDTO.setTargetName(depart.getName());
                    }
                }
            }
            findLessonPersonRecommend.addAll(findLessonDepartRecommend);
        }
        return findLessonPersonRecommend;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel appointLessonToPerson(AppointLessonDTO appointLessonDTO) {
        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(appointLessonDTO.getLessonId());
        bLesson = bLessonRepository.selectByPrimaryKey(bLesson);
        //判断课程是否存在
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        //判断课程是否已关闭
        if ("4".equals(bLesson.getReleaseState())){
            throw new ServiceException(ResultServiceEnums.LESSON_CLOSED);
        }

        //获取部门中人员
        List<String> deptList = asList(org.apache.commons.lang.StringUtils.split(appointLessonDTO.getAppointDepart(), ","));
        Set<String> deptSet = new HashSet<>(deptList);
        List<String> inDepartPerson = cotactApiRequestClient.getSubUserIdsByDeptIds(appointLessonDTO.getCorpId(),deptSet);
        //-------------
//        if (StringUtils.isEmpty(appointLessonDTO.getAppointPerson()) && StringUtils.isEmpty(appointLessonDTO.getAppointDepart())) {
//            throw new ServiceException(ResultServiceEnums.INVALID_PARAMETER_VALUE);
//        }
        //按课程查询指派表是否为空
        List<BAppoint> bAppoints = bAppointRepository.selectByLessonId(appointLessonDTO.getLessonId());
        //将指派人截取放入List中
        List<String> userIds = new ArrayList<>();
        List<String> userIdList = new ArrayList<>();
        if (StringUtils.isNotEmpty(appointLessonDTO.getAppointPerson())){
            userIdList = asList(StringUtils.split(appointLessonDTO.getAppointPerson(), ","));
        }
        Set<String> userIdSet = new HashSet<>();
        //将部门和人员id放入set中去重
        if (userIdList != null && userIdList.size()>0){
            userIdSet.addAll(userIdList);
        }
        if (inDepartPerson != null && inDepartPerson.size()>0){
            userIdSet.addAll(inDepartPerson);
        }
        //去重后放入List
        userIds.addAll(new ArrayList<>(userIdSet));

        BAppoint bAppoint;
        //拼接人员和部门中人员
        String userAllString = StringUtils.join(userIdSet , ",");
        if (bAppoints == null || bAppoints.size() == 0) {
            //插入指派表
            bAppoint = new BAppoint();
            bAppoint.setLessonId(appointLessonDTO.getLessonId());
            bAppoint.setAppointerId(appointLessonDTO.getUserId());
            //将部门人员放入
            bAppoint.setDesignatedId(userAllString);
            UserUtil.setCurrentPersistOperation(bAppoint);
            bAppointRepository.insertAppoint(bAppoint);
        } else {
            bAppoint = bAppoints.get(0);
            bAppoint.setAppointerId(appointLessonDTO.getUserId());
            //将部门人员放入
            bAppoint.setDesignatedId(userAllString);
            UserUtil.setCurrentMergeOperation(bAppoint);
            bAppointRepository.updateByPrimaryKeySelective(bAppoint);
            //删除指派未开始
            BLessonPerson delBlessonPerson = new BLessonPerson();
            delBlessonPerson.setAppointId("");
            delBlessonPerson.setIsAppoint("0");
            delBlessonPerson.setApplyDate(null);
            delBlessonPerson.setLessonId(appointLessonDTO.getLessonId());
            bLessonPersonRepository.deleteNotStartByLessonId(delBlessonPerson);
            //指派更新为不指派
            BLessonPerson bLessonPerson = new BLessonPerson();
            bLessonPerson.setIsAppoint("0");
            bLessonPerson.setAppointId("");
            bLessonPerson.setLessonId(appointLessonDTO.getLessonId());
            UserUtil.setCurrentMergeOperation(bLessonPerson);
            bLessonPersonRepository.cancelAppointByPersons(bLessonPerson);
        }
        if (userIds == null || userIds.size() < 1){
            int line = bAppointRepository.deleteByPrimaryKey(bAppoint);
            bLessonRepository.personNumberReCount(bLesson.getBusinessId(),bLesson.getLessonType());
            return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
        }
        //插入b_lesson_person表
        appointLessonDTO.setAppointPersonList(userIdSet);
        //查询已学该课程的学习business_id
        List<BLessonPerson> inLessonPersons = new ArrayList<>();
        if (appointLessonDTO.getAppointPersonList() != null && appointLessonDTO.getAppointPersonList().size()>0){
            inLessonPersons = bLessonPersonRepository.selectPersonIdByLessonId(appointLessonDTO);
        }
        if (inLessonPersons != null && inLessonPersons.size() > 0) {
            List<String> inLessonIds = new ArrayList<>();
            for (BLessonPerson inLessonPerson : inLessonPersons) {
                inLessonIds.add(inLessonPerson.getPersonId());
            }
            //创建对象用于更新
            BLessonPerson updateBLessonPerson = new BLessonPerson();
            //将删除过得重新激活
            updateBLessonPerson.setDelFlag("0");
            updateBLessonPerson.setFlag("1");
            //设置指派信息
            updateBLessonPerson.setIsAppoint("1");
//            updateBLessonPerson.setApplyDate(new Date());//指派时间同步到lesson_person的报名时间
            updateBLessonPerson.setLessonId(appointLessonDTO.getLessonId());
            updateBLessonPerson.setAppointId(bAppoint.getBusinessId());
            UserUtil.setCurrentMergeOperation(updateBLessonPerson);
            Date nowDate = new Date();
            //更新b_lesson_person表,报名时间为空将当前时间给其赋值
            bLessonPersonRepository.updateList(updateBLessonPerson, inLessonIds, nowDate);
            //从学生Id中删除已更新的学生id
            userIds.removeAll(inLessonIds);
        }
//        //如果为培训的话 判断剩余学生加已有学生总数超过最大数 抛异常
//        if ("1".equals(bLesson.getLessonType())){
//            //查找已报名总人数
//            int personNumber = bLessonRepository.personNumberCount(bLesson.getBusinessId());
//            if (personNumber + userIds.size() > Integer.parseInt(bLesson.getMaxApplyPerson())){
//                throw new ServiceException("培训人数上限"+bLesson.getMaxApplyPerson()+"，现已超出"+String.valueOf(Integer.parseInt(bLesson.getMaxApplyPerson())+"人，请重新选择"));
//            }
//        }

        //剩余学生添加至b_lesson_person表
        List<BLessonPerson> bLessonPersonOthers = new ArrayList<>();
        for (String userId : userIds) {
            BLessonPerson bLessonPerson = new BLessonPerson();
            if ("0".equals(bLesson.getLessonType())){
                bLessonPerson.initModel();
                bLessonPerson.setIsFinish("0");
            }
            if ("1".equals(bLesson.getLessonType())){
                bLessonPerson.initTrainModel();
                bLessonPerson.setIsApply("1");
                bLessonPerson.setApplyDate(new Date());
            }

            bLessonPerson.setAppointId(bAppoint.getBusinessId());
            bLessonPerson.setIsAppoint("1");
//            bLessonPerson.setApplyDate(new Date());//指派时间同步到lesson_person的报名时间
            bLessonPerson.setLessonId(appointLessonDTO.getLessonId());
            bLessonPerson.setPersonId(userId);
            bLessonPersonOthers.add(bLessonPerson);
        }

        int line;
        if (bLessonPersonOthers != null && bLessonPersonOthers.size() > 0){
            if ("1".equals(bLesson.getLessonType())){
                //指派培训
                line =  trainInsert("1",bLesson,bLessonPersonOthers);
                bLessonRepository.personNumberReCount(bLesson.getBusinessId(),"1");
                return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
            }
            //指派课程
            line = bLessonPersonRepository.insertList(bLessonPersonOthers);
//            bLessonRepository.personNumberReCount(bLesson.getBusinessId(),"0");
        }else {
            bLessonRepository.personNumberReCount(bLesson.getBusinessId(),bLesson.getLessonType());
            line = 1;
        }
        bLessonRepository.personNumberReCount(bLesson.getBusinessId(),bLesson.getLessonType());
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> findLessonAtBanner(LessonTrainAtBannerDTO lessonTrainDTO) {
        return bLessonRepository.findLessonAtBanner(lessonTrainDTO);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public ILessonCountDTO iLessonCount(CurUser curUser) {
        return bLessonRepository.iLessonCount(curUser);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public ILessonCountDTO iTrainCount(CurUser curUser) {
        return bLessonRepository.iTrainCount(curUser);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public ILessonCountDTO trainPersonCount(String lessonId, CurUser curUser) {
        ILessonCountDTO iLessonCountDTO = bLessonRepository.trainPersonCount(lessonId);
        BLesson bLesson = bLessonRepository.selectInfoById(lessonId,curUser);
        if (bLesson != null){
            Date now = new Date();
            Long time = Long.parseLong(bLesson.getTrainSignTime()) * 60 * 1000;
            Date signStart = new Date(bLesson.getTrainStartDate().getTime() - time);
            if (now.before(signStart)){
                iLessonCountDTO.setTrainType("0");
            }else if ( bLesson.getTrainOverDate().before(now) ){
                iLessonCountDTO.setTrainType("2");
            }else{
                iLessonCountDTO.setTrainType("1");
            }
        }
        return iLessonCountDTO;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLessonPerson> trainPersons(FindTrainPersonDTO param) {
        List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.trainPersons(param);
        if (bLessonPersonList != null && bLessonPersonList.size()>0){
            List<String> departs = new ArrayList<>();
            List<String> ids = new ArrayList<>();
            bLessonPersonList.forEach(bLessonPerson->{
                String id = bLessonPerson.getPersonId();
                ids.add(id);
            });

            MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
            mtUserGetsReqDTO.setCorpId(param.getCorpId());
            mtUserGetsReqDTO.setIds(ids);
            List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
            if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
                bLessonPersonList.forEach(bLessonPerson->{
                    mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                        if (mtUserInfoRspDTO.getId().equals(bLessonPerson.getPersonId())){
                            //设置部门名
                            bLessonPerson.setPersonName(mtUserInfoRspDTO.getName());
                            bLessonPerson.setHeadPic(mtUserInfoRspDTO.getAvatar());
                            if (mtUserInfoRspDTO.getDepts() != null && mtUserInfoRspDTO.getDepts().size() > 0){
                                bLessonPerson.setDepartName(mtUserInfoRspDTO.getDepts().get(0).getName());
                            }
                            StringBuffer sb = new StringBuffer();
                            if (mtUserInfoRspDTO.getDepts() != null) {
                                for (MTUserInfoRspDTO.DeptsBean deptsBean : mtUserInfoRspDTO.getDepts()) {
                                    for (MTUserInfoRspDTO.DeptsBean.PathBean pathBean : deptsBean.getPath()) {
                                        if (null != pathBean)
                                            sb.append(pathBean.getName() + "-");
                                    }
                                }
                            }
                            if (!sb.toString().equals("")) {
                                String string = sb.toString().substring(0, sb.toString().length() - 1);
                                if(string.indexOf("-") != -1){
                                    string = string.substring(string.indexOf("-")+1);
                                }
                                bLessonPerson.setFullDept(string);
                            }
                        }
                    });
                });
            }
        }

        return bLessonPersonList;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> queryListAllLessonByPagination(PageUtil pageUtil, FindListLessonDTO model, List<AllCourseDTO> allCourse) {
        //---------获取部门------------
//        List<String> departs = getDeparts();
        List<String> ids = Arrays.asList(new String[]{model.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(model.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
//        departs.forEach(depart -> {
//            if (StringUtils.isEmpty(depart)) {
//                departs.remove(depart);
//            }
//        });
        //----------------------------
        model.setDeparts(departs);
        List<String> courseIds;
        if (allCourse != null) {
            //查询出所有子分类
            courseIds = Recursion.FindSons(model.getCourse(), allCourse);
            //将一级分类放入分类集合中
            if (StringUtils.isNotEmpty(model.getCourse())) {

                courseIds.add(model.getCourse());
                model.setCourseIds(courseIds);
            }
        }
        List<BLesson> ListAllLesson = null;
        //根据不同传值 查询列表页内容 searchType：查询类型 0 全部 1 推荐 2在学 3收藏"
        switch (model.getSearchType()) {
            case "0":
                ListAllLesson = bLessonRepository.queryListAllLessonByPagination(model);
                break;
            case "1":
                ListAllLesson = bLessonRepository.queryRecommendListByPagination(model);
                break;
            case "2":
                ListAllLesson = bLessonRepository.queryLearnListByPagination(model);
                break;
            case "3":
                ListAllLesson = bLessonRepository.queryCollectListByPagination(model);
                break;
            case "4":
                ListAllLesson = bLessonRepository.queryInterestedListByPagination(model);
                break;
            default:
                throw new ServiceException(ResultServiceEnums.INVALID_PARAMETER_VALUE);
        }

        return ListAllLesson;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public List<BLesson> theLessonMangageByPagination(PageUtil pageUtil, CurUser curUser, BLesson bLesson, List<RoleTypeDepartDTO> roleList) {
        //查询当前登录人所在的企业
        String corpId = curUser.getCorpId();
        String userId = curUser.getUserId();
        //查询当前登录人有什么角色
//        List<RoleTypeDepartDTO> roleList = sysUserMapper.queryRoleTypeByUserId(userId);
        if (roleList == null || roleList.size() < 1) {
            return new ArrayList<>();
        }
        boolean isAdmin = false;
        Set<String> uids = new HashSet<>();
        Set<String> departIds = new HashSet<>();
        for (RoleTypeDepartDTO role : roleList) {
            if ("1".equals(role.getType())) {
                isAdmin = true;
                break;
            } else if ("2".equals(role.getType()) && StringUtils.isNotEmpty(role.getDeparts())) {
                List<String> departs = asList(StringUtils.split(role.getDeparts(), ","));
                departIds.addAll(departs);
                // 调用 feign 获取当前部门及子部门 --->所有人
//                uids.addAll(Arrays.asList(new CurUser(), new CurUser()));
            }
        }
        if (!isAdmin){
            List<GetAllRspDTO> GetAllRspDTOList = cotactApiRequestClient.getUserByDeptIdSet(corpId,departIds);
            if (GetAllRspDTOList != null && GetAllRspDTOList.size()>0){
                for (GetAllRspDTO getAllRspDTO : GetAllRspDTOList) {
                    uids.add(getAllRspDTO.getId());
                }
            }
        }
        //自己
        uids.add(userId);
        //查讯课程的语句
        List<BLesson> bLessons = bLessonRepository.selectAllManageLesson(uids, bLesson, isAdmin, corpId);
        return bLessons;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel deleteRecommend(String lessonId) {

        List<String> recommendIds = bRecommendRepository.selectIdByLessonId(lessonId);
        if (recommendIds == null || recommendIds.size() < 1) {
            throw new ServiceException(ResultServiceEnums.NOT_RECOMMEND);
        }
        //删除推荐表
        BRecommend bRecommend = new BRecommend();
//        bRecommend.setLessonId(lessonId);
//        UserUtil.setCurrentMergeOperation(bRecommend);
        bRecommend.setBusinessId(recommendIds.get(0));
        int line = bRecommendRepository.deleteByPrimaryKey(bRecommend);
//        bRecommend.setDelFlag("1");
//        int line = bRecommendRepository.updateByPrimaryKeySelective(bRecommend);
        //删除推荐子表内容
        bRecommendRepository.deleteRecommendSon(bRecommend);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel setTrainIsSign(SetTrainIsSignDTO setTrainIsSignDTO) {
        BLessonPerson bLessonPerson = bLessonPersonRepository.selectById(setTrainIsSignDTO.getBusinessId());
        bLessonPerson.setTrainIsSign(setTrainIsSignDTO.getTrainIsSign());
        UserUtil.setCurrentMergeOperation(bLessonPerson);
        int line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel addCollect(CurUser curUser, String lessonId) {
        AddMyLearnLessonDTO param = new AddMyLearnLessonDTO(lessonId, curUser.getCorpId(), curUser.getUserId());
        List<BCollect> bLessonPersons = bCollectRepository.selectByPersonAndLesson(param);
//        if (bLessonPersons != null && bLessonPersons.size()>0){
//            BLessonPerson bLessonPerson = bLessonPersons.get(0);
//            bLessonPerson.setIsCollect("1");
//            bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
//        }
        //插入收藏表
        BCollect bCollect = new BCollect();
        bCollect.setLessonId(lessonId);
        bCollect.setPersonId(curUser.getUserId());
        int line = bCollectRepository.insertSelective(bCollect);
        //重新统计课程收藏数
        bLessonRepository.collectNumberReCount(lessonId);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public int toTraining(CurUser curUser, String lessonId) {
        BLessonPerson person ;
        BLessonPerson bLessonPerson = new BLessonPerson();
        //判断当前时间是否在报名范围内
        BLesson bLesson = bLessonRepository.selectInfoById(lessonId,curUser);
        if (!"1".equals(bLesson.getLessonType())){
            throw new ServiceException(ResultServiceEnums.NOT_TRAIN_TYPE);
        }

        Date newDate = new Date();
        if (newDate.before(bLesson.getApplyStartDate())||bLesson.getTrainOverDate().before(newDate)){
            throw new ServiceException(ResultServiceEnums.NOT_APPLY_DATE);
        }

        //step 1 查询数  x 当前报名数  y 总数
        synchronized (this) {
            //step2 判断该用户是否被指派该培训或已经报名
            person = bLessonPersonRepository.getRelInBLessonPerson(lessonId, curUser.getUserId());
            if (person != null && !"1".equals(person.getDelFlag())) {
                if ("1".equals(person.getIsApply())) {
                    log.info("-----------该培训已报名-------------");
                    throw new ServiceException(ResultServiceEnums.TRAIN_APPLYED);
                }
                person.setTrainIsSign("0");
                person.setApplyDate(new Date());
                person.setIsApply("1");
                int line = bLessonPersonRepository.updateLessonPersonByUserId(person);
                bLessonRepository.personNumberReCount(lessonId,"1");
                return line;
            }
            if (person != null && "1".equals(person.getDelFlag())){
                UserUtil.setCurrentMergeOperation(person);
                person.setDelFlag("0");
                person.setIsApply("1");
                person.setApplyDate(new Date());
                //TODO 判断积分是否够用
                MyInfoDTO userInfo =  sysUserMapper.getNameById(curUser.getUserId());
                if (userInfo == null){
                    throw new ServiceException(ResultServiceEnums.COMPANY_NOT_EXISTS);
                }
                if (bLesson.getValueConsume() != null && bLesson.getValueConsume() != 0 && Integer.parseInt(userInfo.getValue()) < bLesson.getValueConsume()){
                    throw new ServiceException(ResultServiceEnums.VALUE_NOT_ENOUGH);
                }
                int line = bLessonPersonRepository.updateByPrimaryKeySelective(person);
                //TODO 更新用户积分明细表  重进计算积分
                //扣除对应积分 添加用户积分详情 更新用户积分
                //增加用户积分详情
                if (bLesson.getValueConsume() != null && bLesson.getValueConsume() != 0){
                    BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"报名培训",bLesson.getValueConsume(),"1");
                    addValueEvent(curUser.getUserId(),bPersonValue,userInfo.getLockNum());
                }
                bLessonRepository.personNumberReCount(bLesson.getBusinessId(),"1");
                return line;
            }
            List<BLessonPerson> bLessonPersons = new ArrayList<>();
            UserUtil.setCurrentPersistOperation(bLessonPerson);
            bLessonPerson.setPersonId(curUser.getUserId());
            bLessonPerson.setLessonId(lessonId);
            bLessonPerson.setIsAppoint("0");
            bLessonPerson.setIsApply("1");
            bLessonPerson.setTrainIsSign("0");
            bLessonPerson.setApplyDate(new Date());
            bLessonPerson.setRemarks("");
            bLessonPersons.add(bLessonPerson);
            //插入  用户id 、培训id、指派状态、报名状态、培训状态、报名时间 、通用字段
//                int line = bLessonPersonRepository.insert(bLessonPerson);
            //TODO 判断积分是否够用
            MyInfoDTO userInfo =  sysUserMapper.getNameById(curUser.getUserId());
            if (userInfo == null){
                throw new ServiceException(ResultServiceEnums.COMPANY_NOT_EXISTS);
            }
            if (bLesson.getValueConsume() != null && bLesson.getValueConsume() != 0 && Integer.parseInt(userInfo.getValue()) < bLesson.getValueConsume()){
                throw new ServiceException(ResultServiceEnums.VALUE_NOT_ENOUGH);
            }
            int line = this.trainInsert("0",bLesson,bLessonPersons);
            //TODO 更新用户积分明细表  重进计算积分
            //扣除对应积分 添加用户积分详情 更新用户积分
            //增加用户积分详情
            if (bLesson.getValueConsume() != null && bLesson.getValueConsume() != 0){
                BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"报名培训",bLesson.getValueConsume(),"1");
                addValueEvent(curUser.getUserId(),bPersonValue,userInfo.getLockNum());
            }
            bLessonRepository.personNumberReCount(bLesson.getBusinessId(),"1");
            return line;

        }

    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel colletLesson(CurUser curUser, String lessonId) {

        //先判断是否收藏该课程或培训
        List<BCollect> one = bCollectRepository.selectOneData(curUser.getUserId(),lessonId);
        if(one.size() > 0 ){
            return new PersistModel(1);
        }
        BCollect bCollect = new BCollect();
        bCollect.setBusinessId(IdGen.uuid());
        bCollect.setLessonId(lessonId);
        bCollect.setPersonId(curUser.getUserId());

        int line = bCollectRepository.insert(bCollect);
        //更新课程表收藏个数
        bLessonRepository.collectNumberReCount(lessonId);
        return new PersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel notColletLesson(CurUser curUser, String lessonId) {
        int line = bCollectRepository.notColletLesson(curUser.getUserId(),lessonId);
        bLessonRepository.collectNumberReCount(lessonId);
        return new PersistModel(line);
    }

    @Override
    public Map<String,Object> getLessonState(String id) {
        BLesson state = bLessonRepository.getLessonState(id);
        if(state==null){
            throw new ServiceException(ResultServiceEnums.NO_DATA);
        }
        Map<String,Object> map = new HashedMap();
        if(!state.getDelFlag().equals("0")){
            map.put("massage","课程被删除");
            map.put("code","101");
        }else if(state.getReleaseState().equals("4")){
            map.put("massage","课程被关闭");
            map.put("code","102");
        }else{
            map.put("massage","课程状态正常");
            map.put("code","100");
        }

        return map;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public BLesson findOneMyTrain(CurUser curUser, String lessonId) {

        BLesson bLesson = bLessonRepository.findOneMyTrain(curUser, lessonId);
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        //获取讲师部门
        List<String> ids = new ArrayList<>();
        ids.add(bLesson.getLecturerId());
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                if (mtUserInfoRspDTO.getDepts() != null && mtUserInfoRspDTO.getDepts().size() > 0 && mtUserInfoRspDTO.getId().equals(bLesson.getLecturerId())){
                    //设置部门名
                    bLesson.setLecturerDeptName(mtUserInfoRspDTO.getDepts().get(0).getName());
                }
            });
        }

        //查询报名人数是否小于规定人数
        int trainApplyCount = bLessonRepository.trainApplyCount(bLesson.getBusinessId());
        if (StringUtils.isNotEmpty(bLesson.getMaxApplyPerson()) && !"0".equals(bLesson.getMaxApplyPerson()) && trainApplyCount >= Integer.parseInt(bLesson.getMaxApplyPerson()) ){
            bLesson.setApplyIsFull("1");
        }else {
            bLesson.setApplyIsFull("0");
        }
        //查询是否收藏
        int collectCount = bCollectRepository.selectCountbyLessonAndUser(curUser,bLesson.getBusinessId());
        if (collectCount>0){
            bLesson.setIsCollect("1");
        }else {
            bLesson.setIsCollect("0");
        }
        //判断报名、签到是否开始
        setDateState(bLesson);
        //添加至b_hot表 并重新统计热度
        int count = bHotRepository.selectCountByPersonId(curUser.getUserId(),lessonId);
        if (count < 1){
            bHotRepository.insertSelective(new BHot(IdGen.uuid(),curUser.getUserId(),lessonId));
            bLessonRepository.hotNumberReCount(lessonId);
        }
        return bLesson;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public BLesson findOneMyTeachTrain(CurUser curUser, String lessonId) {
        BLesson bLesson = bLessonRepository.findOneMyTeachTrain(curUser, lessonId);
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        int trainApplyCount = bLessonRepository.trainApplyCount(bLesson.getBusinessId());
        if (StringUtils.isNotEmpty(bLesson.getMaxApplyPerson()) && !"0".equals(bLesson.getMaxApplyPerson()) && trainApplyCount >= Integer.parseInt(bLesson.getMaxApplyPerson()) ){
            bLesson.setApplyIsFull("1");
        }else {
            bLesson.setApplyIsFull("0");
        }
        int collectCount = bCollectRepository.selectCountbyLessonAndUser(curUser,bLesson.getBusinessId());
        if (collectCount>0){
            bLesson.setIsCollect("1");
        }else {
            bLesson.setIsCollect("0");
        }
        //判断签到是否开始
        setDateState(bLesson);
        return bLesson;
    }

    @Override
    public List<SysUser> findAppointPerson(CurUser curUser, String lessonId) {
        List<BAppoint> bAppoints = bAppointRepository.selectByLessonId(lessonId);
        if (bAppoints == null || bAppoints.size()<1){
            return new ArrayList<>();
        }
        BAppoint bAppoint = bAppoints.get(0);
        List<String> ids = asList(org.apache.commons.lang.StringUtils.split(bAppoint.getDesignatedId(), ","));
        if (ids == null || ids.size() < 1) {
            return new ArrayList<>();
        }
        return sysUserMapper.selectByIds(ids);
    }

    @Override
    public List<BLesson> queryCollectByPagination(PageUtil pageUtil, String lessonType, CurUser curUser) {
        return bLessonRepository.selectMyCollect(lessonType, curUser);
    }

    @Override
    public Map<String,Object> judgeVisibleAuthority(CurUser curUser, String lessonId) {
        BLesson bLesson = bLessonRepository.selectInfoById(lessonId,curUser);
        if (bLesson == null){
            Map<String,Object> resMap = new HashMap<>();
            resMap.put("code",44);
            return resMap;
        }

        List<String> ids = Arrays.asList(new String[]{curUser.getUserId()});
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = cotactApiRequestClient.userGets(mtUserGetsReqDTO);
        List<String> departs = new ArrayList<>();
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            departs = QueryDepart.queryDepart(mtUserInfoRspDTOList);
            departs.removeAll(Collections.singleton(null));
            departs.removeAll(Collections.singleton(""));
        }
        //若已加入该课程中 不需要在可见范围内
        AddMyLearnLessonDTO addMyLearnLessonDTO = new AddMyLearnLessonDTO(lessonId,curUser.getCorpId(),curUser.getUserId());
        List<BLessonPerson>  myLearn = bLessonPersonRepository.selectByPersonAndLesson(addMyLearnLessonDTO);
        if (myLearn != null && myLearn.size() > 0 && myLearn.get(0) != null ){
            Map<String,Object> resMap = new HashMap<>();
            resMap.put("code",200);
            return resMap;
        }

        int result = bLessonRepository.isInViewRange(curUser,lessonId,departs);
        if (result < 1){
            Map<String,Object> resMap = new HashMap<>();
            resMap.put("code",132);
            return resMap;
        }
        Map<String,Object> resMap = new HashMap<>();
        resMap.put("code",200);
        return resMap;
    }

    @Override
    public List<StudentTrackingRspDTO> userManageLessonByPagination(PageUtil pageUtil, UserLessonDTO userLessonDTO) {
        String trainTypeSelStr = "";
        if ("0".equals(userLessonDTO.getTrainType())){
            trainTypeSelStr = "and now() < SUBDATE(bl.train_start_date,interval bl.train_sign_time MINUTE)";
        }
        if ("1".equals(userLessonDTO.getTrainType())){
            trainTypeSelStr = "and now() BETWEEN SUBDATE(bl.train_start_date,interval bl.train_sign_time MINUTE) and bl.train_over_date";
        }
        if ("2".equals(userLessonDTO.getTrainType())){
            trainTypeSelStr = "and now() > bl.train_over_date";
        }

        return bLessonPersonRepository.userManageLesson(userLessonDTO,trainTypeSelStr);
    }

    @Override
    public List<ExamDto> userManageExamByPagination(PageUtil pageUtil, ExamQueryDto dto) {
        List<ExamDto> list = bLessonPersonRepository.userManageExam(dto);
        if(list.size()>0){
            for(ExamDto examDto : list){
                //考试状态 0待参加 1已完成 2已失效 3进行中
                if(examDto.getPaperStatus() == null && examDto.getPaperEndTime().after(new Date())){
                    examDto.setPaperStatus("待参加");
                    examDto.setExamResult(null);
                }else if(examDto.getPaperStatus() == null && examDto.getPaperEndTime().before(new Date())) {
                    examDto.setPaperStatus("已失效");
                    examDto.setExamResult(null);
                }else if("1".equals(examDto.getPaperStatus()) || "2".equals(examDto.getPaperStatus())){
                    examDto.setPaperStatus("已完成");
                }else if("0".equals(examDto.getPaperStatus())){
                    examDto.setPaperStatus("进行中");
                    examDto.setScore(null);
                    examDto.setExamResult(null);
                }
            }
        }
        return list;
    }

    //遍历标签放入List中
    List<BLessonLabel> addLabel(BLesson model) {
        //标签集合
        List<BLessonLabel> bLessonLabels = new ArrayList<>();
        String[] labels = model.getLabels().split(",");
        if (labels.length > 3){
            throw new ServiceException(ResultServiceEnums.LABEL_MORE_NUM);
        }
        int sortNum = 0;
        for (int index = 0; index < labels.length; index++) {
            BLessonLabel bLessonLabel = new BLessonLabel();
            UserUtil.setCurrentPersistOperation(bLessonLabel);
            bLessonLabel.setLabelId(labels[index]);
            bLessonLabel.setLessonId(model.getBusinessId());
            bLessonLabel.setSort(String.valueOf(sortNum));
            bLessonLabels.add(bLessonLabel);
            sortNum++;
        }
        return bLessonLabels;
    }

    //遍历 将可见范围添加值可见范围子表集合中
    List<BViewrangeSon> addBViewrangeSon(List<BViewrangeSon> bViewrangeSons, String[] viewRanges, String type, String viewRangeId) {
        for (int index = 0; index < viewRanges.length; index++) {
            BViewrangeSon bViewrangeSon = new BViewrangeSon();
            bViewrangeSon.setBusinessId(IdGen.uuid());
            bViewrangeSon.setTargetId(viewRanges[index]);
            bViewrangeSon.setType(type);
            bViewrangeSon.setViewrangeId(viewRangeId);
            bViewrangeSon.setSort(String.valueOf(index));
            bViewrangeSons.add(bViewrangeSon);
        }
        return bViewrangeSons;
    }

    //遍历 将推荐添加值推荐子表集合中
    List<BRecommendSon> addBRecommendSon(List<BRecommendSon> bRecommendSons, String[] recommends, String type, String recommendId) {
        for (int index = 0; index < recommends.length; index++) {
            BRecommendSon bRecommendSon = new BRecommendSon();
            bRecommendSon.setBusinessId(IdGen.uuid());
            bRecommendSon.setTargetId(recommends[index]);
            bRecommendSon.setType(type);
            bRecommendSon.setRecommendId(recommendId);
            bRecommendSon.setSort(String.valueOf(index));
            bRecommendSons.add(bRecommendSon);
        }
        return bRecommendSons;
    }

    //设置BViewrange信息
    BViewrange setBViewrangeInfo(BLesson model, BViewrange bViewrange) {
        if (bViewrange == null){
            bViewrange = new BViewrange();
            UserUtil.setCurrentPersistOperation(bViewrange);
            //设置课程id
            bViewrange.setLessonId(model.getBusinessId());
        }else {
            UserUtil.setCurrentMergeOperation(bViewrange);
        }


        //设置可见范围类型（0仅人 1仅部门 2人和部门）
        if (StringUtils.isNotEmpty(model.getViewRangePerson()) && StringUtils.isEmpty(model.getViewRangeDepart())) {
            bViewrange.setType("0");
        } else if (StringUtils.isEmpty(model.getViewRangePerson()) && StringUtils.isNotEmpty(model.getViewRangeDepart())) {
            bViewrange.setType("1");
        } else if (StringUtils.isNotEmpty(model.getViewRangePerson()) && StringUtils.isNotEmpty(model.getViewRangeDepart())) {
            bViewrange.setType("2");
        }
        return bViewrange;
    }

    void setDateState(BLesson bLesson){
        //判断签到是否开始
        Date nowDate = new Date();
        if (bLesson.getTrainStartDate() != null && bLesson.getTrainOverDate() !=null && StringUtils.isNotEmpty(bLesson.getTrainSignTime())){
            Long time = Long.parseLong(bLesson.getTrainSignTime()) * 60 * 1000;
            Date signStart = new Date(bLesson.getTrainStartDate().getTime() - time);
            if (signStart.before(nowDate) && nowDate.before(bLesson.getTrainOverDate())){
                bLesson.setTrainDateState("1");
            }
            if (nowDate.before(signStart)){
                bLesson.setTrainDateState("0");
            }
            if (bLesson.getTrainOverDate().before(nowDate)){
                bLesson.setTrainDateState("2");
            }
        }
        if (bLesson.getApplyStartDate() != null && bLesson.getApplyOverDate() != null){
            if (bLesson.getApplyStartDate().before(nowDate) && nowDate.before(bLesson.getApplyOverDate())){
                bLesson.setIsApplyDate("1");
            }
            if (nowDate.before(bLesson.getApplyStartDate())){
                bLesson.setIsApplyDate("0");
            }
            if (bLesson.getApplyOverDate().before(nowDate)){
                bLesson.setIsApplyDate("2");
            }
        }
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false,rollbackFor= Exception.class)
    //type: 0报名 1指派
    synchronized int trainInsert(String type, BLesson bLesson, List<BLessonPerson> bLessonPersons){
        int line = 0;
        if ("0".equals(type)){
            Map<String, Object> xy = bLessonPersonRepository.selecTraningNum(bLesson.getBusinessId());

            int x = Integer.parseInt(String.valueOf(xy.get("x")));
            int y = Integer.parseInt(String.valueOf(xy.get("y")));
            if (y > 0 && x >= y) {
                throw new ServiceException(ResultServiceEnums.PERSON_MAX);
            }
            line = bLessonPersonRepository.insert(bLessonPersons.get(0));
//            bLessonRepository.personNumberReCount(bLesson.getBusinessId(),"1");
            return line;
        }
        if ("1".equals(type)){
            //如果为培训的话 判断剩余学生加已有学生总数超过最大数 抛异常
            if ("1".equals(bLesson.getLessonType())){
                //查找已报名总人数
                int personNumber = bLessonRepository.personNumberCount(bLesson.getBusinessId());
                if (Integer.parseInt(bLesson.getMaxApplyPerson()) > 0 && personNumber + bLessonPersons.size() > Integer.parseInt(bLesson.getMaxApplyPerson())){
                    throw new ServiceException(400,"培训人数上限"+bLesson.getMaxApplyPerson()+"，现已超出"+String.valueOf(personNumber + bLessonPersons.size()- Integer.parseInt(bLesson.getMaxApplyPerson())+"人，请重新选择"));
                }
                line = bLessonPersonRepository.insertList(bLessonPersons);
//                bLessonRepository.personNumberReCount(bLesson.getBusinessId(),"1");
                return line;
            }
        }
        return line;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false,rollbackFor= Exception.class)
    public void addValueEvent(String personId, BPersonValue bPersonValue, Integer lockNum){
        if (StringUtils.isNotEmpty(personId) && bPersonValue != null && bPersonValue.getValue() != null && bPersonValue.getValue() != 0){
            UserUtil.setCurrentPersistOperation(bPersonValue);
            bPersonValueRepository.add(bPersonValue);
            //积分变化后 s_user表中value字段重新统计
            bPersonValueRepository.updateUserValue(personId, lockNum);
        }
    }

}
