package org.rcisoft.business.bmaterial.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bfile.entity.BTrainFile;
import org.rcisoft.business.bmaterial.dto.DelMoreDTO;
import org.rcisoft.business.bmaterial.dto.FindDTO;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.business.bmaterial.entity.BMaterial;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.stereotype.Repository;


import java.util.List;

/**
* Created with  on 2019-10-12 10:38:31.
*/
@Repository
public interface BMaterialRepository extends BaseMapper<BMaterial> {

    /**
    * 分页查询 bMaterial
    *
    */
    @Select("<script>select * from b_material where 1=1 "
            + "<if test=\"delFlag !=null and delFlag != '' \">and del_flag = #{delFlag} </if> "
            + "<if test=\"flag !=null and flag != '' \">and flag = #{flag} </if> "
            + "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BMaterial> queryBMaterials(BMaterial bMaterial);



    @Insert("<script>insert into b_material " +
            " (business_id,chapter_id,file_name,file_url, " +
            " type,upload_type,file_size,flag,del_flag, " +
            " remarks,create_by,update_by,update_date,create_date,file_time,lesson_id) " +
            " values(#{businessId},#{chapterId},#{fileName},#{fileUrl}, " +
            " #{type},#{uploadType},#{fileSize},#{flag},#{delFlag}, " +
            " #{remarks},#{createBy},#{updateBy},#{updateDate},#{createDate},#{fileTime},#{lessonId}) " +
            "</script>")
    int insertMaterial(BMaterial bMaterial);

    @Select("<script>select bm.*,bl.lesson_name lessonName, bl.lesson_type lessonType, " +
            " CONCAT(if(bl.lesson_type='0','课程：','培训：'),bl.lesson_name) source " +
            " from b_material bm" +
            " left join b_chapter bc on bm.chapter_id = bc.business_id " +
            " left join b_lesson bl on bc.lesson_id = bl.business_id " +
            " where 1=1 " +
            " and bm.del_flag != 1 and bm.flag = 1 " +
//            " and bc.del_flag != 1 and bc.flag = 1 " +
//            " and bl.del_flag != 1 and bl.flag = 1 " +
            " <if test=\"param.uploadType != null and param.uploadType != ''\">and bm.upload_type = #{param.uploadType}</if>" +
            " and bm.create_by = #{param.userId} "+
            "<if test= \" param.typeSort !=null and param.typeSort !='' and param.typeSort =='0'.toString()\">  ORDER BY bm.type DESC  </if> "+
            "<if test= \" param.typeSort !=null and param.typeSort !='' and param.typeSort =='1'.toString()\">  ORDER BY bm.type ASC  </if> "+
            "<if test= \" param.sizeSort !=null and param.sizeSort !='' and param.sizeSort =='0'.toString()\">  ORDER BY CAST(bm.file_size AS UNSIGNED) DESC  </if> "+
            "<if test= \" param.sizeSort !=null and param.sizeSort !='' and param.sizeSort =='1'.toString()\">  ORDER BY CAST(bm.file_size AS UNSIGNED) ASC </if> "+
            "<if test= \" param.sourceSort !=null and param.sourceSort !='' and param.sourceSort =='0'.toString()\"> ORDER BY CAST(bl.lesson_type AS UNSIGNED)  DESC  </if> "+
            "<if test= \" param.sourceSort !=null and param.sourceSort !='' and param.sourceSort =='1'.toString()\"> ORDER BY CAST(bl.lesson_type AS UNSIGNED) ASC </if> "+
            "<if test= \" param.dateSort !=null and param.dateSort !='' and param.dateSort =='0'.toString()\">  ORDER BY bm.create_date DESC  </if> "+
            "<if test= \" param.dateSort !=null and param.dateSort !='' and param.dateSort =='1'.toString()\">  ORDER BY bm.create_date ASC </if> "+
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BMaterial> queryMyBMaterialByPagination(@Param("param") FindDTO findDTO);


    /**
     * 根据trainFileList中的URL 查询b_material表中已存在的url
     * @param trainFileList
     * @return
     */
    @Select("<script>select * from b_material  " +
            " where 1=1 " +
            " and del_flag != 1 and flag = 1 " +
            " <if test=\"uploadType!=null and uploadType != ''\">and upload_type = #{uploadType}</if>" +
            " and file_url in  "+
            "<foreach item='item'  collection='trainFileList' open='(' separator=',' close=')'> #{item.videoUrl} </foreach> " +
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BMaterial> materialHaveUrl(@Param("trainFileList") List<BTrainFile> trainFileList, @Param("uploadType") String uploadType);


    @Insert("<script>insert into b_material " +
            " (business_id,chapter_id,file_name,file_url, " +
            " type,upload_type,file_size,flag,del_flag, " +
            " remarks,create_by,update_by,update_date,create_date,file_time,lesson_id) " +
            " values" +
            "<foreach collection=\"bMaterial\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.chapterId},#{item.fileName},#{item.fileUrl}, " +
            " #{item.type},#{item.uploadType},#{item.fileSize},#{item.flag},#{item.delFlag}, " +
            " #{item.remarks},#{item.createBy},#{item.updateBy},#{item.updateDate},#{item.createDate},#{item.fileTime},#{item.lessonId}) " +
            "</foreach></script>")
    int insertMaterialList(@Param("bMaterial") List<BMaterial> bMaterial);

    /**
     * 批量删除
     * @param delMoreDTO
     * @return
     */
    @Update({"<script>",
            "update b_material",
            "  <set>",
            "    <if test='param.updateBy != null'>update_by=#{param.updateBy},</if>",
            "    <if test='param.updateDate != null'>update_date=#{param.updateDate},</if>",
            "    <if test='param.delFlag != null'>del_flag=#{param.delFlag},</if>",
            "    <if test='param.flag != null'>flag=#{param.flag},</if>",
            "  </set>",
            "where business_id in ",
            "<foreach item='item' index='index' collection='param.idList' open='(' separator=',' close=')'> #{item} </foreach> ",
            "</script>"})
    int deleteMore(@Param("param")DelMoreDTO delMoreDTO);

    @Select("<script>select * from b_material  " +
            " where 1=1 " +
            " and del_flag != 1 and flag = 1 " +
            " <if test=\"uploadType!=null and uploadType != ''\">and upload_type = #{uploadType}</if>" +
            " and file_url in  "+
            "<foreach item='item'  collection='urls' open='(' separator=',' close=')'> #{item} </foreach> " +
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BMaterial> selectByUrl(@Param("urls") List<String> urls, @Param("uploadType") String uploadType);

}

