package org.rcisoft.business.breleasevalue.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.breleasevalue.dto.*;
import org.rcisoft.business.breleasevalue.entity.BReleaseValueSon;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.business.breleasevalue.entity.BReleaseValue;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.stereotype.Repository;


import java.util.List;

/**
* Created with  on 2019-11-13 13:28:38.
*/
@Repository
public interface BReleaseValueRepository extends BaseMapper<BReleaseValue> {

    /**
    * 分页查询 bReleaseValue
    *
    */
    @Select("<script>" +
            "select * from b_release_value  " +
            " where  del_flag != 1 and flag = 1 " +
            " and corp_id = #{corpId} " +
            "<if test=\"title!=null and title != ''\"> and title like concat('%',#{title},'%')  </if> " +
            "<if test=\"id!=null and id != ''\"> and business_id = #{id} </if> " +
            "<if test=\"lessonValueStart!=null and lessonValueStart != ''\"> and lesson_value &gt;= #{lessonValueStart}  </if> " +
            "<if test=\"lessonValueEnd!=null and lessonValueEnd != ''\"> and lesson_value &lt;= #{lessonValueEnd}  </if> " +
            "<if test=\"trainValueStart!=null and trainValueStart != ''\"> and train_value &gt;= #{trainValueStart}  </if> " +
            "<if test=\"trainValueEnd!=null and trainValueEnd != ''\"> and train_value &lt;= #{trainValueEnd}  </if> " +
            " ORDER BY corp_id " +
            "<if test=\"lessonSort!=null and lessonSort != '' and lessonSort == '0'.toString()\"> ,lesson_value asc </if> " +
            "<if test=\"lessonSort!=null and lessonSort != '' and lessonSort == '1'.toString()\"> ,lesson_value desc </if> " +
            "<if test=\"trainSort!=null and trainSort != '' and trainSort == '0'.toString()\"> ,train_value asc </if> " +
            "<if test=\"trainSort!=null and trainSort != '' and trainSort == '1'.toString()\"> ,train_value desc </if> " +
            "<if test=\"timeSort!=null and timeSort != '' and timeSort == '0'.toString()\"> ,update_date asc </if> " +
            "<if test=\"timeSort!=null and timeSort != '' and timeSort == '1'.toString()\"> ,update_date desc </if> " +
            "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BReleaseValue> queryBReleaseValues(ReleaseValueResDto dto);

    /**
     * 插入子表
     * @param list
     * @return
     */
    @Insert("<script>" +
            "insert into b_release_value_son (business_id,release_id,user_id) VALUES " +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            " (#{item.businessId},#{item.releaseId},#{item.userId}) " +
            "</foreach></script>")
    int insertIntoSon (@Param("list") List<BReleaseValueSon> list);

    @Update("update b_release_value set del_flag= 1 where business_id = #{releaseId} and corp_id = #{corpId}")
    int remove(@Param("releaseId") String releaseId,@Param("corpId") String corpId);

    /**
     * 根据用户名删除子表中相关数据
     * @param userId
     * @return
     */
    @Delete("<script><foreach collection=\"userId\" item=\"item\" separator=\";\">" +
            " delete from b_release_value_son where user_id = #{item}" +
            "</foreach></script>")
    int deleteSonForUserId(@Param("userId") String[] userId);

    /**
     * 根据releaseId删除子表中相关数据
     * @param releaseId
     * @return
     */
    @Delete("delete from b_release_value_son where release_id = #{releaseId}")
    int deleteSonForReleaseId(@Param("releaseId") String releaseId);

    /**
     * xml中查询关联用
     * @param releaseId
     * @return
     */
    @Select(" select su.name name " +
            " from b_release_value_son rvs " +
            " left join s_user su on su.business_id = rvs.user_id " +
            " where  rvs.release_id = #{releaseId} ")
    List<String> getNameforReleaseId(@Param("releaseId")String releaseId);
    /**
     * xml中查询关联用
     * @param releaseId
     * @return
     */
    @Select(" select su.name name,su.business_id id " +
            " from b_release_value_son rvs " +
            " left join s_user su on su.business_id = rvs.user_id " +
            " where  rvs.release_id = #{releaseId} ")
    List<UserListDto> getAllforReleaseId(@Param("releaseId")String releaseId);

    /**
     * 查询课程或培训积分设置
     * @param curUser
     * @param dto
     * @return
     */
    @Select("<script>" +
            "select bl.business_id  businessId,bl.lesson_name lessonName,su.name name,bl.release_state releaseState," +
            "bl.value_consume valueConsume,bl.value_gain valueGain ,bl.value_update_date updateDate ,bl.train_start_date trainStart ,bl.train_over_date trainEnd    " +
            "from b_lesson  bl    " +
            "left join s_user su on su.business_id = bl.lecturer_id  " +
            "where bl.del_flag !=1 and bl.flag = 1  " +
            "<if test=\"dto.valueConsumeStart!=null and dto.valueConsumeStart != ''\"> and bl.value_consume  &gt;= #{dto.valueConsumeStart}    </if> " +
            "<if test=\"dto.valueConsumeEnd!=null and dto.valueConsumeEnd != ''\">  and bl.value_consume &lt;= #{dto.valueConsumeEnd}  </if> " +
            "<if test=\"dto.valueGainStart!=null and dto.valueGainStart != ''\"> and bl.value_gain &gt;= #{dto.valueGainStart} </if> " +
            "<if test=\"dto.valueGainEnd!=null and dto.valueGainEnd != ''\"> and bl.value_gain &lt;= #{dto.valueGainEnd}  </if> " +
            "<if test=\"dto.lessonName!=null and dto.lessonName != ''\"> and bl.lesson_name like concat('%',#{dto.lessonName},'%')  </if> " +
            "<if test=\"dto.lecturerName!=null and dto.lecturerName != ''\"> and su.`name` like concat('%',#{dto.lecturerName},'%')  </if> " +
            "<if test=\"dto.status!=null and dto.status != ''\"> and bl.release_state in(${dto.status}) </if> " +
            "<if test=\"dto.trainStatus!=null and dto.trainStatus != ''\"> and ( ${dto.trainStatus} )</if> " +
            "and bl.corp_id = #{curUser.corpId}   " +
            "and bl.lesson_type = #{dto.type} " +
            "</script>")
    List<LessonValueDto> getLessonValue(@Param("curUser") CurUser curUser, @Param("dto") LessonValueSettingQueryDto dto);

    /**
     * 设置课程或培训 积分
     * @param businessId
     * @param valueConsume
     * @param valueGain
     * @return
     */
    @Update("update b_lesson set value_consume = #{valueConsume} ,value_gain = #{valueGain} ,value_update_date = now() where business_id = #{businessId}")
    int updateLessonValue(@Param("businessId") String businessId,@Param("valueConsume") String valueConsume,@Param("valueGain") String valueGain);

    /**
     * 查询用户积分信息
     * @param dto
     * @param userList
     * @return
     */
    @Select("<script>select business_id businessId, name , `value` value ," +
            "(select sum(`value`) from b_person_value where person_id = s_user.business_id and type = 0) valueGain " +
            " from s_user   " +
            " where 1=1   and del_flag !=1    and flag = 1  " +
            " and corp_id = #{dto.corpId} " +
            "<if test=\"userList!=null and userList.size() > 0 \"> " +
            " and business_id in <foreach item='item' index='index' collection='userList' open='(' separator=',' close=')'> #{item} </foreach>" +
            "</if>" +
            "<if test=\"dto.name!=null and dto.name != ''\"> and name like concat('%',#{dto.name},'%') </if> " +
            "<if test=\"dto.nowValueStart!=null and dto.nowValueStart != ''\"> and value &gt;= #{dto.nowValueStart}  </if> " +
            "<if test=\"dto.nowValueEnd!=null and dto.nowValueEnd != ''\"> and value &lt;= #{dto.nowValueEnd}  </if> " +
            "<if test=\"dto.gainValueStart!=null and dto.gainValueStart != ''\">" +
            "   and (select sum(value) from b_person_value where person_id = s_user.business_id and type = 0) &gt;= #{dto.gainValueStart}  </if> " +
            "<if test=\"dto.gainValueEnd!=null and dto.gainValueEnd != ''\"> " +
            "   and (select sum(value) from b_person_value where person_id = s_user.business_id and type = 0) &lt;= #{dto.gainValueEnd}  </if> " +
            "ORDER BY corp_id " +
            "<if test=\"dto.nowValueSort!=null and dto.nowValueSort != '' and dto.nowValueSort == '0'.toString()\"> ,`value` asc  </if> " +
            "<if test=\"dto.nowValueSort!=null and dto.nowValueSort != '' and dto.nowValueSort == '1'.toString()\"> ,`value` desc  </if> " +
            "<if test=\"dto.gainValueSort!=null and dto.gainValueSort != '' and dto.gainValueSort == '0'.toString()\"> ,valueGain asc  </if> " +
            "<if test=\"dto.gainValueSort!=null and dto.gainValueSort != '' and dto.gainValueSort == '1'.toString()\"> ,valueGain desc  </if> " +
            " </script>")
//    @ResultMap(value = "UserValueDtoMap")
    List<UserValueDto> getUserValue(@Param("dto") UserValueResDto dto, @Param("userList")List<String> userList);

    /**
     * xml中引用
     * @param businessId
     * @return
     */
    @Select("select sum(value) valueConsume from b_person_value where type = 1 and del_flag !=1 and flag = 1 and person_id = #{businessId}")
    String getValueConsume(@Param("businessId")String businessId);

    @Select("<script>" +
            "select rv.business_id from b_release_value  rv " +
            "left join b_release_value_son  rvs on rv.business_id = rvs.release_id " +
            "left join s_user su  on su.business_id = rvs.user_id " +
            "where  rv.del_flag != 1 and rv.flag = 1  " +
            "and su.name like concat('%',#{name},'%')" +
            "and rv.corp_id = #{corpId}" +
            "</script>")
    String getReleaseId (@Param("name") String name,@Param("corpId")String corpId);

    @Select("<script>" +
            "select rv.lesson_value lessonValue, rv.train_value trainValue " +
            "from b_release_value  rv " +
            "left join b_release_value_son  rvs on rv.business_id = rvs.release_id " +
//            "left join s_user su  on su.business_id = rvs.user_id " +
            "where  rv.del_flag != 1 and rv.flag = 1  " +
            "and rvs.user_id = #{curUser.userId} " +
            "and rv.corp_id = #{curUser.corpId} " +
            "</script>")
    List<BReleaseValue> selectReleaseValueByPersonId(@Param("curUser")CurUser curUser);
}

