package org.rcisoft.business.breleasevalue.service.impl;

import org.rcisoft.business.breleasevalue.dto.*;
import org.rcisoft.business.breleasevalue.entity.BReleaseValueSon;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.breleasevalue.dao.BReleaseValueRepository;
import org.rcisoft.business.breleasevalue.entity.BReleaseValue;
import org.rcisoft.business.breleasevalue.service.BReleaseValueService;


import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-11-13 13:28:38.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BReleaseValueServiceImpl implements BReleaseValueService {

    @Autowired
    private BReleaseValueRepository bReleaseValueRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    @Override
    public PersistModel save(BReleaseValueDto bReleaseValueDto) {

        if(bReleaseValueDto.getBusinessId() != null){
// 编辑
            BReleaseValue bReleaseValue = new BReleaseValue();
            BeanUtils.copyProperties(bReleaseValueDto, bReleaseValue);
            UserUtil.setCurrentMergeOperation(bReleaseValue);
            int line = bReleaseValueRepository.updateByPrimaryKeySelective(bReleaseValue);
            // 删除子表中的相关记录
            bReleaseValueRepository.deleteSonForReleaseId(bReleaseValueDto.getBusinessId());
            this.splitUser(bReleaseValueDto.getUserIds(),bReleaseValueDto.getBusinessId());
            return new PersistModel(line);
        }
//新增
        BReleaseValue bReleaseValue = new BReleaseValue();
        BeanUtils.copyProperties(bReleaseValueDto, bReleaseValue);
        UserUtil.setCurrentPersistOperation(bReleaseValue);

        this.splitUser(bReleaseValueDto.getUserIds(),bReleaseValue.getBusinessId());
        int line = bReleaseValueRepository.insert(bReleaseValue);
        return new PersistModel(line);
    }

    @Override
    public List<BReleaseValue> queryByPagination(PageUtil pageUtil, ReleaseValueResDto dto) {
        if(dto.getName() != null && !"".equals(dto.getName())){
            String id  = bReleaseValueRepository.getReleaseId(dto.name,dto.getCorpId());
            dto.setId(id);
        }
        List<BReleaseValue> list = bReleaseValueRepository.queryBReleaseValues(dto);
        return list;
    }

    @Override
    public PersistModel delete(String businessId ,String corpId) {
//        删除发布积分表中的数据 del_flag = 1
        int line = bReleaseValueRepository.remove(businessId,corpId) ;
//        删除发布积分子表中与ReleaseId有关的数据
        bReleaseValueRepository.deleteSonForReleaseId(businessId);
        return new PersistModel(line);
    }

    @Override
    public List<LessonValueDto> queryLessonValueByPagination(PageUtil pageUtil, CurUser curUser, LessonValueSettingQueryDto dto) {
        try{
            if(dto.getValueConsumeEnd()!=null && !"".equals(dto.getValueConsumeEnd())){
                Integer.parseInt(dto.getValueConsumeEnd());
            }
            if(dto.getValueConsumeStart()!=null && !"".equals(dto.getValueConsumeStart())){
                Integer.parseInt(dto.getValueConsumeStart());
            }
            if(dto.getValueGainEnd()!=null && !"".equals(dto.getValueGainEnd())){
                Integer.parseInt(dto.getValueGainEnd());
            }
            if(dto.getValueGainStart()!=null && !"".equals(dto.getValueGainStart())){
                Integer.parseInt(dto.getValueGainStart());
            }
        }catch (Exception e){
            throw  new ServiceException(ResultServiceEnums.NUM_IS_ERROR);
        }
        if(dto.getTrainStatus() != null && !"".equals(dto.getTrainStatus())){
            String[] trainStatus = dto.getTrainStatus().split(",");
            StringBuffer sb = new StringBuffer(" 1 != 1");
            for(String str : trainStatus){
                if("0".equals(str)){
                    sb.append("  or bl.train_start_date > now()");
                }if("1".equals(str)){
                    sb.append(" or now() BETWEEN bl.train_start_date and bl.train_over_date");
                }if("2".equals(str)){
                    sb.append(" or bl.train_over_date < now() ");
                }
            }
            dto.setTrainStatus(sb.toString());
        }else {
            dto.setTrainStatus(null);
        }


        List<LessonValueDto> list = bReleaseValueRepository.getLessonValue(curUser,dto);
        for(LessonValueDto valueDto :list){
            //0未发布 1审核 2已发布 3驳回 4关闭
            if("0".equals(valueDto.getReleaseState())){
                valueDto.setReleaseState("未发布");
            }else if("2".equals(valueDto.getReleaseState())){
                if("0".equals(dto.getType())){
                    valueDto.setReleaseState("已发布");
                }else{
                    if(valueDto.getTrainStart().after(new Date())){
                        valueDto.setReleaseState("已发布(未开始)");
                    }else if(valueDto.getTrainStart().before(new Date()) && valueDto.getTrainEnd().after(new Date())){
                        valueDto.setReleaseState("已发布(进行中)");
                    }else if(valueDto.getTrainEnd().before(new Date())){
                        valueDto.setReleaseState("已发布(已结束)");
                    }
                }
            }else if("4".equals(valueDto.getReleaseState())){
                valueDto.setReleaseState("已关闭");
            }
        }
        return list;
    }

    @Override
    public PersistModel updateLessonValue(String businessId, String valueConsume, String valueGain) {
        int line = bReleaseValueRepository.updateLessonValue(businessId, valueConsume, valueGain);
        return new PersistModel(line);
    }

    @Override
    public List<UserValueDto> queryUserValueByPagination(PageUtil pageUtil, UserValueResDto userValueResDto) {
        List<String> deptList = new ArrayList<>();
        if(userValueResDto.getDept()!=null && !"".equals(userValueResDto.getDept())){
            Set<String> deptSets = new HashSet<>();
            deptSets.add(userValueResDto.getDept());
            deptList= mtCotactApiRequestClient.getSubUserIdsByDeptIds(userValueResDto.getCorpId(), deptSets);
        }

        List<UserValueDto> users = bReleaseValueRepository.getUserValue(userValueResDto, deptList);

        List<String> list = new ArrayList<>();
        for (UserValueDto user : users) {
            list.add(user.getBusinessId());
        }
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(userValueResDto.getCorpId());
        mtUserGetsReqDTO.setIds(list);
        List<MTUserInfoRspDTO> depart = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (depart != null) {
            StringBuffer sb = new StringBuffer();
            for (UserValueDto dto : users) {
                for (MTUserInfoRspDTO mtUserInfoRspDTO : depart) {
                    if (dto.getBusinessId().equals(mtUserInfoRspDTO.getId())) {
                        if (mtUserInfoRspDTO.getDepts() != null) {
                            for (MTUserInfoRspDTO.DeptsBean deptsBean : mtUserInfoRspDTO.getDepts()) {
                                for (MTUserInfoRspDTO.DeptsBean.PathBean pathBean : deptsBean.getPath()) {
                                    if (null != pathBean)
                                        sb.append(pathBean.getName() + "-");
                                }
                            }
                        }
                        if (!sb.toString().equals("")) {
                            String string = sb.toString().substring(0, sb.toString().length() - 1);
                            if(string.indexOf("-") != -1){
                                string = string.substring(string.indexOf("-")+1);
                            }
                            dto.setDept(string);
                        }
                        sb = new StringBuffer();
                    }
                }
            }
        }
        return users;
    }

    /**
     * 去掉子表中已有用户信息，插入新的数据
     * @param userIds
     * @param businessId
     */
    public void splitUser(String userIds , String businessId){

        String [] ids = userIds.split(",");
        // 删除子表中与当前选中用户相关的信息
        bReleaseValueRepository.deleteSonForUserId(ids);
        //将关联信息插入子表
        List<BReleaseValueSon> list = new ArrayList<>();
        if(ids.length > 0){
            for (String id : ids){
                BReleaseValueSon bReleaseValueSon = new BReleaseValueSon();
                bReleaseValueSon.setBusinessId(IdGen.uuid());
                bReleaseValueSon.setReleaseId(businessId);
                bReleaseValueSon.setUserId(id);
                list.add(bReleaseValueSon);
            }
            bReleaseValueRepository.insertIntoSon(list);
        }
    }
}
