package org.rcisoft.config;


import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.blesson.dao.BLessonPersonRepository;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;

import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

@Configuration
@ConditionalOnProperty(
        prefix = "eduJob",
        name = {"trainBegin"},
        havingValue = "true"
)
@Component
@Slf4j
public class TrainSignConfig {

    @Autowired
    BLessonPersonRepository bLessonPersonRepository;

    @Autowired
    BLessonRepository bLessonRepository;

    @Value("${eduJob.sec}")
    private String sec;

    @Scheduled(cron = "${eduJob.taskInterval}")
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public void trainBegin() {
        log.debug("--------------正在将未签到的人员置为缺勤------------");
        Date newDate = new Date();
        List<String> lessonIds = bLessonRepository.selectByTrainNowBeforeOver(newDate,sec);
        if (lessonIds != null && lessonIds.size() > 0){
            bLessonPersonRepository.updateByLessonIds(lessonIds);
        }
    }
}
