package org.rcisoft.core.aop;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.reflect.MethodSignature;
import org.rcisoft.business.bdictionary.entity.BDictionary;
import org.rcisoft.common.model.MenuMap;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultExceptionEnum;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.validate.ValidatedResult;
import org.rcisoft.sys.role.dto.InitAddAdminDTO;
import org.rcisoft.sys.role.dto.RoleMenuDTO;
import org.rcisoft.sys.role.dto.UserRoleDTO;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by lcy on 17/11/23.
 */
@Slf4j
@Aspect
public class CurUserAspect {
    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private SysUserService userServiceImpl;
    /**
     * 获取当前登陆人
     * @param joinPoint
     */
    @Before("execution(* org.rcisoft..*.controller.*.*(..))")
    public void beforeInsert(JoinPoint joinPoint) throws ParseException {
        if (joinPoint.getArgs().length < 2)
            return;
        Object model = joinPoint.getArgs()[0];
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        if(!(model instanceof CurUser))
            return;
        //非法权限
        if(null == model)
            throw new ServiceException(ResultExceptionEnum.USER_NO_AUTHEN);
        //免登 不拦截
        if(methodSignature.getName().equals("loginByCode") || methodSignature.getName().equals("refreshToken") || methodSignature.getName().equals("synchronizedUsers"))
            return;
        CurUser curUser = (CurUser) model;
        //非法权限
        log.debug("uid:\t" + curUser.getUserId() + "\t corpId:\t" + curUser.getCorpId() + "\t operate"
            + methodSignature.getName());
        if(StringUtils.isEmpty(curUser.getUserId()) || StringUtils.isEmpty(curUser.getCorpId()))
            throw new ServiceException(ResultExceptionEnum.USER_NO_AUTHEN);
        UserUtil.setCurUser(curUser);

        //多线程判断是否有管理员
        log.info("-------------------OUT-countAdminOut-up------------------");
        int countAdminOut = sysRoleService.queryAdminInCorp(curUser.getCorpId());
        log.info("-------------------OUT-countAdminOut-down------------------");
        if (countAdminOut < 1){
            synchronized (this) {
                //查询本部门是否有管理员
                log.info("-------------------In-countAdminOut-up------------------");
                int countAdmin = sysRoleService.queryAdminInCorp(curUser.getCorpId());
                log.info("-------------------In-countAdminOut-down------------------");
                if (countAdmin == 0) {
                    //随机生成角色Id

                    log.info("--------------------该企业正在初始化--------------------");

                    // 同步企业人员
                    int line = userServiceImpl.synchronizedUsers(curUser.getCorpId());
                    //初始化角色
                    int userRoleResult = sysRoleService.addAdminRoleMenuUser(curUser);
                    // 初始化数据字典表
                    sysRoleService.initDictionary();

                    log.info("--------------------初始化完成--------------------");
                }
            }
        }

    }

}
