package org.rcisoft.sys.role.service.impl;


import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.bdictionary.entity.BDictionary;
import org.rcisoft.business.blesson.dto.RoleTypeDepartDTO;
import org.rcisoft.common.model.MenuMap;
import org.rcisoft.common.util.feignDto.CorpGetManager;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.dept.entity.DeptRole;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.role.dto.*;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.role.service.SysRoleService;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by lcy on 16/7/30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysRoleServiceImpl implements SysRoleService {

    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    MTCotactApiRequestClient cotactApiRequestClient;

    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int persistRole(AddOrUpdateRoleDTO addOrUpdateRoleDTO) {
        SysRole sysRole = new SysRole();
        BeanUtils.copyProperties(addOrUpdateRoleDTO,sysRole);
        int line;
        if(sysRole.getBusinessId()!=null){//修改
            List<SysRole> roles = sysRoleRepository.queryRolesByName(sysRole);
            if(roles.size()>0){
                throw new ServiceException(ResultServiceEnums.ROLE_EXISTS);
            }
            UserUtil.setCurrentMergeOperation(sysRole);
            line = sysRoleRepository.updateByPrimaryKeySelective(sysRole);
        }else{//增加
            List<SysRole> roles = sysRoleRepository.queryRolesByCodeOrName(sysRole);
            if(roles.size()>0){
                throw new ServiceException(ResultServiceEnums.ROLE_EXISTS);
            }
            UserUtil.setCurrentPersistOperation(sysRole);
            line = sysRoleRepository.insertSelective(sysRole);
        }
        return line;
    }

    @Override
    public PersistModel updateRoleData(SysRole sysRole) {
        int line = sysRoleRepository.updateRoleData(sysRole);
        return new PersistModel(line);
    }


    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int removeRole(String id) {
        //获得当前是此角色的人数
        int userCount = sysUserMapper.countUserByRole(id);
        //如果仍然有用户（如果用户被逻辑删除，则忽略此角色，但如果用于被停用，则依然被计算）是此角色，则该角色不可删除
        if(userCount > 0){
            throw new ServiceException(ResultServiceEnums.ROLE_HAS_USED);
        }else{
            //删除该角色所有的关联信息
            sysRoleRepository.deleteRoleMenuByRoleId(id);
            sysRoleRepository.deleteUserRoleByRoleId(id);
            int line = sysRoleRepository.deleteByPrimaryKey(id);
            return line;
        }
    }

    @Override
    public List<SysRole> queryRolesByPagination(PageUtil pageUtil, String corpId) {
//        SysRole sysRole = new SysRole();
//        BeanUtils.copyProperties(findRolePaginDTO,sysRole);
//        sysRole.setNotDeleted();
//        sysRole.setStart();
        return sysRoleRepository.queryRoles(corpId);
    }

    @Override
    public List<SysRole> queryRolesAll(String corpId) {
        return sysRoleRepository.queryAllRoles(corpId);
    }

    @Override
    public List<DeptRole> queryUserRolesByPagination(PageUtil<SysRole> paginationUtility, String userId, SysRole sysRole) {
        return sysRoleRepository.queryUserRolesByPagination(userId, sysRole);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel doAuthorization(String roleId, String role_menus) {
        //删除原始权限
        int line = sysRoleRepository.deleteRoleMenus(roleId);
        //批量插入新的权限
        if(null != role_menus && !"".equals(role_menus.trim())){
            String[] menus = role_menus.split(",");
            List<Map> menuRole = new ArrayList<Map>();
            Map map = new HashMap();
            for (int i = 0 ; i < menus.length ; i++){
                map = new HashMap();
                map.put("roleId",roleId);
                map.put("menuId",menus[i]);
                map.put("id", IdGen.uuid());
                menuRole.add(map);
            }
            line= sysRoleRepository.insertBatchRoleMenu(menuRole);
        }
        return new PersistModel(line);
    }
    
    @Override
    public List<SysRole> queryDeptRolesByPagination(PageUtil<SysRole> paginationUtility, String deptId, SysRole sysRole) {
        return sysRoleRepository.queryDeptRolesByPagination(deptId, sysRole);
    }
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel doAuthorizationDept(String roleId, String role_depts) {
        //删除原始权限
        int line = sysRoleRepository.deleteRoleDepts(roleId);
        //批量插入新的权限
        if(null != role_depts && !"".equals(role_depts.trim())){
            String[] depts = role_depts.split(",");
            List<Map> deptRole = new ArrayList<Map>();
            Map map = null;
            for (int i = 0 ; i < depts.length ; i++){
                map = new HashMap();
                map.put("roleId",roleId);
                map.put("deptId",depts[i]);
                map.put("id", IdGen.uuid());
                deptRole.add(map);
            }
            line = sysRoleRepository.insertBatchRoleDept(deptRole);
        }
        return new PersistModel(line);
    }

	@Override
	public SysRole queryRoleById(String id) {
	       return sysRoleRepository.selectByPrimaryKey(id);
	}

	@Override
	public List<SysRole> queryRolesSelect(SysRole sysRole) {
		return sysRoleRepository.queryRolesSelect(sysRole);
	}

    @Override
    public int queryAdminInCorp(String corpId) {
        return sysRoleRepository.queryAdminInCorp(corpId);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    public int addAdminRoleMenuUser(CurUser curUser) {

        String privilege = "M-LESSON-MANAGE,F-ADD-LESSON,F-UPDATE-LESSON,F-CLOSE-LESSON,F-DELETE-LESSON,M-TRAIN-MANAGE,F-ADD-TRAIN,F-UPDATE-TRAIN,F-CLOSE-TRAIN,F-DELETE-TRAIN,M-PRESERVE,F-DOWNLOAD-PRESERVE,F-REPRINT-PRESERVE,F-DELETE-PRESERVE,M-UPLOAD,F-DOWNLOAD-UPLOAD,F-REPRINT-UPLOAD,F-DELETE-UPLOAD,M-CTS-B,F-ONLINE,P-ONLINE-ADD,P-ONLINE-UPDATE,P-ONLINE-APPOINT,P-ONLINE-TRACK,P-ONLINE-RECOMMEND,P-ONLINE-CLOSE,P-ONLINE-DEL,F-UNDERLINE,P-UNDER-ADD,P-UNDER-UPDATE,P-UNDER-APPOINT,P-UNDER-TRACK,P-UNDER-RECOMMEND,P-UNDER-CLOSE,P-UNDER-DEL,F-CATEGORY,P-CATEGORY-ADD,P-CATEGORY-UPDATE,P-CATEGORY-DEL,F-LABEL,P-LABEL-ADD,P-LABEL-UPDATE,P-LABEL-DEL,F-INTEGRAL-M,P-NULL-2,M-QDB,F-QDB,P-QDB-ADD,P-QDB-UPDATE,P-QDB-DELETE,P-QDB-ANALYSIS,F-QES,P-QES-ADD,P-QES-UPDATE,P-QES-DELETE,P-QES-IMPORT,M-PAPER,F-PAPER-CATE,P-PAPER-CATE-ADD,P-PAPER-CATE-UPDATE,P-PAPER-CATE-DELETE,F-PAPER,P-PAPER-ADD,P-PAPER-UPDATE,P-PAPER-DELETE,P-PAPER-CONFIG,P-PAPER-HISTORY-VIEW,P-PAPER-HISTORY-CHECK,M-ANALYSIS,P-ANA-PAPER,P-ANA-EXAM,P-ANA-SCORE,M-SYSTEM,F-MAIN-MANAGE,P-COLUMN,P-BANNER,P-OUTER-LINK,F-USER-MANAGE,P-STOP-START,P-ROLE-EDIT,P-SYNCHRO,F-ROLE-MANAGE,P-ROLE-MANAGE,P-OPERATION,P-DATALIMIT";
        //主管理员角色id
        String mRId = IdGen.uuid();
        //初始化角色（新加公司超级管理员）
        InitAddAdminDTO initAddAdminDTO = new InitAddAdminDTO();
        initAddAdminDTO.setRId(mRId);
        initAddAdminDTO.setRName("智信主管理员(默认)");
        initAddAdminDTO.setRStatus("1");
        initAddAdminDTO.setRCreatedate(new Date());
        initAddAdminDTO.setRPrivilege(privilege);
        initAddAdminDTO.setType("1");
        initAddAdminDTO.setCorpId(curUser.getCorpId());
        initAddAdminDTO.setRType("0");
        //设置s_role_menu表插入参数值
        List<String> menuIds = new ArrayList<String>(MenuMap.menuMap.values());
        List<RoleMenuDTO> roleMenuDTOs = setRoleMenu(menuIds,mRId);
        //将主管理员插入tm_admin_role、s_role_menu表
        int line;
        sysRoleRepository.addAdminRole(initAddAdminDTO);
        line = sysRoleRepository.addRoleMenu(roleMenuDTOs);
        //设置s_r_user_role表插入参数值
        //获取所有主管理员
        List<CorpGetManager> userMainList = cotactApiRequestClient.corpGetManager(curUser.getCorpId(),"1");
        if (userMainList != null && userMainList.size() > 0){
            userMainList = this.accountIdToBusinessId(userMainList,curUser.getCorpId());
            List<UserRoleDTO> userRoleDTOList = setUserRole(userMainList,mRId);
            //插入s_r_user_role
            sysRoleRepository.addUserRole(userRoleDTOList);
        }

        //子管理员角色id
        String sRId = IdGen.uuid();
        //初始化角色（新加公司管理员）
        InitAddAdminDTO initAddSubAdminDTO = new InitAddAdminDTO();
        initAddSubAdminDTO.setRId(sRId);
        initAddSubAdminDTO.setRName("智信子管理员(默认)");
        initAddSubAdminDTO.setRStatus("1");
        initAddSubAdminDTO.setRCreatedate(new Date());
        initAddSubAdminDTO.setRPrivilege(privilege);
        initAddSubAdminDTO.setType("1");
        initAddSubAdminDTO.setCorpId(curUser.getCorpId());
        initAddSubAdminDTO.setRType("1");
        //设置s_role_menu表插入参数值
//        List<String> menuIds = new ArrayList<String>(MenuMap.menuMap.values());
        List<RoleMenuDTO> subRoleMenuDTOs = setRoleMenu(menuIds,sRId);

        //添加默认 讲师、学生 权限
        String teacher = IdGen.uuid();
        InitAddAdminDTO initTeacherDTO = new InitAddAdminDTO();
        initTeacherDTO.setRId(teacher);
        initTeacherDTO.setRName("讲师(默认)");
        initTeacherDTO.setRStatus("1");
        initTeacherDTO.setRCreatedate(new Date());
        initTeacherDTO.setRPrivilege("M-LESSON-MANAGE,F-ADD-LESSON,F-UPDATE-LESSON,F-CLOSE-LESSON,F-DELETE-LESSON,M-TRAIN-MANAGE,F-ADD-TRAIN,F-UPDATE-TRAIN,F-CLOSE-TRAIN,F-DELETE-TRAIN,M-UPLOAD,F-DOWNLOAD-UPLOAD,F-REPRINT-UPLOAD,F-DELETE-UPLOAD,M-CTS-B,F-ONLINE,P-ONLINE-ADD,P-ONLINE-UPDATE,P-ONLINE-APPOINT,P-ONLINE-TRACK,P-ONLINE-RECOMMEND,P-ONLINE-CLOSE,P-ONLINE-DEL,F-UNDERLINE,P-UNDER-ADD,P-UNDER-UPDATE,P-UNDER-APPOINT,P-UNDER-TRACK,P-UNDER-RECOMMEND,P-UNDER-CLOSE,P-UNDER-DEL");
        initTeacherDTO.setType("0");
        initTeacherDTO.setCorpId(curUser.getCorpId());
        initTeacherDTO.setRType("2");
        List<String> teacherMenu = new ArrayList<String>();
        teacherMenu.add("0001");
        teacherMenu.add("0011");
        teacherMenu.add("0013");
        List<RoleMenuDTO> teacherMenuDTOs = setRoleMenu(teacherMenu,teacher);

        sysRoleRepository.addAdminRole(initTeacherDTO);
        sysRoleRepository.addRoleMenu(teacherMenuDTOs);

        String student = IdGen.uuid();
        InitAddAdminDTO initStudentDTO = new InitAddAdminDTO();
        initStudentDTO.setRId(student);
        initStudentDTO.setRName("学员(默认)");
        initStudentDTO.setRStatus("1");
        initStudentDTO.setRCreatedate(new Date());
        initStudentDTO.setRPrivilege("M-PRESERVE,F-DOWNLOAD-PRESERVE,F-REPRINT-PRESERVE,F-DELETE-PRESERVE");
        initStudentDTO.setType("0");
        initStudentDTO.setCorpId(curUser.getCorpId());
        initStudentDTO.setRType("2");
        //学院默认没有后台权限，不用加菜单
        sysRoleRepository.addAdminRole(initStudentDTO);

        //将子管理员 插入tm_admin_role、s_role_menu、
        sysRoleRepository.addAdminRole(initAddSubAdminDTO);
        sysRoleRepository.addRoleMenu(subRoleMenuDTOs);
        //获取所有子管理员
        //设置s_r_user_role表插入参数值
        List<CorpGetManager> userSubMainList = cotactApiRequestClient.corpGetManager(curUser.getCorpId(),"2");
        if (userSubMainList != null && userSubMainList.size() > 0){
            userSubMainList = this.accountIdToBusinessId(userSubMainList,curUser.getCorpId());
            List<UserRoleDTO> userSubRoleDTOList = setUserRole(userSubMainList,sRId);
            //插入s_r_user_role表
            sysRoleRepository.addUserRole(userSubRoleDTOList);
        }

        return line;
    }

    @Override
    public List<RoleTypeDepartDTO> queryRoleTypeByUserId(CurUser curUser) {
        return sysUserMapper.queryRoleTypeByUserId(curUser.getUserId());
    }

    @Override
    public int initDictionary() {
        List<BDictionary> dictionaryList = new ArrayList<>();
        BDictionary bDictionary1 = new BDictionary();
        bDictionary1.setName("关注人数");
        bDictionary1.setType("GZRS");
        bDictionary1.setValue("5");
        bDictionary1.setSort(1);
        UserUtil.setCurrentPersistOperation(bDictionary1);
        BDictionary bDictionary2 = new BDictionary();
        bDictionary2.setName("在学人数");
        bDictionary2.setType("ZXRS");
        bDictionary2.setValue("7");
        bDictionary2.setSort(2);
        UserUtil.setCurrentPersistOperation(bDictionary2);
//        BDictionary bDictionary3 = new BDictionary();
//        bDictionary3.setName("总裁读书会");
//        bDictionary3.setType("DSH");
//        bDictionary3.setValue("http://www.winnerbook.com.cn/mobile/index.php?m=site");
//        bDictionary3.setSort(3);
//        UserUtil.setCurrentPersistOperation(bDictionary3);

        dictionaryList.add(bDictionary1);
        dictionaryList.add(bDictionary2);
//        dictionaryList.add(bDictionary3);

        return sysUserMapper.initDictionary(dictionaryList);
    }


    List<RoleMenuDTO> setRoleMenu(List<String> menuIds,String rId){
        List<RoleMenuDTO> roleMenuDTOs = new ArrayList<>();
        menuIds.forEach(menuId->{
            RoleMenuDTO roleMenuDTO = new RoleMenuDTO();
            roleMenuDTO.setBusinessId(IdGen.uuid());
            roleMenuDTO.setRoleId(rId);
            roleMenuDTO.setMenuId(menuId);
            roleMenuDTOs.add(roleMenuDTO);
        });
        return roleMenuDTOs;
    }

    List<UserRoleDTO> setUserRole(List<CorpGetManager> userMainList ,String rId){
        List<UserRoleDTO> userRoleDTOList = new ArrayList<>();
        userMainList.forEach(user->{
            UserRoleDTO userRoleDTO = new UserRoleDTO();
            userRoleDTO.setBusinessId(IdGen.uuid());
            userRoleDTO.setRoleId(rId);
            userRoleDTO.setUserId(user.getId());
            userRoleDTOList.add(userRoleDTO);
        });

        return userRoleDTOList;
    }

    /**
     * 根据AccountId查询用户user表中真实BusinessId
     * @param MainList
     * @return
     */
    List<CorpGetManager> accountIdToBusinessId(List<CorpGetManager> MainList,String corpId){
        StringBuffer sb = new StringBuffer();
        for(CorpGetManager user : MainList){
            sb.append(user.getAccountId()+",");
        }
        String accountId = sb.toString().substring(0,sb.toString().length() - 1);
        List<SysUser> businessId = sysUserMapper.getBusinessIdByAccountId(accountId,corpId);

        if(businessId != null && businessId.size() > 0){
            for(SysUser sysUser:businessId){
                for(CorpGetManager c : MainList){
                    if(sysUser.getAccountId().equals(c.getAccountId())){
                        c.setId(sysUser.getBusinessId());
                    }
                }
            }
        }
        return MainList;
    }

}
