package org.rcisoft.sys.user.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.synchronizationtime.entity.SynchronizationTime;
import org.rcisoft.business.synchronizationtime.service.SynchronizationTimeService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.common.util.feignDto.GetAllRspDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.menu.entity.SysMenu;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.dto.QueryMenuResDTO;
import org.rcisoft.sys.user.dto.QuerySysUserDTO;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.expression.spel.ast.Assign;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * Created by Andrew on 2017/7/6.
 */
@Api(tags = "07 用户管理")
@RestController
@RequestMapping(value = "/user")
public class SysUserController extends PaginationController<SysUser> {

    @Autowired
    private SysUserService userServiceImpl;
    @Autowired
    private SynchronizationTimeService synchronizationTimeServiceImpl;

    /**
     * 同步人员
     * @param
     * @return
     */
    @Transactional
    @GetMapping("/gets")
    @ApiOperation(value = "|personnels|同步人员", notes = "同步人员信息")
    public Result synchronizedUsers(@RequestParam String corpId) throws ParseException {
        int line = userServiceImpl.synchronizedUsers(corpId);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "同步"+line+"条人员信息");

    }




    @Autowired
    private Global global;

    @ApiOperation(value = "701 分页查询用户", notes = "分页查询用户")
    @ApiImplicitParams({@ApiImplicitParam(name = "name", value = "用户名（模糊查询条件）", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "flag", value = "用户启用状态  0未启用  1启用", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "roleName", value = "用户角色", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryUsers")
    public GridModel queryUsers(CurUser curUser, @Valid QuerySysUserDTO dto, BindingResult br) {
        dto.setCorpId(curUser.getCorpId());
        userServiceImpl.queryUsersByPagination(getPaginationUtility(), dto);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value = "702 分配角色", notes = "分配角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "用户id（批量操作时 id用，隔开）", required = true, dataType = "varchar", paramType = "path"),
            @ApiImplicitParam(name = "roleId", value = "角色id（批量操作时 id用，隔开）", required = true, dataType = "varchar", paramType = "path")})
    @PostMapping(value = "/AssignRoles")
    public Result AssignRoles(CurUser curUser, @Valid String id, BindingResult br,String roleId) {
        int line = userServiceImpl.AssignRoles(id,roleId);
        Map<String,String> map = new HashMap<>();
        map.put("roleIds",roleId);
        map.put("userIds",id);
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }

//    @ApiOperation(value = "703 删除用户", notes = "删除用户")
//    @ApiImplicitParam(name = "businessId", value = "用户id", required = true, dataType = "varchar", paramType = "path")
//    @PostMapping(value = "/delete")
//    public Result roleDelete(CurUser curUser, @Valid String businessId, BindingResult br) {
//        int line = userServiceImpl.removeUser(businessId);
//        return Result.builder(new PersistModel(line),
//                MessageConstant.MESSAGE_ALERT_SUCCESS,
//                MessageConstant.MESSAGE_ALERT_ERROR,
//                businessId);
//    }

    @ApiOperation(value = "704 停用用户", notes = "停用用户")
    @ApiImplicitParam(name = "businessId", value = "用户Id（批量操作时 id用，隔开）", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "/stop")
    public Result stop(CurUser curUser, @Valid String businessId, BindingResult br) {
        int line = userServiceImpl.stopUserByUserBusinessId(businessId);
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value = "705 启用用户", notes = "启用用户")
    @ApiImplicitParam(name = "businessId", value = "用户Id（批量操作时 id用，隔开）", required = true, dataType = "varchar", paramType = "path")
    @PostMapping(value = "/start")
    public Result Start(CurUser curUser, @Valid String businessId, BindingResult br) {
        int line = userServiceImpl.startUserByUserBusinessId(businessId);
        return Result.builder(new PersistModel(line),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value = "706 根据userId 查询菜单", notes = "userId 和 modelId查询 menu")
    @ApiImplicitParams({@ApiImplicitParam(name = "modelId", value = "前台菜单10  后台菜单20", required = true, dataType = "varchar", paramType = "path"),
            @ApiImplicitParam(name = "type", value = "查总裁读书会菜单  客户端传2 手机端传0  不传查全部（modelId传 10） ", required = true, dataType = "varchar", paramType = "path")})
    @GetMapping("/queryMenuById")
    public Result queryMenu(CurUser curUser,String modelId,String type) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.queryMenusById(curUser,modelId,type));
    }

    /**
     * 查询菜单
     * @param user
     * @return
     */
    @GetMapping(value = "/queryMyMenu")
    public Result queryMyMenu(CurUser user) {
        List<SysMenu> menus = userServiceImpl.queryUserMenus(user);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                menus);
    }

    @ApiOperation(value = "707 查询部门", notes = "查询部门")
    @GetMapping("/queryDepart")
    public Result queryMenu(CurUser curUser) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.queryAllDepart());
    }

    @ApiOperation(value = "708 我的智学 个人信息",notes = "我的智学 个人信息及课程、培训、考试数量")
    @GetMapping("/queryMyInfo")
    public Result queryMyInfo(CurUser curUser) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.queryMyInfo(curUser));
    }

    @ApiOperation(value = "709 查询当前登录人名字",notes = "查询当前登录人名字")
    @GetMapping("/getName")
    public Result getNameByUserId(CurUser curUser) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.queryMyName(curUser));
    }
    @ApiOperation(value = "710 查询菜单、按钮权限",notes = "查询当前登录人菜单和按钮权限")
    @GetMapping("/getAllRole")
    public Result getAllRoleByUserId(CurUser curUser) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.queryAllRoleById(curUser));
    }

    @GetMapping("/test")
    public Result testUser(CurUser curUser,QuerySysUserDTO dto) {
        dto.setCorpId(curUser.getCorpId());
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.test(dto));
    }

    @ApiOperation(value = "711 根具Id查询名字",notes = "根具Id查询名字")
    @GetMapping("/getNameById")
    public Result getNameById(CurUser curUser,@Valid String id,BindingResult br) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                userServiceImpl.getNameById(id));
    }


}
