package org.rcisoft.sys.user.service.impl;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang.StringUtils;
import org.rcisoft.business.bexternallink.dto.MenuDTO;
import org.rcisoft.business.synchronizationtime.entity.SynchronizationTime;
import org.rcisoft.business.synchronizationtime.service.SynchronizationTimeService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.util.feignDto.GetAllRspDTO;
import org.rcisoft.common.util.feignDto.MTGetUserByDeptIdsRspDTO;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.menu.dao.SysMenuRepository;
import org.rcisoft.sys.user.dto.QueryMenuResDTO;
import org.rcisoft.sys.menu.entity.SysMenu;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.dto.QuerySysUserDTO;
import org.rcisoft.sys.user.dto.MyInfoDTO;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Array;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

import static java.util.Arrays.asList;

@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class SysUserServiceImpl implements SysUserService {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysMenuRepository sysMenuRepository;

    @Autowired
    private SynchronizationTimeService synchronizationTimeServiceImpl;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    @Autowired
    private Global global;

    @Override
    public List<SysUser> queryUsersByPagination(PageUtil pageUtil, QuerySysUserDTO dto) {
        List<SysUser> users = this.qusers(dto);
        return users;
    }

    /**
     * 添加用户对应的 部门和岗位
     * @param dto
     * @return
     */
    public List<SysUser> qusers(QuerySysUserDTO dto) {
        dto.setType("0");
        //step1:调用外部接口 查询出 该部门下所有人（取id）
        if("0".equals(dto.getType())){                      //包含子部门
            if(dto.getDept()!=null && !"".equals(dto.getDept()) ){
                List<String> deptIds = new ArrayList<>();
                deptIds.add(dto.getDept());
                Set<String> deptSets = new HashSet<>(deptIds);
                List<String> data = mtCotactApiRequestClient.getSubUserIdsByDeptIds(dto.getCorpId(), deptSets);
                if(data.size() != 0){
                    dto.setUserIds(data);
                }else{
                    dto.setUserIds(Arrays.asList(""));
                }
            }
        }else{                                              //不包含子部门
            if(dto.getDept()!=null && !"".equals(dto.getDept())){
                List<Long> deptIds = new ArrayList<>();
                deptIds.add(Long.parseLong(dto.getDept()));
                List<MTGetUserByDeptIdsRspDTO> data = mtCotactApiRequestClient.userGetUserByDeptIds(dto.getCorpId(),deptIds);
                List<String> uids = new ArrayList<>();
                if(data != null ){
                    for (MTGetUserByDeptIdsRspDTO uid : data) {
                        uids.add(uid.getId());
                    }
                    if(uids.size() != 0){
                        dto.setUserIds(uids);
                    }else{
                        dto.setUserIds(Arrays.asList(""));
                    }
                }else{
                    dto.setUserIds(Arrays.asList(""));
                }
            }
        }

        //step2：先查出符合条件的所有人
        List<SysUser> users = sysUserMapper.queryUsers(dto);

        //step3:将符合条件的用户 加上部门和岗位字段
        List<String> list = new ArrayList<>();
            for (SysUser user : users) {
            list.add(user.getBusinessId());
        }
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(dto.getCorpId());
        mtUserGetsReqDTO.setIds(list);
        List<MTUserInfoRspDTO> depart = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);

        if (depart != null) {
            StringBuffer sb = new StringBuffer();
            String position = "";
            for (SysUser sysUser : users) {
                for (MTUserInfoRspDTO mtUserInfoRspDTO : depart) {
                    if (sysUser.getBusinessId().equals(mtUserInfoRspDTO.getId())) {
                        if (mtUserInfoRspDTO.getDepts() != null) {
                            for (MTUserInfoRspDTO.DeptsBean deptsBean : mtUserInfoRspDTO.getDepts()) {
                                for (MTUserInfoRspDTO.DeptsBean.PathBean pathBean : deptsBean.getPath()) {
                                    if (null != pathBean)
                                        sb.append(pathBean.getName() + "-");
                                }
                                position = deptsBean.getPosition() == null ? "" : String.valueOf(deptsBean.getPosition());
                            }
                        }
                        if (!sb.toString().equals("")) {
                            String string = sb.toString().substring(0, sb.toString().length() - 1);
                            if(string.indexOf("-") != -1){
                                string = string.substring(string.indexOf("-")+1);
                            }
                            sysUser.setDepart(string);
                        }
                        sb = new StringBuffer();
                        sysUser.setPosition(position);
                    }
                }
            }
        }
        return users;
    }

    @Override
    public int removeUser(String id) {
        SysUser user = new SysUser();
        user.setDeleted();
        user.setStop();
        user.setBusinessId(id);
        return sysUserMapper.deleteByPrimaryKey(user);
    }

    @Override
    public int AssignRoles(String userId, String roleId) {

        //主管理员和子管理员不能被分配角色（角色不可改变）
        String[] adminAllType = new String[]{"0","1"};
        //查询管理员角色的userid
        List<String> adminAllIds = sysUserMapper.adminId(adminAllType);
        //将超级管理员人员userid 从String[]中剔除
        if (adminAllIds.size() != 0) {
            for (int i = 0; i < adminAllIds.size(); i++) {
                userId = userId.replace(adminAllIds.get(i), "");
            }
        }
        if("".equals(roleId)){
            //如果没有勾选角色 则清空该用户所有角色
            List<String> uid = Arrays.asList(userId.split(","));
            return sysUserMapper.delUserRoleById(uid);
        }
        List<String> userIds = asList(StringUtils.split(userId, ","));
        List<String> roleIds = asList(StringUtils.split(roleId, ","));
        List<UserRole> userRoles = new ArrayList<>();
        //清空user_role中关于ids的所有数据
        if(userIds.size() == 0){
            throw  new ServiceException(ResultServiceEnums.NOT_USE_ADMIN_ROLE);
        }
        sysUserMapper.delUserRoleById(userIds);
        for (int i = 0; i < roleIds.size(); i++) {
            for (int j = 0; j < userIds.size(); j++) {
                UserRole userRole = new UserRole();
                userRole.setUserId(userIds.get(j));
                userRole.setRoleId(roleIds.get(i));
                UserUtil.setCurrentPersistOperation(userRole);
                userRoles.add(userRole);
            }
        }
        return sysUserMapper.insertUserRoleById(userRoles);
    }

    @Override
    public void updateUser(SysUser user, String uid, int line) {
        if (!user.isEmpty()) {
            user.setBusinessId(uid);
            int l1 = sysUserMapper.updateByPrimaryKeySelective(user);
            if (l1 != line)
                throw new ServiceException(ResultServiceEnums.UPDATE_ERROR);
        }
    }

    @Override
    public String checkPassword(String userId) {
        return sysUserMapper.queryPasswordById(userId);
    }


    @Override
    public int stopOrStartUserByUserBusinessId(String businessId) {
        SysUser sysUser = sysUserMapper.selectByPrimaryKey(businessId);
        String status = sysUser.getFlag().equals("0") ? "1" : "0";
        sysUser.setFlag(status);
        return sysUserMapper.stopOrStartUserByUserBusinessId(sysUser);
    }

    @Override
    public int stopUserByUserBusinessId(String businessId) {

        if(businessId == null || "".equals(businessId)){
            throw new ServiceException(ResultServiceEnums.USER_IS_NULL);
        }
        //主管理员不能被停用角色
        String[] adminType = new String[]{"0"};
        //查询主管理员角色的userid
        List<String> adminIds = sysUserMapper.adminId(adminType);
        //剔除主管理员的id
        if (adminIds.size() != 0) {
            for (int i = 0; i < adminIds.size(); i++) {
                businessId = businessId.replace(adminIds.get(i), "");
            }
        }
        if(businessId == null || "".equals(businessId)){
            throw new ServiceException(ResultServiceEnums.MANAGE_NOT_STOP);
        }
        List<String> ids = asList(StringUtils.split(businessId, ","));
        return sysUserMapper.stopFlag(ids);
    }

    @Override
    public int startUserByUserBusinessId(String businessId) {
        if(businessId == null && "".equals(businessId)){
            throw new ServiceException(ResultServiceEnums.USER_IS_NULL);
        }
        List<String> ids = asList(StringUtils.split(businessId, ","));
        return sysUserMapper.startFlag(ids);
    }

    @Override
    public List<SysMenu> queryUserMenus(CurUser user) {
        SysMenu sysMenu = new SysMenu();
        sysMenu.setNormal();
        sysMenu.setBusinessId(user.getUserId());
        return sysMenuRepository.queryMyMenu(sysMenu);
    }

    @Override
    public Object queryMenusById(CurUser curUser, String modelId,String type) {
        //查询登录人启用状态
        int x = sysUserMapper.getUserflag(curUser);
        if(x < 1){
            Map<String,Object>  map = new HashedMap();
            map.put("massage","用户未启用");
            map.put("code",400);
            return map;
        }
        if("10".equals(modelId)){
            List<MenuDTO> list = sysUserMapper.getReadingMenu(type,curUser.getCorpId());
            for(int i = 0;i < list.size();i++){
                list.get(i).setHref("a"+(i+1));
            }
            return list;
        }

        List<QueryMenuResDTO> menus = sysUserMapper.queryMenus(curUser.getCorpId(), curUser.getUserId(), modelId);
        String[] strings = new String[]{"题库管理", "试卷管理", "统计分析"};
        QueryMenuResDTO newMenu = new QueryMenuResDTO();
        newMenu.setMenuName("在线考试");
        newMenu.setHref("newTest");
        newMenu.setMenuId("000");

        int size = menus.size();
        for (String str : strings) {
            for (QueryMenuResDTO menu : menus) {
                if (menu.getMenuName().equals(str)) {
                    menus.add(newMenu);
                    break;
                }
            }
            if (size < menus.size()) {
                break;
            }
        }
        return menus;
    }

    public SysUser  getUserState(String id){
        return  sysUserMapper.queryUserState(id);
    }

    @Override
    public Object queryAllRoleById(CurUser curUser) {
        List<String> roles = sysUserMapper.queryAllRoleById(curUser);
        //查询登录人启用状态
        int x = sysUserMapper.getUserflag(curUser);
        if(x < 1){
            Map<String,Object>  map = new HashedMap();
            map.put("massage","用户未启用");
            map.put("code",400);
            return map;
        }

        StringBuffer sb = new StringBuffer();
        if(roles != null && roles.size() > 0){
            for(String role : roles){
                sb.append(role+",");
            }
            String str1 =  sb.toString().substring(0,sb.toString().length() - 1);
            String [] strs = str1.split(",");
            Set roleSet = new HashSet();
            for(String s : strs){
                roleSet.add(s);
            }
            return StringUtils.join(roleSet.toArray(), ",");

        }else {
            Map<String,Object> map = new HashedMap();
            map.put("massage","用户启用但无权限");
            map.put("code",500);
            return map;
        }
    }

    @Override
    public Map<String, Object> queryAllDepart() {
        String str = "{\n" +
                "\t\"courseList\": [\n" +
                "\n" +
                "\t\t{\n" +
                "\t\t\t\"title\": \"部门1\",\n" +
                "\t\t\t\"key\": \"8ae79c10-d3af-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\"children\": [{\n" +
                "\t\t\t\t\t\"title\": \"部门1-1\",\n" +
                "\t\t\t\t\t\"key\": \"d6e5a068-d3ce-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t\t\"children\": [{\n" +
                "\t\t\t\t\t\t\t\"title\": \"部门1-1-1\",\n" +
                "\t\t\t\t\t\t\t\"key\": \"ee471839-d3ce-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t\t\t\t\"children\": [],\n" +
                "\t\t\t\t\t\t\t\"pid\": \"d6e5a068-d3ce-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t\t\t},\n" +
                "\t\t\t\t\t\t{\n" +
                "\t\t\t\t\t\t\t\"title\": \"部门1-1-2\",\n" +
                "\t\t\t\t\t\t\t\"key\": \"ee4a7136-d3ce-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t\t\t\t\"children\": [],\n" +
                "\t\t\t\t\t\t\t\"pid\": \"d6e5a068-d3ce-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t\t\t}\n" +
                "\t\t\t\t\t],\n" +
                "\t\t\t\t\t\"pid\": \"8ae79c10-d3af-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t},\n" +
                "\t\t\t\t{\n" +
                "\t\t\t\t\t\"title\": \"部门1-2\",\n" +
                "\t\t\t\t\t\"key\": \"d6e910f4-d3ce-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t\t\"children\": [],\n" +
                "\t\t\t\t\t\"pid\": \"8ae79c10-d3af-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\t}\n" +
                "\t\t\t],\n" +
                "\t\t\t\"pid\": \"00000000\",\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"title\": \"部门2\",\n" +
                "\t\t\t\"key\": \"8aea4777-d3af-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\"children\": [],\n" +
                "\t\t\t\"pid\": \"00000000\",\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"title\": \"部门3\",\n" +
                "\t\t\t\"key\": \"8aed258e-d3af-11e9-8d54-b05adaca59d9\",\n" +
                "\t\t\t\"children\": [],\n" +
                "\t\t\t\"pid\": \"00000000\",\n" +
                "\t\t}\n" +
                "\t]\n" +
                "}";
        Map maps = (Map) JSON.parse(str);
        return maps;
    }

    @Override
    public int insertOrUpdateList(List<SysUser> sysUserList) {
        int line = 0;
        if (null != sysUserList && sysUserList.size() > 0) {
            for (SysUser sysUser : sysUserList) {
                SysUser user = sysUserMapper.selectOne(sysUser);
                if (null != user && StringUtils.isNotEmpty(sysUser.getBusinessId())) {
                    sysUser.setUpdateDate(new Date());
                    sysUserMapper.updateByPrimaryKeySelective(sysUser);
                    line++;
                } else {
                    sysUser.setCreateDate(new Date());
                    sysUser.setUpdateDate(new Date());
                    sysUserMapper.insertSelective(sysUser);
                    line++;
                }
            }
        }
        return line;
    }

    @Override
    public int synchronizedUsers(String corpId) throws ParseException {
        SynchronizationTime synchronizationTime = synchronizationTimeServiceImpl.getById("1");
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");  //调用智信接口根据更新时间查询所有人员
        if (null == synchronizationTime) {
            synchronizationTime = new SynchronizationTime();
            synchronizationTime.setBusinessId("1");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            synchronizationTime.setSynchronizationTime(sdf.parse("1999-01-01 00:00:00"));
            synchronizationTimeServiceImpl.save(synchronizationTime);
        }
        String updateTime = df.format(synchronizationTime.getSynchronizationTime());
        List<GetAllRspDTO> userGetRspDTOList = mtCotactApiRequestClient.accountGetMyInfoList(corpId, updateTime);
//        if (null == userGetRspDTOList) {
//            throw new ServiceException(ResultServiceEnums.ZX_FAILED);
//        }
        List<SysUser> personnelsList = new ArrayList<>();
        int line = 0;
        if (null != userGetRspDTOList && userGetRspDTOList.size() > 0) {
            log.info("同步人员......" + userGetRspDTOList.size() + "记录");
            for (GetAllRspDTO userInfo : userGetRspDTOList) {
                SysUser personnels = new SysUser();
                UserUtil.setCurrentPersistOperation(personnels);
                personnels.setCorpId(corpId);
                personnels.setBusinessId(userInfo.getId());   //id
                personnels.setName(userInfo.getName());  //姓名
                personnels.setHeadPic(userInfo.getAvatar());  //头像
                personnels.setPhone(userInfo.getMobile()); //  手机号
                personnels.setDelFlag(userInfo.getIsDel());  //在职离职状态
//                personnels.setAge(0);
//                personnels.setGender(0);
                if (null !=userInfo.getCreateAt()){
                    Instant instant = userInfo.getCreateAt().toInstant();
                    ZoneId zone = ZoneId.systemDefault();
//                    personnels.setCreateDate(LocalDateTime.ofInstant(instant, zone));
                }
                personnels.setAccountId(userInfo.getAccountId());
                personnelsList.add(personnels);
            }
            line = this.insertOrUpdateList(personnelsList);
        }
        if (line > 0) {
            // 同步成功后，更新同步时间
            synchronizationTime.setSynchronizationTime(new Date());
            synchronizationTimeServiceImpl.upadteById(synchronizationTime);
        }else {
            log.info("同步人员......0.......记录");
        }
        return line;
    }

    @Override
    public MyInfoDTO queryMyInfo(CurUser curUser) {
        MyInfoDTO myInfoDTOCount = sysUserMapper.queryMyLessonCount(curUser);
        MyInfoDTO myInfoDTO = sysUserMapper.queryMyInfo(curUser);
        //获取头像等信息
        List<String> ids = new ArrayList<>();
        ids.add(curUser.getUserId());
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        mtUserGetsReqDTO.setIds(ids);
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                if (mtUserInfoRspDTO.getId().equals(curUser.getUserId())) {
                    //设置部门名
                    myInfoDTO.setName(mtUserInfoRspDTO.getName());
                    myInfoDTO.setHeadPic(mtUserInfoRspDTO.getAvatar());
                }
            });
        }
        myInfoDTO.setLessonCount(myInfoDTOCount.getLessonCount());
        myInfoDTO.setTrainCount(myInfoDTOCount.getTrainCount());
        myInfoDTO.setNotFinishedExam(myInfoDTOCount.getNotFinishedExam());
        return myInfoDTO;
    }

    @Override
    public MyInfoDTO queryMyName(CurUser curUser) {
        return sysUserMapper.queryMyInfo(curUser);
    }

    @Override
    public List<MTUserInfoRspDTO> getName(CurUser curUser) {
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(curUser.getCorpId());
        List<String> list = new ArrayList<>();
        list.add(curUser.getUserId());
        mtUserGetsReqDTO.setIds(list);
        return mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
    }

    @Override
    public List<SysUser> test(QuerySysUserDTO dto) {
        return this.qusers(dto);
    }

    @Override
    public MyInfoDTO getNameById(String id) {
        return sysUserMapper.getNameById(id);
    }
}
